@@ -85,14 +85,15 @@ AbstractCommand::AbstractCommand
  const SocketHandle& s,
  const SharedHandle<SocketRecvBuffer>& socketRecvBuffer,
  bool incNumConnection)
-  : Command(cuid), checkPoint_(global::wallclock),
+  : Command(cuid), checkPoint_(global::wallclock()),
     timeout_(requestGroup->getTimeout()),
     requestGroup_(requestGroup),
     req_(req), fileEntry_(fileEntry), e_(e), socket_(s),
     socketRecvBuffer_(socketRecvBuffer),
     checkSocketIsReadable_(false), checkSocketIsWritable_(false),
     nameResolverCheck_(false),
-    incNumConnection_(incNumConnection)
+    incNumConnection_(incNumConnection),
+    serverStatTimer_(global::wallclock())
 {
   if(socket_ && socket_->isOpen()) {
     setReadCheckSocket(socket_);
@@ -117,6 +118,22 @@ AbstractCommand::~AbstractCommand() {
   }
 }
 
+void AbstractCommand::useFasterRequest
+(const SharedHandle<Request>& fasterRequest)
+{
+  A2_LOG_INFO(fmt("CUID#%lld - Use faster Request hostname=%s, port=%u",
+                  getCuid(),
+                  fasterRequest->getHost().c_str(),
+                  fasterRequest->getPort()));
+  // Cancel current Request object and use faster one.
+  fileEntry_->removeRequest(req_);
+  Command* command =
+    InitiateConnectionCommandFactory::createInitiateConnectionCommand
+    (getCuid(), fasterRequest, fileEntry_, requestGroup_, e_);
+  e_->setNoWait(true);
+  e_->addCommand(command);
+}
+
 bool AbstractCommand::execute() {
   A2_LOG_DEBUG(fmt("CUID#%lld - socket: read:%d, write:%d, hup:%d, err:%d",
                    getCuid(),
@@ -158,17 +175,26 @@ bool AbstractCommand::execute() {
          !getPieceStorage()->hasMissingUnusedPiece()) {
         SharedHandle<Request> fasterRequest = fileEntry_->findFasterRequest(req_);
         if(fasterRequest) {
-          A2_LOG_INFO(fmt("CUID#%lld - Use faster Request hostname=%s, port=%u",
-                          getCuid(),
-                          fasterRequest->getHost().c_str(),
-                          fasterRequest->getPort()));
-          // Cancel current Request object and use faster one.
-          fileEntry_->removeRequest(req_);
-          Command* command =
-            InitiateConnectionCommandFactory::createInitiateConnectionCommand
-            (getCuid(), fasterRequest, fileEntry_, requestGroup_, e_);
-          e_->setNoWait(true);
-          e_->addCommand(command);
+          useFasterRequest(fasterRequest);
+          return true;
+        }
+      }
+      // Don't use this feature if PREF_MAX_{OVERALL_}DOWNLOAD_LIMIT
+      // is used or total length is unknown.
+      if(req_ && fileEntry_->getLength() > 0 &&
+         e_->getRequestGroupMan()->getMaxOverallDownloadSpeedLimit() == 0 &&
+         requestGroup_->getMaxDownloadSpeedLimit() == 0 &&
+         serverStatTimer_.difference(global::wallclock()) >= 10) {
+        serverStatTimer_ = global::wallclock();
+        std::vector<std::pair<size_t, std::string> > usedHosts;
+        if(getOption()->getAsBool(PREF_SELECT_LEAST_USED_HOST)) {
+          getDownloadEngine()->getRequestGroupMan()->getUsedHosts(usedHosts);
+        }
+        SharedHandle<Request> fasterRequest =
+          fileEntry_->findFasterRequest
+          (req_, usedHosts, e_->getRequestGroupMan()->getServerStatMan());
+        if(fasterRequest) {
+          useFasterRequest(fasterRequest);
           return true;
         }
       }
@@ -183,7 +209,7 @@ bool AbstractCommand::execute() {
 #endif // ENABLE_ASYNC_DNS
        (!checkSocketIsReadable_ && !checkSocketIsWritable_ &&
         !nameResolverCheck_)) {
-      checkPoint_ = global::wallclock;
+      checkPoint_ = global::wallclock();
       if(getPieceStorage()) {
         if(!req_ || req_->getMaxPipelinedRequest() == 1 ||
            // Why the following condition is necessary? That's because
@@ -237,7 +263,7 @@ bool AbstractCommand::execute() {
       throw DL_RETRY_EX(fmt(MSG_NETWORK_PROBLEM,
                             socket_->getSocketError().c_str()));
     } else {
-      if(checkPoint_.difference(global::wallclock) >= timeout_) {
+      if(checkPoint_.difference(global::wallclock()) >= timeout_) {
         // timeout triggers ServerStat error state.
         SharedHandle<ServerStat> ss =
           e_->getRequestGroupMan()->getOrCreateServerStat(req_->getHost(),
@@ -313,7 +339,7 @@ bool AbstractCommand::execute() {
       tryReserved();
       return true;
     } else {
-      Timer wakeTime(global::wallclock);
+      Timer wakeTime(global::wallclock());
       wakeTime.advance(getOption()->getAsInt(PREF_RETRY_WAIT));
       req_->setWakeTime(wakeTime);
       return prepareForRetry(0);
