@@ -40,10 +40,6 @@
 #include <cstring>
 #include <cassert>
 
-#ifdef __MINGW32__
-# include <windows.h>
-#endif // __MINGW32__
-
 #include "File.h"
 #include "util.h"
 #include "message.h"
@@ -95,8 +91,8 @@ void AbstractDiskWriter::openExistingFile(uint64_t totalLength)
   } else {
     flags |= O_RDWR;
   }
-
-  while((fd_ = open(filename_.c_str(), flags, OPEN_MODE)) == -1 &&
+  while((fd_ = a2open(utf8ToWChar(filename_).c_str(),
+                      flags, OPEN_MODE)) == -1 &&
         errno == EINTR);
   if(fd_ < 0) {
     int errNum = errno;
@@ -114,8 +110,9 @@ void AbstractDiskWriter::createFile(int addFlags)
   assert(!filename_.empty());
   util::mkdirs(File(filename_).getDirname());
 
-  while((fd_ = open(filename_.c_str(), O_CREAT|O_RDWR|O_TRUNC|O_BINARY|addFlags,
-                    OPEN_MODE)) == -1 && errno == EINTR);
+  while((fd_ = a2open(utf8ToWChar(filename_).c_str(),
+                      O_CREAT|O_RDWR|O_TRUNC|O_BINARY|addFlags,
+                      OPEN_MODE)) == -1 && errno == EINTR);
   if(fd_ < 0) {
     int errNum = errno;
     throw DL_ABORT_EX3
