@@ -41,6 +41,7 @@
 #include "BtRuntime.h"
 #include "Peer.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "prefs.h"
 #include "Option.h"
 #include "BtConstants.h"
@@ -51,6 +52,7 @@
 #include "bittorrent_helper.h"
 #include "wallclock.h"
 #include "util.h"
+#include "fmt.h"
 
 namespace aria2 {
 
@@ -59,36 +61,35 @@ ActivePeerConnectionCommand::ActivePeerConnectionCommand
  RequestGroup* requestGroup,
  DownloadEngine* e,
  time_t interval)
-  :
-  Command(cuid),
-  _requestGroup(requestGroup),
-  interval(interval),
-  e(e),
-  _numNewConnection(5)
+  : Command(cuid),
+    requestGroup_(requestGroup),
+    interval_(interval),
+    e_(e),
+    numNewConnection_(5)
 {
-  _requestGroup->increaseNumCommand();
+  requestGroup_->increaseNumCommand();
 }
 
 ActivePeerConnectionCommand::~ActivePeerConnectionCommand()
 {
-  _requestGroup->decreaseNumCommand();
+  requestGroup_->decreaseNumCommand();
 }
 
 bool ActivePeerConnectionCommand::execute() {
-  if(_btRuntime->isHalt()) {
+  if(btRuntime_->isHalt()) {
     return true;
   }
-  if(checkPoint.difference(global::wallclock) >= interval) {
-    checkPoint = global::wallclock;
-    TransferStat tstat = _requestGroup->calculateStat();
+  if(checkPoint_.difference(global::wallclock()) >= interval_) {
+    checkPoint_ = global::wallclock();
+    TransferStat tstat = requestGroup_->calculateStat();
     const unsigned int maxDownloadLimit =
-      _requestGroup->getMaxDownloadSpeedLimit();
-    const unsigned int maxUploadLimit = _requestGroup->getMaxUploadSpeedLimit();
+      requestGroup_->getMaxDownloadSpeedLimit();
+    const unsigned int maxUploadLimit = requestGroup_->getMaxUploadSpeedLimit();
     unsigned int thresholdSpeed;
-    if(_requestGroup->getDownloadContext()->
-       getAttribute(bittorrent::BITTORRENT).containsKey(bittorrent::METADATA)) {
+    if(!bittorrent::getTorrentAttrs
+       (requestGroup_->getDownloadContext())->metadata.empty()) {
       thresholdSpeed =
-        _requestGroup->getOption()->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT);
+        requestGroup_->getOption()->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT);
     } else {
       thresholdSpeed = 0;
     }
@@ -96,79 +97,78 @@ bool ActivePeerConnectionCommand::execute() {
       thresholdSpeed = std::min(maxDownloadLimit, thresholdSpeed);
     }
     if(// for seeder state
-       (_pieceStorage->downloadFinished() && _btRuntime->lessThanMaxPeers() &&
+       (pieceStorage_->downloadFinished() && btRuntime_->lessThanMaxPeers() &&
         (maxUploadLimit == 0 || tstat.getUploadSpeed() < maxUploadLimit*0.8)) ||
        // for leecher state
-       (!_pieceStorage->downloadFinished() &&
+       (!pieceStorage_->downloadFinished() &&
         (tstat.getDownloadSpeed() < thresholdSpeed ||
-         _btRuntime->lessThanMinPeers()))) {
+         btRuntime_->lessThanMinPeers()))) {
 
       unsigned int numConnection = 0;
-      if(_pieceStorage->downloadFinished()) {
-        if(_btRuntime->getMaxPeers() > _btRuntime->getConnections()) {
+      if(pieceStorage_->downloadFinished()) {
+        if(btRuntime_->getMaxPeers() > btRuntime_->getConnections()) {
           numConnection =
-            std::min(_numNewConnection,
-                     _btRuntime->getMaxPeers()-_btRuntime->getConnections());
+            std::min(numNewConnection_,
+                     btRuntime_->getMaxPeers()-btRuntime_->getConnections());
         }
       } else {
-        numConnection = _numNewConnection;
+        numConnection = numNewConnection_;
       }
 
       for(unsigned int numAdd = numConnection;
-          numAdd > 0 && _peerStorage->isPeerAvailable(); --numAdd) {
-        SharedHandle<Peer> peer = _peerStorage->getUnusedPeer();
+          numAdd > 0 && peerStorage_->isPeerAvailable(); --numAdd) {
+        SharedHandle<Peer> peer = peerStorage_->getUnusedPeer();
         connectToPeer(peer);
       }
-      if(_btRuntime->getConnections() == 0 &&
-         !_pieceStorage->downloadFinished()) {
-        _btAnnounce->overrideMinInterval(BtAnnounce::DEFAULT_ANNOUNCE_INTERVAL);
+      if(btRuntime_->getConnections() == 0 &&
+         !pieceStorage_->downloadFinished()) {
+        btAnnounce_->overrideMinInterval(BtAnnounce::DEFAULT_ANNOUNCE_INTERVAL);
       }
     }
   }
-  e->commands.push_back(this);
+  e_->addCommand(this);
   return false;
 }
 
 void ActivePeerConnectionCommand::connectToPeer(const SharedHandle<Peer>& peer)
 {
-  if(peer.isNull()) {
+  if(!peer) {
     return;
   }
-  peer->usedBy(e->newCUID());
+  peer->usedBy(e_->newCUID());
   PeerInitiateConnectionCommand* command =
-    new PeerInitiateConnectionCommand(peer->usedBy(), _requestGroup, peer, e,
-                                      _btRuntime);
-  command->setPeerStorage(_peerStorage);
-  command->setPieceStorage(_pieceStorage);
-  e->commands.push_back(command);
-  if(logger->info()) {
-    logger->info(MSG_CONNECTING_TO_PEER,
-                 util::itos(cuid).c_str(), peer->ipaddr.c_str());
-  }
+    new PeerInitiateConnectionCommand(peer->usedBy(), requestGroup_, peer, e_,
+                                      btRuntime_);
+  command->setPeerStorage(peerStorage_);
+  command->setPieceStorage(pieceStorage_);
+  e_->addCommand(command);
+  A2_LOG_INFO(fmt(MSG_CONNECTING_TO_PEER,
+                  getCuid(),
+                  peer->getIPAddress().c_str()));
 }
 
 void ActivePeerConnectionCommand::setBtRuntime
 (const SharedHandle<BtRuntime>& btRuntime)
 {
-  _btRuntime = btRuntime;
+  btRuntime_ = btRuntime;
 }
 
 void ActivePeerConnectionCommand::setPieceStorage
 (const SharedHandle<PieceStorage>& pieceStorage)
 {
-  _pieceStorage = pieceStorage;
+  pieceStorage_ = pieceStorage;
 }
 
 void ActivePeerConnectionCommand::setPeerStorage
 (const SharedHandle<PeerStorage>& peerStorage)
 {
-  _peerStorage = peerStorage;
+  peerStorage_ = peerStorage;
 }
 
 void ActivePeerConnectionCommand::setBtAnnounce
 (const SharedHandle<BtAnnounce>& btAnnounce)
 {
-  _btAnnounce = btAnnounce;
+  btAnnounce_ = btAnnounce;
 }
 
 } // namespace aria2
