@@ -37,7 +37,6 @@
 #include <cassert>
 #include <cstring>
 
-#include "util.h"
 #include "array_fun.h"
 #include "bitfield.h"
 
@@ -61,7 +60,7 @@ BitfieldMan::BitfieldMan(size_t blockLength, uint64_t totalLength)
    cachedFilteredTotalLength_(0)
 {
   if(blockLength_ > 0 && totalLength_ > 0) {
-    blocks_ = totalLength_/blockLength_+(totalLength_%blockLength_ ? 1 : 0);
+    blocks_ = (totalLength_+blockLength_-1)/blockLength_;
     bitfieldLength_ = blocks_/8+(blocks_%8 ? 1 : 0);
     bitfield_ = new unsigned char[bitfieldLength_];
     useBitfield_ = new unsigned char[bitfieldLength_];
@@ -131,6 +130,11 @@ BitfieldMan::~BitfieldMan() {
   delete [] filterBitfield_;
 }
 
+size_t BitfieldMan::getLastBlockLength() const
+{
+  return totalLength_-blockLength_*(blocks_-1);
+}
+
 size_t BitfieldMan::getBlockLength(size_t index) const
 {
   if(index == blocks_-1) {
@@ -165,11 +169,11 @@ bool BitfieldMan::hasMissingPiece
 bool BitfieldMan::getFirstMissingUnusedIndex(size_t& index) const
 {
   if(filterEnabled_) {
-    return bitfield::getFirstMissingIndex
+    return bitfield::getFirstSetBitIndex
       (index, ~array(bitfield_)&~array(useBitfield_)&array(filterBitfield_),
        blocks_);
   } else {
-    return bitfield::getFirstMissingIndex
+    return bitfield::getFirstSetBitIndex
       (index, ~array(bitfield_)&~array(useBitfield_), blocks_);
   }
 }
@@ -178,11 +182,11 @@ size_t BitfieldMan::getFirstNMissingUnusedIndex
 (std::vector<size_t>& out, size_t n) const
 {
   if(filterEnabled_) {
-    return bitfield::getFirstNMissingIndex
+    return bitfield::getFirstNSetBitIndex
       (std::back_inserter(out), n,
        ~array(bitfield_)&~array(useBitfield_)&array(filterBitfield_), blocks_);
   } else {
-    return bitfield::getFirstNMissingIndex
+    return bitfield::getFirstNSetBitIndex
       (std::back_inserter(out), n,
        ~array(bitfield_)&~array(useBitfield_), blocks_);
   }
@@ -191,10 +195,10 @@ size_t BitfieldMan::getFirstNMissingUnusedIndex
 bool BitfieldMan::getFirstMissingIndex(size_t& index) const
 {
   if(filterEnabled_) {
-    return bitfield::getFirstMissingIndex
+    return bitfield::getFirstSetBitIndex
       (index, ~array(bitfield_)&array(filterBitfield_), blocks_);
   } else {
-    return bitfield::getFirstMissingIndex(index, ~array(bitfield_), blocks_);
+    return bitfield::getFirstSetBitIndex(index, ~array(bitfield_), blocks_);
   }
 }
 
@@ -305,6 +309,136 @@ bool BitfieldMan::getSparseMissingUnusedIndex
   }
 }
 
+namespace {
+template<typename Array>
+bool getGeomMissingUnusedIndex
+(size_t& index,
+ size_t minSplitSize,
+ const Array& bitfield,
+ const unsigned char* useBitfield,
+ size_t blockLength,
+ size_t blocks,
+ double base,
+ size_t offsetIndex)
+{
+  double start = 0;
+  double end = 1;
+  while(start+offsetIndex < blocks) {
+    index = blocks;
+    for(size_t i = start+offsetIndex,
+          eoi = std::min(blocks, static_cast<size_t>(end+offsetIndex));
+        i < eoi; ++i) {
+      if(bitfield::test(useBitfield, blocks, i)) {
+        break;
+      } else if(!bitfield::test(bitfield, blocks, i)) {
+        index = i;
+        break;
+      }
+    }
+    if(index < blocks) {
+      return true;
+    } else {
+      start = end;
+      end *= base;
+    }
+  }
+  return getSparseMissingUnusedIndex(index, minSplitSize,
+                                     bitfield, useBitfield,
+                                     blockLength, blocks);
+}
+} // namespace
+
+bool BitfieldMan::getGeomMissingUnusedIndex
+(size_t& index,
+ size_t minSplitSize,
+ const unsigned char* ignoreBitfield,
+ size_t ignoreBitfieldLength,
+ double base,
+ size_t offsetIndex) const
+{
+  if(filterEnabled_) {
+    return aria2::getGeomMissingUnusedIndex
+      (index, minSplitSize,
+       array(ignoreBitfield)|~array(filterBitfield_)|
+       array(bitfield_)|array(useBitfield_),
+       useBitfield_, blockLength_, blocks_,
+       base, offsetIndex);
+  } else {
+    return aria2::getGeomMissingUnusedIndex
+      (index, minSplitSize,
+       array(ignoreBitfield)|array(bitfield_)|array(useBitfield_),
+       useBitfield_, blockLength_, blocks_,
+       base, offsetIndex);
+  }
+}
+
+namespace {
+template<typename Array>
+bool getInorderMissingUnusedIndex
+(size_t& index,
+ size_t minSplitSize,
+ const Array& bitfield,
+ const unsigned char* useBitfield,
+ size_t blockLength_,
+ size_t blocks)
+{
+  // We always return first piece if it is available.
+  if(!bitfield::test(bitfield, blocks, 0) &&
+     !bitfield::test(useBitfield, blocks, 0)) {
+    index = 0;
+    return true;
+  }
+  for(size_t i = 1; i < blocks;) {
+    if(!bitfield::test(bitfield, blocks, i) &&
+       !bitfield::test(useBitfield, blocks, i)) {
+      // If previous piece has already been retrieved, we can download
+      // from this index.
+      if(!bitfield::test(useBitfield, blocks, i-1) &&
+         bitfield::test(bitfield, blocks, i-1)) {
+        index = i;
+        return true;
+      }
+      // Check free space of minSplitSize.
+      size_t j;
+      for(j = i; j < blocks; ++j) {
+        if(bitfield::test(bitfield, blocks, j) ||
+           bitfield::test(useBitfield, blocks, j)) {
+          break;
+        }
+        if((j-i+1)*blockLength_ >= minSplitSize) {
+          index = j;
+          return true;
+        }
+      }
+      i = j+1;
+    } else {
+      ++i;
+    }
+  }
+  return false;
+}
+} // namespace
+
+bool BitfieldMan::getInorderMissingUnusedIndex
+(size_t& index,
+ size_t minSplitSize,
+ const unsigned char* ignoreBitfield,
+ size_t ignoreBitfieldLength) const
+{
+  if(filterEnabled_) {
+    return aria2::getInorderMissingUnusedIndex
+      (index, minSplitSize,
+       array(ignoreBitfield)|~array(filterBitfield_)|
+       array(bitfield_)|array(useBitfield_),
+       useBitfield_, blockLength_, blocks_);
+  } else {
+    return aria2::getInorderMissingUnusedIndex
+      (index, minSplitSize,
+       array(ignoreBitfield)|array(bitfield_)|array(useBitfield_),
+       useBitfield_, blockLength_, blocks_);
+  }
+}
+
 namespace {
 template<typename Array>
 bool copyBitfield(unsigned char* dst, const Array& src, size_t blocks)
@@ -701,6 +835,39 @@ bool BitfieldMan::isBitSetOffsetRange(uint64_t offset, uint64_t length) const
   return true;
 }
 
+uint64_t BitfieldMan::getOffsetCompletedLength
+(uint64_t offset,
+ uint64_t length) const
+{
+  uint64_t res = 0;
+  if(length == 0 || totalLength_ <= offset) {
+    return 0;
+  }
+  if(totalLength_ < offset+length) {
+    length = totalLength_-offset;
+  }
+  size_t start = offset/blockLength_;
+  size_t end = (offset+length-1)/blockLength_;
+  if(start == end) {
+    if(isBitSet(start)) {
+      res = length;
+    }
+  } else {
+    if(isBitSet(start)) {
+      res += (start+1)*blockLength_-offset;
+    }
+    for(size_t i = start+1; i <= end-1; ++i) {
+      if(isBitSet(i)) {
+        res += blockLength_;
+      }
+    }
+    if(isBitSet(end)) {
+      res += offset+length-end*blockLength_;
+    }
+  }
+  return res;
+}
+
 uint64_t BitfieldMan::getMissingUnusedLength(size_t startingIndex) const
 {
   if(startingIndex < 0 || blocks_ <= startingIndex) {
@@ -716,4 +883,30 @@ uint64_t BitfieldMan::getMissingUnusedLength(size_t startingIndex) const
   return length;
 }
 
+BitfieldMan::Range::Range(size_t startIndex, size_t endIndex)
+ :
+  startIndex(startIndex),
+  endIndex(endIndex)
+{}
+  
+size_t BitfieldMan::Range::getSize() const
+{
+  return endIndex-startIndex;
+}
+
+size_t BitfieldMan::Range::getMidIndex() const
+{
+  return (endIndex-startIndex)/2+startIndex;
+}
+
+bool BitfieldMan::Range::operator<(const Range& range) const
+{
+  return getSize() < range.getSize();
+}
+    
+bool BitfieldMan::Range::operator==(const Range& range) const
+{
+  return getSize() == range.getSize();
+}
+
 } // namespace aria2
