@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_DEPENDENCY_H_
-#define _D_BT_DEPENDENCY_H_
+#ifndef D_BT_DEPENDENCY_H
+#define D_BT_DEPENDENCY_H
 
 #include "Dependency.h"
 
@@ -41,16 +41,14 @@ namespace aria2 {
 
 class RequestGroup;
 class Option;
-class Logger;
 
 class BtDependency : public Dependency
 {
 private:
-  WeakHandle<RequestGroup> dependant_;
+  RequestGroup* dependant_;
   SharedHandle<RequestGroup> dependee_;
-  Logger* logger_;
 public:
-  BtDependency(const WeakHandle<RequestGroup>& dependant,
+  BtDependency(RequestGroup* dependant,
                const SharedHandle<RequestGroup>& dependee);
 
   virtual ~BtDependency();
@@ -62,4 +60,4 @@ typedef SharedHandle<BtDependency> BtDependencyHandle;
 
 } // namespace aria2
 
-#endif // _D_BT_DEPENDENCY_H_
+#endif // D_BT_DEPENDENCY_H
