@@ -44,7 +44,6 @@
 #include "DlAbortEx.h"
 #include "message.h"
 #include "util.h"
-#include "StringFormat.h"
 #include "a2functional.h"
 
 namespace aria2 {
@@ -58,6 +57,8 @@ BtExtendedMessage::BtExtendedMessage
   msgLength_(0)
 {}
 
+BtExtendedMessage::~BtExtendedMessage() {}
+
 unsigned char* BtExtendedMessage::createMessage()
 {
   /**
@@ -99,7 +100,7 @@ BtExtendedMessage::create(const SharedHandle<ExtensionMessageFactory>& factory,
 {
   bittorrent::assertPayloadLengthGreater(1, dataLength, NAME);
   bittorrent::assertID(ID, data, NAME);
-  assert(!factory.isNull());
+  assert(factory);
   ExtensionMessageHandle extmsg = factory->createMessage(data+1,
                                                          dataLength-1);
   BtExtendedMessageHandle message(new BtExtendedMessage(extmsg));
@@ -108,7 +109,7 @@ BtExtendedMessage::create(const SharedHandle<ExtensionMessageFactory>& factory,
 
 void BtExtendedMessage::doReceivedAction()
 {
-  if(!extensionMessage_.isNull()) {
+  if(extensionMessage_) {
     extensionMessage_->doReceivedAction();
   }
 }
