@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_INTERESTED_MESSAGE_H_
-#define _D_BT_INTERESTED_MESSAGE_H_
+#ifndef D_BT_INTERESTED_MESSAGE_H
+#define D_BT_INTERESTED_MESSAGE_H
 
 #include "ZeroBtMessage.h"
 
@@ -46,9 +46,10 @@ typedef SharedHandle<BtInterestedMessage> BtInterestedMessageHandle;
 
 class BtInterestedMessage : public ZeroBtMessage {
 private:
-  SharedHandle<PeerStorage> _peerStorage;
+  SharedHandle<PeerStorage> peerStorage_;
 public:
-  BtInterestedMessage():ZeroBtMessage(ID, NAME) {}
+  BtInterestedMessage();
+  virtual ~BtInterestedMessage();
 
   static const uint8_t ID = 2;
 
@@ -68,4 +69,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_BT_INTERESTED_MESSAGE_H_
+#endif // D_BT_INTERESTED_MESSAGE_H
