@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -39,19 +39,19 @@ namespace aria2 {
 
 const std::string BtKeepAliveMessage::NAME("keep alive");
 
-const unsigned char* BtKeepAliveMessage::getMessage() {
-  if(!msg) {
-    /**
-     * len --- 0, 4bytes
-     * total: 4bytes
-     */
-    msg = new unsigned char[MESSAGE_LENGTH];
-    memset(msg, 0, MESSAGE_LENGTH);
-  }
+unsigned char* BtKeepAliveMessage::createMessage()
+{
+  /**
+   * len --- 0, 4bytes
+   * total: 4bytes
+   */
+  unsigned char* msg = new unsigned char[MESSAGE_LENGTH];
+  memset(msg, 0, MESSAGE_LENGTH);
   return msg;
 }
 
-size_t BtKeepAliveMessage::getMessageLength() {
+size_t BtKeepAliveMessage::getMessageLength()
+{
   return MESSAGE_LENGTH;
 }
 
