@@ -32,15 +32,15 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_PIECE_MESSAGE_VALIDATOR_H_
-#define _D_BT_PIECE_MESSAGE_VALIDATOR_H_
+#ifndef D_BT_PIECE_MESSAGE_VALIDATOR_H
+#define D_BT_PIECE_MESSAGE_VALIDATOR_H
 
 #include "BtMessageValidator.h"
-#include "BtPieceMessage.h"
-#include "bittorrent_helper.h"
 
 namespace aria2 {
 
+class BtPieceMessage;
+
 class BtPieceMessageValidator : public BtMessageValidator {
 private:
   const BtPieceMessage* message_;
@@ -49,20 +49,15 @@ private:
 public:
   BtPieceMessageValidator(const BtPieceMessage* message,
                           size_t numPiece,
-                          size_t pieceLength):
-    message_(message),
-    numPiece_(numPiece),
-    pieceLength_(pieceLength) {}
+                          size_t pieceLength);
 
-  virtual void validate()
-  {
-    bittorrent::checkIndex(message_->getIndex(), numPiece_);
-    bittorrent::checkBegin(message_->getBegin(), pieceLength_);
-  }
+  ~BtPieceMessageValidator();
+
+  virtual void validate();
 };
 
 typedef SharedHandle<BtPieceMessageValidator> BtPieceMessageValidatorHandle;
 
 } // namespace aria2
 
-#endif // _D_BT_PIECE_MESSAGE_VALIDATOR_H_
+#endif // D_BT_PIECE_MESSAGE_VALIDATOR_H
