@@ -41,13 +41,14 @@
 #include "LogFactory.h"
 #include "SimpleRandomizer.h"
 #include "wallclock.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-BtSeederStateChoke::BtSeederStateChoke():
-  round_(0),
-  lastRound_(0),
-  logger_(LogFactory::getInstance()) {}
+BtSeederStateChoke::BtSeederStateChoke()
+  : round_(0),
+    lastRound_(0)
+{}
 
 BtSeederStateChoke::~BtSeederStateChoke() {}
 
@@ -56,10 +57,43 @@ BtSeederStateChoke::PeerEntry::PeerEntry
   peer_(peer),
   outstandingUpload_(peer->countOutstandingUpload()),
   lastAmUnchoking_(peer->getLastAmUnchoking()),
-  recentUnchoking_(lastAmUnchoking_.difference(global::wallclock) < TIME_FRAME),
+  recentUnchoking_(lastAmUnchoking_.difference(global::wallclock()) < TIME_FRAME),
   uploadSpeed_(peer->calculateUploadSpeed())
 {}
 
+BtSeederStateChoke::PeerEntry::PeerEntry(const PeerEntry& c)
+  : peer_(c.peer_),
+    outstandingUpload_(c.outstandingUpload_),
+    lastAmUnchoking_(c.lastAmUnchoking_),
+    recentUnchoking_(c.recentUnchoking_),
+    uploadSpeed_(c.uploadSpeed_)
+{}
+
+BtSeederStateChoke::PeerEntry::~PeerEntry() {}
+
+void BtSeederStateChoke::PeerEntry::swap(PeerEntry& c)
+{
+  using std::swap;
+  swap(peer_, c.peer_);
+  swap(outstandingUpload_, c.outstandingUpload_);
+  swap(lastAmUnchoking_, c.lastAmUnchoking_);
+  swap(recentUnchoking_, c.recentUnchoking_);
+  swap(uploadSpeed_, c.uploadSpeed_);
+}
+
+BtSeederStateChoke::PeerEntry& BtSeederStateChoke::PeerEntry::operator=
+(const PeerEntry& c)
+{
+  if(this != &c) {
+    peer_ = c.peer_;
+    outstandingUpload_ = c.outstandingUpload_;
+    lastAmUnchoking_ = c.lastAmUnchoking_;
+    recentUnchoking_ = c.recentUnchoking_;
+    uploadSpeed_ = c.uploadSpeed_;
+  }
+  return *this;
+}
+
 bool
 BtSeederStateChoke::PeerEntry::operator<(const PeerEntry& rhs) const
 {
@@ -100,8 +134,9 @@ void BtSeederStateChoke::unchoke
   for(std::vector<PeerEntry>::iterator eoi = peers.end();
       r != eoi && count; ++r, --count) {
     (*r).getPeer()->chokingRequired(false);
-    logger_->info("RU: %s, ulspd=%u", (*r).getPeer()->getIPAddress().c_str(),
-                  (*r).getUploadSpeed());
+    A2_LOG_INFO(fmt("RU: %s, ulspd=%u",
+                    (*r).getPeer()->getIPAddress().c_str(),
+                    (*r).getUploadSpeed()));
   }
 
   if(round_ < 2) {
@@ -111,7 +146,7 @@ void BtSeederStateChoke::unchoke
       std::random_shuffle(r, peers.end(),
                           *(SimpleRandomizer::getInstance().get()));
       (*r).getPeer()->optUnchoking(true);
-      logger_->info("POU: %s", (*r).getPeer()->getIPAddress().c_str());
+      A2_LOG_INFO(fmt("POU: %s", (*r).getPeer()->getIPAddress().c_str()));
     }
   }
 }
@@ -124,14 +159,14 @@ public:
     peer->chokingRequired(true);
   }
 };
-}
+} // namespace
 
 void
 BtSeederStateChoke::executeChoke
 (const std::vector<SharedHandle<Peer> >& peerSet)
 {
-  logger_->info("Seeder state, %d choke round started", round_);
-  lastRound_ = global::wallclock;
+  A2_LOG_INFO(fmt("Seeder state, %d choke round started", round_));
+  lastRound_ = global::wallclock();
 
   std::vector<PeerEntry> peerEntries;
 
@@ -151,4 +186,11 @@ BtSeederStateChoke::executeChoke
   }
 }
 
+void swap
+(BtSeederStateChoke::PeerEntry& a,
+ BtSeederStateChoke::PeerEntry& b)
+{
+  a.swap(b);
+}
+
 } // namespace aria2
