@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BYTE_ARRAY_DISK_WRITER_H_
-#define _D_BYTE_ARRAY_DISK_WRITER_H_
+#ifndef D_BYTE_ARRAY_DISK_WRITER_H
+#define D_BYTE_ARRAY_DISK_WRITER_H
 
 #include "DiskWriter.h"
 #include <sstream>
@@ -42,11 +42,11 @@ namespace aria2 {
 
 class ByteArrayDiskWriter : public DiskWriter {
 private:
-  std::stringstream buf;
-
+  std::stringstream buf_;
+  size_t maxLength_;
   void clear();
 public:
-  ByteArrayDiskWriter();
+  ByteArrayDiskWriter(size_t maxLength = 5*1024*1024);
   virtual ~ByteArrayDiskWriter();
 
   virtual void initAndOpenFile(uint64_t totalLength = 0);
@@ -62,23 +62,13 @@ public:
 
   virtual uint64_t size();
 
-  virtual void enableDirectIO() {}
+  void setString(const std::string& s);
 
-  virtual void disableDirectIO() {}
-
-  void setString(const std::string& s)
-  {
-    buf.str(s);
-  }
-
-  std::string getString() const
-  {
-    return buf.str();
-  }
+  std::string getString() const;
 };
 
 typedef SharedHandle<ByteArrayDiskWriter> ByteArrayDiskWriterHandle;
 
 } // namespace aria2
 
-#endif // _D_BYTE_ARRAY_DISK_WRITER_H_
+#endif // D_BYTE_ARRAY_DISK_WRITER_H
