@@ -41,9 +41,6 @@
 #include "FileAllocationEntry.h"
 #include "DownloadEngine.h"
 #include "Option.h"
-#include "DownloadContext.h"
-#include "RequestGroupMan.h"
-#include "ServerStatMan.h"
 
 namespace aria2 {
 
@@ -61,7 +58,7 @@ void CheckIntegrityEntry::validateChunk()
 
 uint64_t CheckIntegrityEntry::getTotalLength()
 {
-  if(validator_.isNull()) {
+  if(!validator_) {
     return 0;
   } else {
     return validator_->getTotalLength();
@@ -70,7 +67,7 @@ uint64_t CheckIntegrityEntry::getTotalLength()
 
 off_t CheckIntegrityEntry::getCurrentLength()
 {
-  if(validator_.isNull()) {
+  if(!validator_) {
     return 0;
   } else {
     return validator_->getCurrentOffset();
@@ -97,10 +94,6 @@ void CheckIntegrityEntry::proceedFileAllocation
   } else {
     entry->prepareForNextAction(commands, e);
   }
-  // Disable directIO when fallocation() is going to be used.
-  if(getRequestGroup()->getOption()->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
-    entry->disableDirectIO();
-  }
 }
 
 void CheckIntegrityEntry::setValidator
