@@ -35,55 +35,92 @@
 #include "CookieStorage.h"
 
 #include <cstring>
+#include <cstdio>
 #include <algorithm>
-#include <fstream>
 
 #include "util.h"
 #include "LogFactory.h"
 #include "Logger.h"
 #include "DlAbortEx.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "NsCookieParser.h"
 #include "File.h"
 #include "a2functional.h"
 #include "A2STR.h"
 #include "message.h"
+#include "cookie_helper.h"
+#include "BufferedFile.h"
 #ifdef HAVE_SQLITE3
 # include "Sqlite3CookieParserImpl.h"
 #endif // HAVE_SQLITE3
 
 namespace aria2 {
 
+CookieStorage::DomainEntry::DomainEntry(const std::string& domain)
+ : key_(util::isNumericHost(domain)?domain:cookie::reverseDomainLevel(domain))
+{}
+
 CookieStorage::DomainEntry::DomainEntry
-(const std::string& domain):key_(domain)
+(const DomainEntry& c)
+  : key_(c.key_),
+    lastAccessTime_(c.lastAccessTime_),
+    cookies_(c.cookies_)
+{}
+
+CookieStorage::DomainEntry::~DomainEntry() {}
+
+CookieStorage::DomainEntry& CookieStorage::DomainEntry::operator=
+(const DomainEntry& c)
 {
-  std::reverse(key_.begin(), key_.end());
+  if(this != &c) {
+    key_ = c.key_;
+    lastAccessTime_ = c.lastAccessTime_;
+    cookies_ = c.cookies_;
+  }
+  return *this;
 }
 
-void CookieStorage::DomainEntry::updateLastAccess()
+void CookieStorage::DomainEntry::swap(CookieStorage::DomainEntry& c)
 {
-  lastAccess_ = time(0);
+  using std::swap;
+  swap(key_, c.key_);
+  swap(lastAccessTime_, c.lastAccessTime_);
+  swap(cookies_, c.cookies_);
 }
 
-bool CookieStorage::DomainEntry::addCookie(const Cookie& cookie)
+void swap(CookieStorage::DomainEntry& a, CookieStorage::DomainEntry& b)
 {
-  updateLastAccess();
-  std::deque<Cookie>::iterator i = std::find(cookies_.begin(), cookies_.end(),
-                                             cookie);
+  a.swap(b);
+}
+
+bool CookieStorage::DomainEntry::addCookie(const Cookie& cookie, time_t now)
+{
+  setLastAccessTime(now);
+  std::deque<Cookie>::iterator i =
+    std::find(cookies_.begin(), cookies_.end(), cookie);
   if(i == cookies_.end()) {
-    if(cookie.isExpired()) {
+    if(cookie.isExpired(now)) {
       return false;
     } else {
       if(cookies_.size() >= CookieStorage::MAX_COOKIE_PER_DOMAIN) {
-        std::deque<Cookie>::iterator m = std::min_element
-          (cookies_.begin(), cookies_.end(), LeastRecentAccess<Cookie>());
-        *m = cookie;
+        cookies_.erase
+          (std::remove_if(cookies_.begin(), cookies_.end(),
+                          std::bind2nd
+                          (std::mem_fun_ref(&Cookie::isExpired), now)),
+           cookies_.end());
+        if(cookies_.size() >= CookieStorage::MAX_COOKIE_PER_DOMAIN) {
+          std::deque<Cookie>::iterator m = std::min_element
+            (cookies_.begin(), cookies_.end(), LeastRecentAccess<Cookie>());
+          *m = cookie;
+        } else {
+          cookies_.push_back(cookie);
+        }
       } else {
         cookies_.push_back(cookie);
       }
       return true;
     }
-  } else if(cookie.isExpired()) {
+  } else if(cookie.isExpired(now)) {
     cookies_.erase(i);
     return false;
   } else {
@@ -97,28 +134,43 @@ bool CookieStorage::DomainEntry::contains(const Cookie& cookie) const
   return std::find(cookies_.begin(), cookies_.end(), cookie) != cookies_.end();
 }
 
-void CookieStorage::DomainEntry::writeCookie(std::ostream& o) const
+bool CookieStorage::DomainEntry::writeCookie(BufferedFile& fp) const
 {
   for(std::deque<Cookie>::const_iterator i = cookies_.begin(),
         eoi = cookies_.end(); i != eoi; ++i) {
-    o << (*i).toNsCookieFormat() << "\n";
+    std::string data = (*i).toNsCookieFormat();
+    data += "\n";
+    if(fp.write(data.data(), data.size()) != data.size()) {
+      return false;
+    }
   }
+  return true;
 }
 
-CookieStorage::CookieStorage():logger_(LogFactory::getInstance()) {}
+size_t CookieStorage::DomainEntry::countCookie() const
+{
+  return cookies_.size();
+}
+
+
+bool CookieStorage::DomainEntry::operator<(const DomainEntry& de) const
+{
+  return key_ < de.key_;
+}
+
+CookieStorage::CookieStorage() {}
 
 CookieStorage::~CookieStorage() {}
 
+namespace {
 // See CookieStorageTest::testDomainIsFull() in CookieStorageTest.cc
-static const size_t DOMAIN_EVICTION_TRIGGER = 2000;
+const size_t DOMAIN_EVICTION_TRIGGER = 2000;
 
-static const double DOMAIN_EVICTION_RATE = 0.1;
+const double DOMAIN_EVICTION_RATE = 0.1;
+} // namespace
 
-bool CookieStorage::store(const Cookie& cookie)
+bool CookieStorage::store(const Cookie& cookie, time_t now)
 {
-  if(!cookie.good()) {
-    return false;
-  }
   if(domains_.size() >= DOMAIN_EVICTION_TRIGGER) {
     std::sort(domains_.begin(), domains_.end(),
               LeastRecentAccess<DomainEntry>());
@@ -131,9 +183,9 @@ bool CookieStorage::store(const Cookie& cookie)
     std::lower_bound(domains_.begin(), domains_.end(), v);
   bool added = false;
   if(i != domains_.end() && (*i).getKey() == v.getKey()) {
-    added = (*i).addCookie(cookie);
+    added = (*i).addCookie(cookie, now);
   } else {
-    added = v.addCookie(cookie);
+    added = v.addCookie(cookie, now);
     if(added) {
       domains_.insert(i, v);
     }
@@ -141,13 +193,15 @@ bool CookieStorage::store(const Cookie& cookie)
   return added;
 }
 
-bool CookieStorage::parseAndStore(const std::string& setCookieString,
-                                  const std::string& requestHost,
-                                  const std::string& requestPath)
+bool CookieStorage::parseAndStore
+(const std::string& setCookieString,
+ const std::string& requestHost,
+ const std::string& defaultPath,
+ time_t now)
 {
-  Cookie cookie = parser_.parse(setCookieString, requestHost, requestPath);
-  if(cookie.validate(requestHost, requestPath)) {
-    return store(cookie);
+  Cookie cookie;
+  if(cookie::parse(cookie, setCookieString, requestHost, defaultPath, now)) {
+    return store(cookie, now);
   } else {
     return false;
   }
@@ -164,6 +218,7 @@ struct CookiePathDivider {
   }
 };
 
+namespace {
 class CookiePathDividerConverter {
 public:
   CookiePathDivider operator()(const Cookie& cookie) const
@@ -176,50 +231,49 @@ public:
     return cookiePathDivider.cookie_;
   }
 };
+} // namespace
 
+namespace {
 class OrderByPathDepthDesc:public std::binary_function<Cookie, Cookie, bool> {
 public:
   bool operator()
   (const CookiePathDivider& lhs, const CookiePathDivider& rhs) const
   {
-    // Sort by path-length.
+    // From http://tools.ietf.org/html/rfc6265#section-5.4:
+    // 2.  The user agent SHOULD sort the cookie-list in the following
+    //    order:
     //
-    // RFC2965 says: Note that the NAME=VALUE pair for the cookie with
-    // the more specific Path attribute, /acme/ammo, comes before the
-    // one with the less specific Path attribute, /acme.  Further note
-    // that the same cookie name appears more than once.
+    //    *  Cookies with longer paths are listed before cookies with
+    //       shorter paths.
     //
-    // Netscape spec says: When sending cookies to a server, all
-    // cookies with a more specific path mapping should be sent before
-    // cookies with less specific path mappings. For example, a cookie
-    // "name1=foo" with a path mapping of "/" should be sent after a
-    // cookie "name1=foo2" with a path mapping of "/bar" if they are
-    // both to be sent.
-    int comp = lhs.pathDepth_-rhs.pathDepth_;
-    if(comp == 0) {
-      return lhs.cookie_.getCreationTime() < rhs.cookie_.getCreationTime();
-    } else {
-      return comp > 0;
-    }
+    //    *  Among cookies that have equal-length path fields, cookies with
+    //       earlier creation-times are listed before cookies with later
+    //       creation-times.
+    return lhs.pathDepth_ > rhs.pathDepth_ ||
+      (!(rhs.pathDepth_ > lhs.pathDepth_) &&
+       lhs.cookie_.getCreationTime() < rhs.cookie_.getCreationTime());
   }
 };
+} // namespace
 
+namespace {
 template<typename DomainInputIterator, typename CookieOutputIterator>
-static void searchCookieByDomainSuffix
+void searchCookieByDomainSuffix
 (const std::string& domain,
  DomainInputIterator first, DomainInputIterator last, CookieOutputIterator out,
  const std::string& requestHost,
  const std::string& requestPath,
- time_t date, bool secure)
+ time_t now, bool secure)
 {
   CookieStorage::DomainEntry v(domain);
   std::deque<CookieStorage::DomainEntry>::iterator i =
     std::lower_bound(first, last, v);
   if(i != last && (*i).getKey() == v.getKey()) {
-    (*i).updateLastAccess();
-    (*i).findCookie(out, requestHost, requestPath, date, secure);
+    (*i).setLastAccessTime(now);
+    (*i).findCookie(out, requestHost, requestPath, now, secure);
   }
 }
+} // namespace
 
 bool CookieStorage::contains(const Cookie& cookie) const
 {
@@ -233,36 +287,32 @@ bool CookieStorage::contains(const Cookie& cookie) const
   }
 }
 
-std::vector<Cookie> CookieStorage::criteriaFind(const std::string& requestHost,
-                                               const std::string& requestPath,
-                                               time_t date, bool secure)
+std::vector<Cookie> CookieStorage::criteriaFind
+(const std::string& requestHost,
+ const std::string& requestPath,
+ time_t now,
+ bool secure)
 {
   std::vector<Cookie> res;
-  bool numericHost = util::isNumericHost(requestHost);
-  searchCookieByDomainSuffix
-    ((!numericHost && requestHost.find(A2STR::DOT_C) == std::string::npos)?
-     requestHost+".local":requestHost,
-     domains_.begin(), domains_.end(),
-     std::back_inserter(res),
-     requestHost, requestPath, date, secure);
-  if(!numericHost) {
-    std::string normRequestHost = Cookie::normalizeDomain(requestHost);
-    std::vector<std::string> domainComponents;
-    util::split(normRequestHost, std::back_inserter(domainComponents),
-                A2STR::DOT_C);
-    if(domainComponents.size() <= 1) {
-      return res;
-    }
-    std::reverse(domainComponents.begin(), domainComponents.end());
-    std::string domain = A2STR::DOT_C;
-    domain += domainComponents[0];
-    for(std::vector<std::string>::const_iterator di =
-          domainComponents.begin()+1, eoi = domainComponents.end();
-        di != eoi; ++di) {
-      domain = strconcat(A2STR::DOT_C, *di, domain);
-      searchCookieByDomainSuffix(domain, domains_.begin(), domains_.end(),
-                                 std::back_inserter(res),
-                                 normRequestHost, requestPath, date, secure);
+  if(requestPath.empty()) {
+    return res;
+  }
+  if(util::isNumericHost(requestHost)) {
+    searchCookieByDomainSuffix
+      (requestHost, domains_.begin(), domains_.end(), std::back_inserter(res),
+       requestHost, requestPath, now, secure);
+  } else {
+    std::vector<std::string> levels;
+    util::split(requestHost, std::back_inserter(levels),A2STR::DOT_C);
+    std::reverse(levels.begin(), levels.end());
+    std::string domain;
+    for(std::vector<std::string>::const_iterator i =
+          levels.begin(), eoi = levels.end();
+        i != eoi; ++i, domain.insert(domain.begin(), '.')) {
+      domain.insert(domain.begin(), (*i).begin(), (*i).end());
+      searchCookieByDomainSuffix
+        (domain, domains_.begin(), domains_.end(),
+         std::back_inserter(res), requestHost, requestPath, now, secure);
     }
   }
   std::vector<CookiePathDivider> divs;
@@ -284,48 +334,44 @@ size_t CookieStorage::size() const
   return numCookie;
 }
 
-bool CookieStorage::load(const std::string& filename)
+bool CookieStorage::load(const std::string& filename, time_t now)
 {
   char header[16]; // "SQLite format 3" plus \0
-  std::ifstream s(filename.c_str(), std::ios::binary);
-  if(!s) {
-    logger_->error("Failed to open cookie file %s", filename.c_str());
-    return false;
-  }
-  s.get(header, sizeof(header));
-  if(!s) {
-    logger_->error("Failed to read header of cookie file %s",
-                   filename.c_str());
-    return false;
+  size_t headlen;
+  {
+    BufferedFile fp(filename, BufferedFile::READ);
+    if(!fp) {
+      A2_LOG_ERROR(fmt("Failed to open cookie file %s", filename.c_str()));
+      return false;
+    }
+    headlen = fp.read(header, sizeof(header));
   }
   try {
-    if(std::string(header) == "SQLite format 3") {
+    if(headlen == 16 && memcmp(header, "SQLite format 3\0", 16) == 0) {
 #ifdef HAVE_SQLITE3
       std::vector<Cookie> cookies;
       try {
         Sqlite3MozCookieParser(filename).parse(cookies);
       } catch(RecoverableException& e) {
-        if(logger_->info()) {
-          logger_->info(EX_EXCEPTION_CAUGHT, e);
-          logger_->info("This does not look like Firefox3 cookie file."
-                        " Retrying, assuming it is Chromium cookie file.");
-        }
+        A2_LOG_INFO_EX(EX_EXCEPTION_CAUGHT, e);
+        A2_LOG_INFO("This does not look like Firefox3 cookie file."
+                    " Retrying, assuming it is Chromium cookie file.");
         // Try chrome cookie format
         Sqlite3ChromiumCookieParser(filename).parse(cookies);
       }
-      storeCookies(cookies.begin(), cookies.end());
+      storeCookies(cookies.begin(), cookies.end(), now);
 #else // !HAVE_SQLITE3
       throw DL_ABORT_EX
         ("Cannot read SQLite3 database because SQLite3 support is disabled by"
          " configuration.");
 #endif // !HAVE_SQLITE3
     } else {
-      std::vector<Cookie> cookies = NsCookieParser().parse(filename);
-      storeCookies(cookies.begin(), cookies.end());
+      std::vector<Cookie> cookies = NsCookieParser().parse(filename, now);
+      storeCookies(cookies.begin(), cookies.end(), now);
     }
     return true;
   } catch(RecoverableException& e) {
-    logger_->error("Failed to load cookies from %s", filename.c_str());
+    A2_LOG_ERROR(fmt("Failed to load cookies from %s", filename.c_str()));
     return false;
   }
 }
@@ -334,30 +380,42 @@ bool CookieStorage::saveNsFormat(const std::string& filename)
 {
   std::string tempfilename = filename+"__temp";
   {
-    std::ofstream o(tempfilename.c_str(), std::ios::binary);
-    if(!o) {
-      logger_->error("Cannot create cookie file %s, cause %s",
-                     filename.c_str(), strerror(errno));
+    BufferedFile fp(tempfilename, BufferedFile::WRITE);
+    if(!fp) {
+      A2_LOG_ERROR(fmt("Cannot create cookie file %s", filename.c_str()));
       return false;
     }
     for(std::deque<DomainEntry>::const_iterator i = domains_.begin(),
           eoi = domains_.end(); i != eoi; ++i) {
-      (*i).writeCookie(o);
+      if(!(*i).writeCookie(fp)) {
+        A2_LOG_ERROR(fmt("Failed to save cookies to %s", filename.c_str()));
+        return false;
+      }
     }
-    o.flush();
-    if(!o) {
-      logger_->error("Failed to save cookies to %s, cause %s",
-                     filename.c_str(), strerror(errno));
+    if(fp.close() == EOF) {
+      A2_LOG_ERROR(fmt("Failed to save cookies to %s", filename.c_str()));
       return false;
     }  
   }
   if(File(tempfilename).renameTo(filename)) {
     return true;
   } else {
-    logger_->error("Could not rename file %s as %s",
-                   tempfilename.c_str(), filename.c_str());
+    A2_LOG_ERROR(fmt("Could not rename file %s as %s",
+                     tempfilename.c_str(),
+                     filename.c_str()));
     return false;
   }
 }
 
 } // namespace aria2
+
+namespace std {
+template<>
+void swap<aria2::CookieStorage::DomainEntry>
+(aria2::CookieStorage::DomainEntry& a,
+ aria2::CookieStorage::DomainEntry& b)
+{
+  a.swap(b);
+}
+} // namespace std
+
