@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -35,7 +35,6 @@
 #include "DHTConnectionImpl.h"
 
 #include <utility>
-#include <deque>
 #include <algorithm>
 
 #include "LogFactory.h"
@@ -44,54 +43,63 @@
 #include "util.h"
 #include "Socket.h"
 #include "SimpleRandomizer.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DHTConnectionImpl::DHTConnectionImpl():_socket(new SocketCore(SOCK_DGRAM)),
-				       _logger(LogFactory::getInstance()) {}
+DHTConnectionImpl::DHTConnectionImpl(int family)
+  : socket_(new SocketCore(SOCK_DGRAM)),
+    family_(family)
+{}
 
 DHTConnectionImpl::~DHTConnectionImpl() {}
 
-bool DHTConnectionImpl::bind(uint16_t& port, IntSequence& ports)
+bool DHTConnectionImpl::bind
+(uint16_t& port, const std::string& addr, IntSequence& ports)
 {
-  std::deque<int32_t> randPorts = ports.flush();
+  std::vector<int32_t> randPorts = ports.flush();
   std::random_shuffle(randPorts.begin(), randPorts.end(),
-		      *SimpleRandomizer::getInstance().get());
+                      *SimpleRandomizer::getInstance().get());
   
-  for(std::deque<int32_t>::const_iterator portItr = randPorts.begin();
-      portItr != randPorts.end(); ++portItr) {
+  for(std::vector<int32_t>::const_iterator portItr = randPorts.begin(),
+        eoi = randPorts.end(); portItr != eoi; ++portItr) {
     if(!(0 < (*portItr) && (*portItr) <= 65535)) {
       continue;
     }
     port = (*portItr);
-    if(bind(port)) {
+    if(bind(port, addr)) {
       return true;
     }
   }
   return false;
 }
 
-bool DHTConnectionImpl::bind(uint16_t& port)
+bool DHTConnectionImpl::bind(uint16_t& port, const std::string& addr)
 {
+  const int ipv = (family_ == AF_INET) ? 4 : 6;
   try {
-    _socket->bind(port);
-    _socket->setNonBlockingMode();
+    if(addr.empty()) {
+      socket_->bind(A2STR::NIL, port, family_);
+    } else {
+      socket_->bind(addr, port, family_);
+    }
+    socket_->setNonBlockingMode();
     std::pair<std::string, uint16_t> svaddr;
-    _socket->getAddrInfo(svaddr);
+    socket_->getAddrInfo(svaddr);
     port = svaddr.second;
-    _logger->notice("DHT: listening to port %d", port);
+    A2_LOG_NOTICE(fmt("IPv%d DHT: listening to port %u", ipv, port));
     return true;
   } catch(RecoverableException& e) {
-    _logger->error("Failed to bind for DHT. port=%u", e, port);
+    A2_LOG_ERROR_EX(fmt("IPv%d DHT: failed to bind port %u", ipv, port), e);
   }
   return false;
 }
 
 ssize_t DHTConnectionImpl::receiveMessage(unsigned char* data, size_t len,
-					  std::string& host, uint16_t& port)
+                                          std::string& host, uint16_t& port)
 {
   std::pair<std::string, uint16_t> remoteHost;
-  ssize_t length = _socket->readDataFrom(data, len, remoteHost);
+  ssize_t length = socket_->readDataFrom(data, len, remoteHost);
   if(length == 0) {
     return length;
   } else {
@@ -102,9 +110,9 @@ ssize_t DHTConnectionImpl::receiveMessage(unsigned char* data, size_t len,
 }
 
 ssize_t DHTConnectionImpl::sendMessage(const unsigned char* data, size_t len,
-				       const std::string& host, uint16_t port)
+                                       const std::string& host, uint16_t port)
 {
-  return _socket->writeData(data, len, host, port);
+  return socket_->writeData(data, len, host, port);
 }
 
 } // namespace aria2
