@@ -49,7 +49,7 @@ const std::string DHTMessage::ID("id");
 DHTMessage::DHTMessage(const SharedHandle<DHTNode>& localNode,
                        const SharedHandle<DHTNode>& remoteNode,
                        const std::string& transactionID):
-  _localNode(localNode), _remoteNode(remoteNode), _transactionID(transactionID)
+  localNode_(localNode), remoteNode_(remoteNode), transactionID_(transactionID)
 {
   if(transactionID.empty()) {
     generateTransactionID();
@@ -62,7 +62,7 @@ void DHTMessage::generateTransactionID()
 {
   unsigned char tid[DHT_TRANSACTION_ID_LENGTH];
   util::generateRandomData(tid, DHT_TRANSACTION_ID_LENGTH);
-  _transactionID = std::string(&tid[0], &tid[DHT_TRANSACTION_ID_LENGTH]);
+  transactionID_ = std::string(&tid[0], &tid[DHT_TRANSACTION_ID_LENGTH]);
 }
 
 } // namespace aria2
