@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DHT_MESSAGE_TRACKER_ENTRY_H_
-#define _D_DHT_MESSAGE_TRACKER_ENTRY_H_
+#ifndef D_DHT_MESSAGE_TRACKER_ENTRY_H
+#define D_DHT_MESSAGE_TRACKER_ENTRY_H
 
 #include "common.h"
 
@@ -42,7 +42,6 @@
 #include "SharedHandle.h"
 #include "DHTConstants.h"
 #include "TimerA2.h"
-#include "wallclock.h"
 
 namespace aria2 {
 
@@ -69,6 +68,8 @@ public:
                          const SharedHandle<DHTMessageCallback>& callback =
                          SharedHandle<DHTMessageCallback>());
 
+  ~DHTMessageTrackerEntry();
+
   bool isTimeout() const;
 
   void extendTimeout();
@@ -90,12 +91,9 @@ public:
     return callback_;
   }  
 
-  int64_t getElapsedMillis() const
-  {
-    return dispatchedTime_.differenceInMillis(global::wallclock);
-  }
+  int64_t getElapsedMillis() const;
 };
 
 } // namespace aria2
 
-#endif // _D_DHT_MESSAGE_TRACKER_ENTRY_H_
+#endif // D_DHT_MESSAGE_TRACKER_ENTRY_H
