@@ -34,13 +34,11 @@
 /* copyright --> */
 #include "DHTSetup.h"
 
-#include <fstream>
 #include <algorithm>
 
 #include "LogFactory.h"
 #include "Logger.h"
 #include "util.h"
-#include "File.h"
 #include "DHTNode.h"
 #include "DHTConnectionImpl.h"
 #include "DHTRoutingTable.h"
@@ -70,16 +68,11 @@
 #include "RecoverableException.h"
 #include "a2functional.h"
 #include "DownloadEngine.h"
-#include "wallclock.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
-#include "FileEntry.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DHTSetup::DHTSetup():logger_(LogFactory::getInstance()) {}
+DHTSetup::DHTSetup() {}
 
 DHTSetup::~DHTSetup() {}
 
@@ -105,17 +98,15 @@ void DHTSetup::setup
       e->getOption()->get(family == AF_INET?PREF_DHT_FILE_PATH:
                           PREF_DHT_FILE_PATH6);
     try {
-      std::ifstream in(dhtFile.c_str(), std::ios::binary);
-      if(!in) {
-        throw DL_ABORT_EX("Could not open file");
-      }
-      deserializer.deserialize(in);
+      deserializer.deserialize(dhtFile);
       localNode = deserializer.getLocalNode();
     } catch(RecoverableException& e) {
-      logger_->error("Exception caught while loading DHT routing table from %s",
-                     e, dhtFile.c_str());
+      A2_LOG_ERROR_EX
+        (fmt("Exception caught while loading DHT routing table from %s",
+             dhtFile.c_str()),
+         e);
     }
-    if(localNode.isNull()) {
+    if(!localNode) {
       localNode.reset(new DHTNode());
     }
 
@@ -132,10 +123,8 @@ void DHTSetup::setup
       }
       localNode->setPort(port);
     }
-    if(logger_->debug()) {
-      logger_->debug("Initialized local node ID=%s",
-                     util::toHex(localNode->getID(), DHT_ID_LENGTH).c_str());
-    }
+    A2_LOG_DEBUG(fmt("Initialized local node ID=%s",
+                     util::toHex(localNode->getID(), DHT_ID_LENGTH).c_str()));
     SharedHandle<DHTRoutingTable> routingTable(new DHTRoutingTable(localNode));
 
     SharedHandle<DHTMessageFactoryImpl> factory
@@ -167,10 +156,10 @@ void DHTSetup::setup
     receiver->setRoutingTable(routingTable);
 
     taskFactory->setLocalNode(localNode);
-    taskFactory->setRoutingTable(routingTable);
-    taskFactory->setMessageDispatcher(dispatcher);
-    taskFactory->setMessageFactory(factory);
-    taskFactory->setTaskQueue(taskQueue);
+    taskFactory->setRoutingTable(routingTable.get());
+    taskFactory->setMessageDispatcher(dispatcher.get());
+    taskFactory->setMessageFactory(factory.get());
+    taskFactory->setTaskQueue(taskQueue.get());
     taskFactory->setTimeout(messageTimeout);
 
     routingTable->setTaskQueue(taskQueue);
@@ -179,11 +168,11 @@ void DHTSetup::setup
     peerAnnounceStorage->setTaskQueue(taskQueue);
     peerAnnounceStorage->setTaskFactory(taskFactory);
 
-    factory->setRoutingTable(routingTable);
-    factory->setConnection(connection);
-    factory->setMessageDispatcher(dispatcher);
-    factory->setPeerAnnounceStorage(peerAnnounceStorage);
-    factory->setTokenTracker(tokenTracker);
+    factory->setRoutingTable(routingTable.get());
+    factory->setConnection(connection.get());
+    factory->setMessageDispatcher(dispatcher.get());
+    factory->setPeerAnnounceStorage(peerAnnounceStorage.get());
+    factory->setTokenTracker(tokenTracker.get());
     factory->setLocalNode(localNode);
 
     // assign them into DHTRegistry
@@ -215,9 +204,7 @@ void DHTSetup::setup
           desnodes.begin(), eoi = desnodes.end(); i != eoi; ++i) {
       routingTable->addNode(*i);
     }
-    if(!desnodes.empty() &&
-       deserializer.getSerializedTime().
-       difference() >= DHT_BUCKET_REFRESH_INTERVAL) {
+    if(!desnodes.empty()) {
       SharedHandle<DHTBucketRefreshTask> task
         (static_pointer_cast<DHTBucketRefreshTask>
          (taskFactory->createBucketRefreshTask()));
@@ -247,7 +234,7 @@ void DHTSetup::setup
         tempCommands->push_back(command);
       }
     } else {
-      logger_->info("No DHT entry point specified.");
+      A2_LOG_INFO("No DHT entry point specified.");
     }
     {
       DHTInteractionCommand* command =
@@ -295,8 +282,9 @@ void DHTSetup::setup
     commands.insert(commands.end(), tempCommands->begin(), tempCommands->end());
     tempCommands->clear();
   } catch(RecoverableException& e) {
-    logger_->error("Exception caught while initializing DHT functionality."
-                   " DHT is disabled.", e);
+    A2_LOG_ERROR_EX(fmt("Exception caught while initializing DHT functionality."
+                        " DHT is disabled."),
+                    e);
     if(family == AF_INET) {
       DHTRegistry::clearData();
     } else {
