@@ -35,7 +35,7 @@
 #include "DefaultBtProgressInfoFile.h"
 
 #include <cstring>
-#include <fstream>
+#include <cstdio>
 
 #include "PieceStorage.h"
 #include "Piece.h"
@@ -54,6 +54,7 @@
 #include "fmt.h"
 #include "array_fun.h"
 #include "DownloadContext.h"
+#include "BufferedFile.h"
 #ifdef ENABLE_BITTORRENT
 # include "PeerStorage.h"
 # include "BtRuntime.h"
@@ -101,27 +102,30 @@ bool DefaultBtProgressInfoFile::isTorrentDownload()
 #endif // !ENABLE_BITTORRENT
 }
 
+#define WRITE_CHECK(fp, ptr, count)                                     \
+  if(fp.write((ptr), (count)) != (count)) {                             \
+    throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));   \
+  }
+
 // Since version 0001, Integers are saved in binary form, network byte order.
 void DefaultBtProgressInfoFile::save()
 {
   A2_LOG_INFO(fmt(MSG_SAVING_SEGMENT_FILE, filename_.c_str()));
   std::string filenameTemp = filename_+"__temp";
   {
-    std::ofstream o(filenameTemp.c_str(), std::ios::out|std::ios::binary);
-    if(!o) {
-      throw DL_ABORT_EX
-        (fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
+    BufferedFile fp(filenameTemp, BufferedFile::WRITE);
+    if(!fp) {
+      throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
     }
 #ifdef ENABLE_BITTORRENT
     bool torrentDownload = isTorrentDownload();
 #else // !ENABLE_BITTORRENT
     bool torrentDownload = false;
 #endif // !ENABLE_BITTORRENT
-
     // file version: 16 bits
     // values: '1'
     char version[] = { 0x00u, 0x01u };
-    o.write(version, sizeof(version));
+    WRITE_CHECK(fp, version, sizeof(version));
     // extension: 32 bits
     // If this is BitTorrent download, then 0x00000001
     // Otherwise, 0x00000000
@@ -130,33 +134,29 @@ void DefaultBtProgressInfoFile::save()
     if(torrentDownload) {
       extension[3] = 1;
     }
-    o.write(reinterpret_cast<const char*>(&extension), sizeof(extension));
+    WRITE_CHECK(fp, extension, sizeof(extension));
     if(torrentDownload) {
 #ifdef ENABLE_BITTORRENT
       // infoHashLength:
       // length: 32 bits
       const unsigned char* infoHash = bittorrent::getInfoHash(dctx_);
       uint32_t infoHashLengthNL = htonl(INFO_HASH_LENGTH);
-      o.write(reinterpret_cast<const char*>(&infoHashLengthNL),
-              sizeof(infoHashLengthNL));
+      WRITE_CHECK(fp, &infoHashLengthNL, sizeof(infoHashLengthNL));
       // infoHash:
-      o.write(reinterpret_cast<const char*>(infoHash), INFO_HASH_LENGTH);
+      WRITE_CHECK(fp, infoHash, INFO_HASH_LENGTH);
 #endif // ENABLE_BITTORRENT
     } else {
       // infoHashLength:
       // length: 32 bits
       uint32_t infoHashLength = 0;
-      o.write(reinterpret_cast<const char*>(&infoHashLength),
-              sizeof(infoHashLength));
+      WRITE_CHECK(fp, &infoHashLength, sizeof(infoHashLength));
     }
     // pieceLength: 32 bits
     uint32_t pieceLengthNL = htonl(dctx_->getPieceLength());
-    o.write(reinterpret_cast<const char*>(&pieceLengthNL),
-            sizeof(pieceLengthNL));
+    WRITE_CHECK(fp, &pieceLengthNL, sizeof(pieceLengthNL));
     // totalLength: 64 bits
     uint64_t totalLengthNL = hton64(dctx_->getTotalLength());
-    o.write(reinterpret_cast<const char*>(&totalLengthNL),
-            sizeof(totalLengthNL));
+    WRITE_CHECK(fp, &totalLengthNL, sizeof(totalLengthNL));
     // uploadLength: 64 bits
     uint64_t uploadLengthNL = 0;
 #ifdef ENABLE_BITTORRENT
@@ -165,20 +165,17 @@ void DefaultBtProgressInfoFile::save()
       uploadLengthNL = hton64(stat.getAllTimeUploadLength());
     }
 #endif // ENABLE_BITTORRENT
-    o.write(reinterpret_cast<const char*>(&uploadLengthNL),
-            sizeof(uploadLengthNL));
+    WRITE_CHECK(fp, &uploadLengthNL, sizeof(uploadLengthNL));
     // bitfieldLength: 32 bits
     uint32_t bitfieldLengthNL = htonl(pieceStorage_->getBitfieldLength());
-    o.write(reinterpret_cast<const char*>(&bitfieldLengthNL),
-            sizeof(bitfieldLengthNL));
+    WRITE_CHECK(fp, &bitfieldLengthNL, sizeof(bitfieldLengthNL));
     // bitfield
-    o.write(reinterpret_cast<const char*>(pieceStorage_->getBitfield()),
-            pieceStorage_->getBitfieldLength());
+    WRITE_CHECK(fp, pieceStorage_->getBitfield(),
+                pieceStorage_->getBitfieldLength());
     // the number of in-flight piece: 32 bits
     // TODO implement this
     uint32_t numInFlightPieceNL = htonl(pieceStorage_->countInFlightPiece());
-    o.write(reinterpret_cast<const char*>(&numInFlightPieceNL),
-            sizeof(numInFlightPieceNL));
+    WRITE_CHECK(fp, &numInFlightPieceNL, sizeof(numInFlightPieceNL));
     std::vector<SharedHandle<Piece> > inFlightPieces;
     inFlightPieces.reserve(pieceStorage_->countInFlightPiece());
     pieceStorage_->getInFlightPieces(inFlightPieces);
@@ -186,35 +183,25 @@ void DefaultBtProgressInfoFile::save()
           inFlightPieces.begin(), eoi = inFlightPieces.end();
         itr != eoi; ++itr) {
       uint32_t indexNL = htonl((*itr)->getIndex());
-      o.write(reinterpret_cast<const char*>(&indexNL), sizeof(indexNL));
+      WRITE_CHECK(fp, &indexNL, sizeof(indexNL));
       uint32_t lengthNL = htonl((*itr)->getLength());
-      o.write(reinterpret_cast<const char*>(&lengthNL), sizeof(lengthNL));
+      WRITE_CHECK(fp, &lengthNL, sizeof(lengthNL));
       uint32_t bitfieldLengthNL = htonl((*itr)->getBitfieldLength());
-      o.write(reinterpret_cast<const char*>(&bitfieldLengthNL),
-              sizeof(bitfieldLengthNL));
-      o.write(reinterpret_cast<const char*>((*itr)->getBitfield()),
-              (*itr)->getBitfieldLength());
+      WRITE_CHECK(fp, &bitfieldLengthNL, sizeof(bitfieldLengthNL));
+      WRITE_CHECK(fp, (*itr)->getBitfield(), (*itr)->getBitfieldLength());
     }
-    o.flush();
-    if(!o) {
-      throw DL_ABORT_EX
-        (fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
+    if(fp.close() == EOF) {
+      throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
     }
     A2_LOG_INFO(MSG_SAVED_SEGMENT_FILE);
   }
   if(!File(filenameTemp).renameTo(filename_)) {
-    throw DL_ABORT_EX
-      (fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
+    throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_WRITE, filename_.c_str()));
   }
 }
 
-#define CHECK_STREAM(in, length)                                        \
-  if(in.gcount() != length) {                                           \
-    throw DL_ABORT_EX(fmt("Failed to read segment file %s."             \
-                          " Unexpected EOF.",                           \
-                          filename_.c_str()));                          \
-  }                                                                     \
-  if(!in) {                                                             \
+#define READ_CHECK(fp, ptr, count)                                      \
+  if(fp.read((ptr), (count)) != (count)) {                              \
     throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_READ, filename_.c_str()));    \
   }
 
@@ -224,14 +211,12 @@ void DefaultBtProgressInfoFile::save()
 void DefaultBtProgressInfoFile::load() 
 {
   A2_LOG_INFO(fmt(MSG_LOADING_SEGMENT_FILE, filename_.c_str()));
-  std::ifstream in(filename_.c_str(), std::ios::in|std::ios::binary);
-  if(!in) {
-    throw DL_ABORT_EX
-      (fmt(EX_SEGMENT_FILE_READ, filename_.c_str()));
+  BufferedFile fp(filename_, BufferedFile::READ);
+  if(!fp) {
+    throw DL_ABORT_EX(fmt(EX_SEGMENT_FILE_READ, filename_.c_str()));
   }
   unsigned char versionBuf[2];
-  in.read(reinterpret_cast<char*>(versionBuf), sizeof(versionBuf));
-  CHECK_STREAM(in, sizeof(versionBuf));
+  READ_CHECK(fp, versionBuf, sizeof(versionBuf));
   std::string versionHex = util::toHex(versionBuf, sizeof(versionBuf));
   int version;
   if(DefaultBtProgressInfoFile::V0000 == versionHex) {
@@ -240,12 +225,10 @@ void DefaultBtProgressInfoFile::load()
     version = 1;
   } else {
     throw DL_ABORT_EX
-      (fmt("Unsupported ctrl file version: %s",
-           versionHex.c_str()));
+      (fmt("Unsupported ctrl file version: %s", versionHex.c_str()));
   }
   unsigned char extension[4];
-  in.read(reinterpret_cast<char*>(extension), sizeof(extension));
-  CHECK_STREAM(in, sizeof(extension));
+  READ_CHECK(fp, extension, sizeof(extension));
   bool infoHashCheckEnabled = false;
   if(extension[3]&1 && isTorrentDownload()) {
     infoHashCheckEnabled = true;
@@ -253,21 +236,18 @@ void DefaultBtProgressInfoFile::load()
   }
 
   uint32_t infoHashLength;
-  in.read(reinterpret_cast<char*>(&infoHashLength), sizeof(infoHashLength));
-  CHECK_STREAM(in, sizeof(infoHashLength));
+  READ_CHECK(fp, &infoHashLength, sizeof(infoHashLength));
   if(version >= 1) {
     infoHashLength = ntohl(infoHashLength);
   }
   if((infoHashLength < 0) ||
      ((infoHashLength == 0) && infoHashCheckEnabled)) {
-    throw DL_ABORT_EX
-      (fmt("Invalid info hash length: %d", infoHashLength));
+    throw DL_ABORT_EX(fmt("Invalid info hash length: %d", infoHashLength));
   }
   if(infoHashLength > 0) {
     array_ptr<unsigned char> savedInfoHash(new unsigned char[infoHashLength]);
-    in.read(reinterpret_cast<char*>
-            (static_cast<unsigned char*>(savedInfoHash)), infoHashLength);
-    CHECK_STREAM(in, static_cast<int>(infoHashLength));
+    READ_CHECK(fp, static_cast<unsigned char*>(savedInfoHash),
+               infoHashLength);
 #ifdef ENABLE_BITTORRENT
     if(infoHashCheckEnabled) {
       const unsigned char* infoHash = bittorrent::getInfoHash(dctx_);
@@ -284,15 +264,13 @@ void DefaultBtProgressInfoFile::load()
   }
 
   uint32_t pieceLength;
-  in.read(reinterpret_cast<char*>(&pieceLength), sizeof(pieceLength));
-  CHECK_STREAM(in, sizeof(pieceLength));
+  READ_CHECK(fp, &pieceLength, sizeof(pieceLength));
   if(version >= 1) {
     pieceLength = ntohl(pieceLength);
   }
 
   uint64_t totalLength;
-  in.read(reinterpret_cast<char*>(&totalLength), sizeof(totalLength));
-  CHECK_STREAM(in, sizeof(totalLength));
+  READ_CHECK(fp, &totalLength, sizeof(totalLength));
   if(version >= 1) {
     totalLength = ntoh64(totalLength);
   }
@@ -303,8 +281,7 @@ void DefaultBtProgressInfoFile::load()
            util::itos(totalLength).c_str()));
   }
   uint64_t uploadLength;
-  in.read(reinterpret_cast<char*>(&uploadLength), sizeof(uploadLength));
-  CHECK_STREAM(in, sizeof(uploadLength));
+  READ_CHECK(fp, &uploadLength, sizeof(uploadLength));
   if(version >= 1) {
     uploadLength = ntoh64(uploadLength);
   }
@@ -315,8 +292,7 @@ void DefaultBtProgressInfoFile::load()
 #endif // ENABLE_BITTORRENT
   // TODO implement the conversion mechanism between different piece length.
   uint32_t bitfieldLength;
-  in.read(reinterpret_cast<char*>(&bitfieldLength), sizeof(bitfieldLength));
-  CHECK_STREAM(in, sizeof(bitfieldLength));
+  READ_CHECK(fp, &bitfieldLength, sizeof(bitfieldLength));
   if(version >= 1) {
     bitfieldLength = ntohl(bitfieldLength);
   }
@@ -330,16 +306,13 @@ void DefaultBtProgressInfoFile::load()
   }
 
   array_ptr<unsigned char> savedBitfield(new unsigned char[bitfieldLength]);
-  in.read(reinterpret_cast<char*>
-          (static_cast<unsigned char*>(savedBitfield)), bitfieldLength);
-  CHECK_STREAM(in, static_cast<int>(bitfieldLength));
+  READ_CHECK(fp, static_cast<unsigned char*>(savedBitfield),
+             bitfieldLength);
   if(pieceLength == dctx_->getPieceLength()) {
     pieceStorage_->setBitfield(savedBitfield, bitfieldLength);
 
     uint32_t numInFlightPiece;
-    in.read(reinterpret_cast<char*>(&numInFlightPiece),
-            sizeof(numInFlightPiece));
-    CHECK_STREAM(in, sizeof(numInFlightPiece));
+    READ_CHECK(fp, &numInFlightPiece, sizeof(numInFlightPiece));
     if(version >= 1) {
       numInFlightPiece = ntohl(numInFlightPiece);
     }
@@ -347,30 +320,24 @@ void DefaultBtProgressInfoFile::load()
     inFlightPieces.reserve(numInFlightPiece);
     while(numInFlightPiece--) {
       uint32_t index;
-      in.read(reinterpret_cast<char*>(&index), sizeof(index));
-      CHECK_STREAM(in, sizeof(index));
+      READ_CHECK(fp, &index, sizeof(index));
       if(version >= 1) {
         index = ntohl(index);
       }
       if(!(index < dctx_->getNumPieces())) {
-        throw DL_ABORT_EX
-          (fmt("piece index out of range: %u", index));
+        throw DL_ABORT_EX(fmt("piece index out of range: %u", index));
       }
       uint32_t length;
-      in.read(reinterpret_cast<char*>(&length), sizeof(length));
-      CHECK_STREAM(in, sizeof(length));
+      READ_CHECK(fp, &length, sizeof(length));
       if(version >= 1) {
         length = ntohl(length);
       }
       if(!(length <=dctx_->getPieceLength())) {
-        throw DL_ABORT_EX
-          (fmt("piece length out of range: %u", length));
+        throw DL_ABORT_EX(fmt("piece length out of range: %u", length));
       }
       SharedHandle<Piece> piece(new Piece(index, length));
       uint32_t bitfieldLength;
-      in.read(reinterpret_cast<char*>(&bitfieldLength),
-              sizeof(bitfieldLength));
-      CHECK_STREAM(in, sizeof(bitfieldLength));
+      READ_CHECK(fp, &bitfieldLength, sizeof(bitfieldLength));
       if(version >= 1) {
         bitfieldLength = ntohl(bitfieldLength);
       }
@@ -383,9 +350,8 @@ void DefaultBtProgressInfoFile::load()
       }
       array_ptr<unsigned char> pieceBitfield
         (new unsigned char[bitfieldLength]);
-      in.read(reinterpret_cast<char*>
-              (static_cast<unsigned char*>(pieceBitfield)), bitfieldLength);
-      CHECK_STREAM(in, static_cast<int>(bitfieldLength));
+      READ_CHECK(fp, static_cast<unsigned char*>(pieceBitfield),
+                 bitfieldLength);
       piece->setBitfield(pieceBitfield, bitfieldLength);
 
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -399,9 +365,7 @@ void DefaultBtProgressInfoFile::load()
     pieceStorage_->addInFlightPiece(inFlightPieces);
   } else {
     uint32_t numInFlightPiece;
-    in.read(reinterpret_cast<char*>(&numInFlightPiece),
-            sizeof(numInFlightPiece));
-    CHECK_STREAM(in, sizeof(numInFlightPiece));
+    READ_CHECK(fp, &numInFlightPiece, sizeof(numInFlightPiece));
     if(version >= 1) {
       numInFlightPiece = ntohl(numInFlightPiece);
     }
