@@ -39,113 +39,142 @@
 #include "LogFactory.h"
 #include "Logger.h"
 #include "message.h"
-#include "a2time.h"
 #include "Peer.h"
 #include "BtRuntime.h"
 #include "BtSeederStateChoke.h"
 #include "BtLeecherStateChoke.h"
 #include "PieceStorage.h"
 #include "wallclock.h"
+#include "a2functional.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-static const int MAX_PEER_LIST_SIZE = 1024;
+namespace {
 
-DefaultPeerStorage::DefaultPeerStorage(const Option* option):
-  option(option),
-  logger(LogFactory::getInstance()),
-  removedPeerSessionDownloadLength(0),
-  removedPeerSessionUploadLength(0),
-  _seederStateChoke(new BtSeederStateChoke()),
-  _leecherStateChoke(new BtLeecherStateChoke()),
-  _lastTransferStatMapUpdated(0)
+const size_t MAX_PEER_LIST_SIZE = 1024;
+const size_t MAX_PEER_LIST_UPDATE = 100;
+
+} // namespace
+
+DefaultPeerStorage::DefaultPeerStorage()
+  : maxPeerListSize_(MAX_PEER_LIST_SIZE),
+    removedPeerSessionDownloadLength_(0),
+    removedPeerSessionUploadLength_(0),
+    seederStateChoke_(new BtSeederStateChoke()),
+    leecherStateChoke_(new BtLeecherStateChoke()),
+    lastTransferStatMapUpdated_(0)
 {}
 
 DefaultPeerStorage::~DefaultPeerStorage()
 {
-  delete _seederStateChoke;
-  delete _leecherStateChoke;
+  delete seederStateChoke_;
+  delete leecherStateChoke_;
 }
 
+namespace {
 class FindIdenticalPeer {
 private:
-  SharedHandle<Peer> _peer;
+  SharedHandle<Peer> peer_;
 public:
-  FindIdenticalPeer(const SharedHandle<Peer>& peer):_peer(peer) {}
+  FindIdenticalPeer(const SharedHandle<Peer>& peer):peer_(peer) {}
 
   bool operator()(const SharedHandle<Peer>& peer) const {
-    return (_peer == peer) ||
-      ((_peer->ipaddr == peer->ipaddr) && (_peer->port == peer->port));
+    return (*peer_ == *peer) ||
+      ((peer_->getIPAddress() == peer->getIPAddress()) &&
+       (peer_->getPort() == peer->getPort()));
   }
 };
+} // namespace
 
 bool DefaultPeerStorage::isPeerAlreadyAdded(const SharedHandle<Peer>& peer)
 {
-  return std::find_if(peers.begin(), peers.end(),
-                      FindIdenticalPeer(peer)) != peers.end();
-}
-
-static size_t calculateMaxPeerListSize(const SharedHandle<BtRuntime>& btRuntime)
-{
-  if(btRuntime.isNull()) {
-    return MAX_PEER_LIST_SIZE;
-  }
-  return btRuntime->getMaxPeers() == 0 ?
-    MAX_PEER_LIST_SIZE :
-    btRuntime->getMaxPeers()+(btRuntime->getMaxPeers() >> 2);
+  return std::find_if(peers_.begin(), peers_.end(),
+                      FindIdenticalPeer(peer)) != peers_.end();
 }
 
 bool DefaultPeerStorage::addPeer(const SharedHandle<Peer>& peer) {
   if(isPeerAlreadyAdded(peer)) {
-    if(logger->debug()) {
-      logger->debug("Adding %s:%u is rejected because it has been already"
-                    " added.", peer->ipaddr.c_str(), peer->port);
-    }
+    A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it has been already"
+                     " added.",
+                     peer->getIPAddress().c_str(), peer->getPort()));
     return false;
   }
-  size_t maxPeerListSize = calculateMaxPeerListSize(_btRuntime);
-  if(peers.size() >= maxPeerListSize) {
-    deleteUnusedPeer(peers.size()-maxPeerListSize+1);
+  const size_t peerListSize = peers_.size();
+  if(peerListSize >= maxPeerListSize_) {
+    deleteUnusedPeer(peerListSize-maxPeerListSize_+1);
   }
-  peers.push_front(peer);
+  peers_.push_front(peer);
+  A2_LOG_DEBUG(fmt("Now peer list contains %lu peers",
+                   static_cast<unsigned long>(peers_.size())));
   return true;
 }
 
 void DefaultPeerStorage::addPeer(const std::vector<SharedHandle<Peer> >& peers)
 {
+  size_t added = 0;
+  size_t addMax = std::min(maxPeerListSize_, MAX_PEER_LIST_UPDATE);
   for(std::vector<SharedHandle<Peer> >::const_iterator itr = peers.begin(),
-        eoi = peers.end(); itr != eoi; ++itr) {
+        eoi = peers.end(); itr != eoi && added < addMax; ++itr) {
     const SharedHandle<Peer>& peer = *itr;
-    if(addPeer(peer)) {
-      if(logger->debug()) {
-        logger->debug(MSG_ADDING_PEER, peer->ipaddr.c_str(), peer->port);
-      }
+    if(isPeerAlreadyAdded(peer)) {
+      A2_LOG_DEBUG(fmt("Adding %s:%u is rejected because it has been already"
+                       " added.",
+                       peer->getIPAddress().c_str(), peer->getPort()));
+      continue;
+    } else {
+      A2_LOG_DEBUG(fmt(MSG_ADDING_PEER,
+                       peer->getIPAddress().c_str(), peer->getPort()));
     }
-  }  
+    peers_.push_front(peer);
+    ++added;
+  }
+  const size_t peerListSize = peers_.size();
+  if(peerListSize >= maxPeerListSize_) {
+    deleteUnusedPeer(peerListSize-maxPeerListSize_);
+  }
+  A2_LOG_DEBUG(fmt("Now peer list contains %lu peers",
+                   static_cast<unsigned long>(peers_.size())));
+}
+
+void DefaultPeerStorage::addDroppedPeer(const SharedHandle<Peer>& peer)
+{
+  droppedPeers_.push_front(peer);
+  if(droppedPeers_.size() > 50) {
+    droppedPeers_.pop_back();
+  }
 }
 
 const std::deque<SharedHandle<Peer> >& DefaultPeerStorage::getPeers()
 {
-  return peers;
+  return peers_;
 }
 
+const std::deque<SharedHandle<Peer> >& DefaultPeerStorage::getDroppedPeers()
+{
+  return droppedPeers_;
+}
+
+namespace {
 class FindFinePeer {
 public:
   bool operator()(const SharedHandle<Peer>& peer) const {
     return peer->unused() && peer->isGood();
   }
 };
+} // namespace
 
 SharedHandle<Peer> DefaultPeerStorage::getUnusedPeer() {
   std::deque<SharedHandle<Peer> >::const_iterator itr =
-    std::find_if(peers.begin(), peers.end(), FindFinePeer());
-  if(itr == peers.end()) {
+    std::find_if(peers_.begin(), peers_.end(), FindFinePeer());
+  if(itr == peers_.end()) {
     return SharedHandle<Peer>();
   } else {
     return *itr;
   }
 }
 
+namespace {
 class FindPeer {
 private:
   std::string ipaddr;
@@ -155,15 +184,16 @@ public:
     ipaddr(ipaddr), port(port) {}
 
   bool operator()(const SharedHandle<Peer>& peer) const {
-    return ipaddr == peer->ipaddr && port == peer->port;
+    return ipaddr == peer->getIPAddress() && port == peer->getPort();
   }
 };
+} // namespace
 
 SharedHandle<Peer> DefaultPeerStorage::getPeer(const std::string& ipaddr,
                                                uint16_t port) const {
   std::deque<SharedHandle<Peer> >::const_iterator itr =
-    std::find_if(peers.begin(), peers.end(), FindPeer(ipaddr, port));
-  if(itr == peers.end()) {
+    std::find_if(peers_.begin(), peers_.end(), FindPeer(ipaddr, port));
+  if(itr == peers_.end()) {
     return SharedHandle<Peer>();
   } else {
     return *itr;
@@ -171,35 +201,38 @@ SharedHandle<Peer> DefaultPeerStorage::getPeer(const std::string& ipaddr,
 }
 
 size_t DefaultPeerStorage::countPeer() const {
-  return peers.size();
+  return peers_.size();
 }
 
 bool DefaultPeerStorage::isPeerAvailable() {
-  return !getUnusedPeer().isNull();
+  return getUnusedPeer();
 }
 
+namespace {
 class CollectActivePeer {
 private:
-  std::vector<SharedHandle<Peer> >& _activePeers;
+  std::vector<SharedHandle<Peer> >& activePeers_;
 public:
   CollectActivePeer(std::vector<SharedHandle<Peer> >& activePeers):
-    _activePeers(activePeers) {}
+    activePeers_(activePeers) {}
 
   void operator()(const SharedHandle<Peer>& peer)
   {
     if(peer->isActive()) {
-      _activePeers.push_back(peer);
+      activePeers_.push_back(peer);
     }
   }
 };
+} // namespace
 
 void DefaultPeerStorage::getActivePeers
 (std::vector<SharedHandle<Peer> >& activePeers)
 {
-  std::for_each(peers.begin(), peers.end(), CollectActivePeer(activePeers));
+  std::for_each(peers_.begin(), peers_.end(), CollectActivePeer(activePeers));
 }
 
-static TransferStat calculateStatFor(const SharedHandle<Peer>& peer)
+namespace {
+TransferStat calculateStatFor(const SharedHandle<Peer>& peer)
 {
   TransferStat s;
   s.downloadSpeed = peer->calculateDownloadSpeed();
@@ -208,16 +241,16 @@ static TransferStat calculateStatFor(const SharedHandle<Peer>& peer)
   s.sessionUploadLength = peer->getSessionUploadLength();
   return s;
 }
+} // namespace
 
 TransferStat DefaultPeerStorage::calculateStat()
 {
   TransferStat stat;
-  if(_lastTransferStatMapUpdated.differenceInMillis(global::wallclock) >= 250) {
-    if(logger->debug()) {
-      logger->debug("Updating TransferStat of PeerStorage");
-    }
-    _lastTransferStatMapUpdated = global::wallclock;
-    _peerTransferStatMap.clear();
+  if(lastTransferStatMapUpdated_.differenceInMillis(global::wallclock())+
+     A2_DELTA_MILLIS >= 250) {
+    A2_LOG_DEBUG("Updating TransferStat of PeerStorage");
+    lastTransferStatMapUpdated_ = global::wallclock();
+    peerTransferStatMap_.clear();
     std::vector<SharedHandle<Peer> > activePeers;
     getActivePeers(activePeers);
     for(std::vector<SharedHandle<Peer> >::const_iterator i =
@@ -228,53 +261,59 @@ TransferStat DefaultPeerStorage::calculateStat()
       s.sessionDownloadLength = (*i)->getSessionDownloadLength();
       s.sessionUploadLength = (*i)->getSessionUploadLength();
 
-      _peerTransferStatMap[(*i)->getID()] = calculateStatFor(*i);
+      peerTransferStatMap_[(*i)->getID()] = calculateStatFor(*i);
       stat += s;
     }
-    _cachedTransferStat = stat;
+    cachedTransferStat_ = stat;
   } else {
-    stat = _cachedTransferStat;
+    stat = cachedTransferStat_;
   }
-  stat.sessionDownloadLength += removedPeerSessionDownloadLength;
-  stat.sessionUploadLength += removedPeerSessionUploadLength;
-  stat.setAllTimeUploadLength(_btRuntime->getUploadLengthAtStartup()+
+  stat.sessionDownloadLength += removedPeerSessionDownloadLength_;
+  stat.sessionUploadLength += removedPeerSessionUploadLength_;
+  stat.setAllTimeUploadLength(btRuntime_->getUploadLengthAtStartup()+
                               stat.getSessionUploadLength());
   return stat;
 }
 
 void DefaultPeerStorage::updateTransferStatFor(const SharedHandle<Peer>& peer)
 {
-  if(logger->debug()) {
-    logger->debug("Updating TransferStat for peer %s", peer->getID().c_str());
-  }
+  A2_LOG_DEBUG(fmt("Updating TransferStat for peer %s", peer->getID().c_str()));
   std::map<std::string, TransferStat>::iterator itr =
-    _peerTransferStatMap.find(peer->getID());
-  assert(itr != _peerTransferStatMap.end());
-  _cachedTransferStat -= (*itr).second;
+    peerTransferStatMap_.find(peer->getID());
+  if(itr == peerTransferStatMap_.end()) {
+    return;
+  }
+  cachedTransferStat_ -= (*itr).second;
   TransferStat s = calculateStatFor(peer);
-  _cachedTransferStat += s;
+  cachedTransferStat_ += s;
   (*itr).second = s;
 }
 
 TransferStat DefaultPeerStorage::getTransferStatFor
 (const SharedHandle<Peer>& peer)
 {
-  return _peerTransferStatMap[peer->getID()];
+  std::map<std::string, TransferStat>::const_iterator itr =
+    peerTransferStatMap_.find(peer->getID());
+  if(itr == peerTransferStatMap_.end()) {
+    return TransferStat();
+  } else {
+    return (*itr).second;
+  }
 }
 
 void DefaultPeerStorage::deleteUnusedPeer(size_t delSize) {
   std::deque<SharedHandle<Peer> > temp;
   for(std::deque<SharedHandle<Peer> >::const_reverse_iterator itr =
-        peers.rbegin(), eoi = peers.rend(); itr != eoi; ++itr) {
+        peers_.rbegin(), eoi = peers_.rend(); itr != eoi; ++itr) {
     const SharedHandle<Peer>& p = *itr;
     if(p->unused() && delSize > 0) {
       onErasingPeer(p);
-      delSize--;
+      --delSize;
     } else {
       temp.push_front(p);
     }
   }
-  peers = temp;
+  peers_.swap(temp);
 }
 
 void DefaultPeerStorage::onErasingPeer(const SharedHandle<Peer>& peer) {}
@@ -283,10 +322,14 @@ void DefaultPeerStorage::onReturningPeer(const SharedHandle<Peer>& peer)
 {
   if(peer->isActive()) {
     TransferStat removedStat(calculateStatFor(peer));
-    removedPeerSessionDownloadLength += removedStat.getSessionDownloadLength();
-    removedPeerSessionUploadLength += removedStat.getSessionUploadLength();
-    _cachedTransferStat -= removedStat;
+    removedPeerSessionDownloadLength_ += removedStat.getSessionDownloadLength();
+    removedPeerSessionUploadLength_ += removedStat.getSessionUploadLength();
+    cachedTransferStat_ -= removedStat;
 
+    if(peer->isDisconnectedGracefully() && !peer->isIncomingPeer()) {
+      peer->startBadCondition();
+      addDroppedPeer(peer);
+    }
     // Execute choking algorithm if unchoked and interested peer is
     // disconnected.
     if(!peer->amChoking() && peer->peerInterested()) {
@@ -298,14 +341,12 @@ void DefaultPeerStorage::onReturningPeer(const SharedHandle<Peer>& peer)
 void DefaultPeerStorage::returnPeer(const SharedHandle<Peer>& peer)
 {
   std::deque<SharedHandle<Peer> >::iterator itr =
-    std::find(peers.begin(), peers.end(), peer);
-  if(itr == peers.end()) {
-    if(logger->debug()) {
-      logger->debug("Cannot find peer %s:%u in PeerStorage.",
-                    peer->ipaddr.c_str(), peer->port);
-    }
+    std::find_if(peers_.begin(), peers_.end(), derefEqual(peer));
+  if(itr == peers_.end()) {
+    A2_LOG_DEBUG(fmt("Cannot find peer %s:%u in PeerStorage.",
+                     peer->getIPAddress().c_str(), peer->getPort()));
   } else {
-    peers.erase(itr);
+    peers_.erase(itr);
 
     onReturningPeer(peer);
     onErasingPeer(peer);
@@ -315,12 +356,12 @@ void DefaultPeerStorage::returnPeer(const SharedHandle<Peer>& peer)
 bool DefaultPeerStorage::chokeRoundIntervalElapsed()
 {
   const time_t CHOKE_ROUND_INTERVAL = 10;
-  if(_pieceStorage->downloadFinished()) {
-    return _seederStateChoke->getLastRound().
-      difference(global::wallclock) >= CHOKE_ROUND_INTERVAL;
+  if(pieceStorage_->downloadFinished()) {
+    return seederStateChoke_->getLastRound().
+      difference(global::wallclock()) >= CHOKE_ROUND_INTERVAL;
   } else {
-    return _leecherStateChoke->getLastRound().
-      difference(global::wallclock) >= CHOKE_ROUND_INTERVAL;
+    return leecherStateChoke_->getLastRound().
+      difference(global::wallclock()) >= CHOKE_ROUND_INTERVAL;
   }
 }
 
@@ -328,21 +369,21 @@ void DefaultPeerStorage::executeChoke()
 {
   std::vector<SharedHandle<Peer> > activePeers;
   getActivePeers(activePeers);
-  if(_pieceStorage->downloadFinished()) {
-    return _seederStateChoke->executeChoke(activePeers);
+  if(pieceStorage_->downloadFinished()) {
+    return seederStateChoke_->executeChoke(activePeers);
   } else {
-    return _leecherStateChoke->executeChoke(activePeers);
+    return leecherStateChoke_->executeChoke(activePeers);
   }
 }
 
 void DefaultPeerStorage::setPieceStorage(const SharedHandle<PieceStorage>& ps)
 {
-  _pieceStorage = ps;
+  pieceStorage_ = ps;
 }
 
 void DefaultPeerStorage::setBtRuntime(const SharedHandle<BtRuntime>& btRuntime)
 {
-  _btRuntime = btRuntime;
+  btRuntime_ = btRuntime;
 }
 
 } // namespace aria2
