@@ -58,6 +58,7 @@
 #include "RarestPieceSelector.h"
 #include "DefaultStreamPieceSelector.h"
 #include "InorderStreamPieceSelector.h"
+#include "GeomStreamPieceSelector.h"
 #include "array_fun.h"
 #include "PieceStatMan.h"
 #include "wallclock.h"
@@ -83,11 +84,11 @@ DefaultPieceStorage::DefaultPieceStorage
   const std::string& pieceSelectorOpt =
     option_->get(PREF_STREAM_PIECE_SELECTOR);
   if(pieceSelectorOpt.empty() || pieceSelectorOpt == A2_V_DEFAULT) {
-    streamPieceSelector_ = SharedHandle<StreamPieceSelector>
-      (new DefaultStreamPieceSelector(bitfieldMan_));
+    streamPieceSelector_.reset(new DefaultStreamPieceSelector(bitfieldMan_));
   } else if(pieceSelectorOpt == V_INORDER) {
-    streamPieceSelector_ = SharedHandle<StreamPieceSelector>
-      (new InorderStreamPieceSelector(bitfieldMan_));
+    streamPieceSelector_.reset(new InorderStreamPieceSelector(bitfieldMan_));
+  } else if(pieceSelectorOpt == A2_V_GEOM) {
+    streamPieceSelector_.reset(new GeomStreamPieceSelector(bitfieldMan_, 1.5));
   }
 }
 
@@ -655,7 +656,7 @@ size_t DefaultPieceStorage::getPieceLength(size_t index)
 
 void DefaultPieceStorage::advertisePiece(cuid_t cuid, size_t index)
 {
-  HaveEntry entry(cuid, index, global::wallclock);
+  HaveEntry entry(cuid, index, global::wallclock());
   haves_.push_front(entry);
 }
 
@@ -686,7 +687,7 @@ public:
   FindElapsedHave(time_t elapsed):elapsed(elapsed) {}
 
   bool operator()(const HaveEntry& have) {
-    if(have.getRegisteredTime().difference(global::wallclock) >= elapsed) {
+    if(have.getRegisteredTime().difference(global::wallclock()) >= elapsed) {
       return true;
     } else {
       return false;
@@ -809,4 +810,9 @@ size_t DefaultPieceStorage::getNextUsedIndex(size_t index)
   return bitfieldMan_->countBlock();
 }
 
+void DefaultPieceStorage::onDownloadIncomplete()
+{
+  streamPieceSelector_->onBitfieldInit();
+}
+
 } // namespace aria2
