@@ -45,7 +45,6 @@
 namespace aria2 {
 
 class FileEntry;
-class Logger;
 class FileAllocationIterator;
 
 class DiskAdaptor:public BinaryStream {
@@ -53,16 +52,9 @@ private:
   std::vector<SharedHandle<FileEntry> > fileEntries_;
 
   bool fallocate_;
-
-  Logger* logger_;
-protected:
-  Logger* getLogger() const
-  {
-    return logger_;
-  }
 public:
   DiskAdaptor();
-  virtual ~DiskAdaptor() {}
+  virtual ~DiskAdaptor();
 
   virtual void openFile() = 0;
 
@@ -89,10 +81,6 @@ public:
 
   virtual SharedHandle<FileAllocationIterator> fileAllocationIterator() = 0;
 
-  virtual void enableDirectIO() {}
-
-  virtual void disableDirectIO() {}
-
   virtual void enableReadOnly() {}
 
   virtual void disableReadOnly() {}
