@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DISK_WRITER_H_
-#define _D_DISK_WRITER_H_
+#ifndef D_DISK_WRITER_H
+#define D_DISK_WRITER_H
 
 #include "BinaryStream.h"
 
@@ -70,12 +70,6 @@ public:
   // Returns file length
   virtual uint64_t size() = 0;
 
-  virtual void enableDirectIO() = 0;
-
-  virtual void disableDirectIO() = 0;
-
-  virtual void allowDirectIO() {}
-
   // Enables read-only mode. After this call, openExistingFile() opens
   // file in read-only mode. This is an optional functionality. The
   // default implementation is do nothing.
@@ -91,4 +85,4 @@ typedef SharedHandle<DiskWriter> DiskWriterHandle;
 
 } // namespace aria2
 
-#endif // _D_DISK_WRITER_H_
+#endif // D_DISK_WRITER_H
