@@ -65,6 +65,7 @@
 #include "BtProgressInfoFile.h"
 #include "DownloadContext.h"
 #include "fmt.h"
+#include "wallclock.h"
 #ifdef ENABLE_BITTORRENT
 # include "BtRegistry.h"
 #endif // ENABLE_BITTORRENT
@@ -73,10 +74,6 @@ namespace aria2 {
 
 namespace global {
 
-// Global clock, this clock is reseted before executeCommand() call to
-// reduce the call gettimeofday() system call.
-Timer wallclock;
-
 // 0 ... running
 // 1 ... stop signal detected
 // 2 ... stop signal processed by DownloadEngine
@@ -95,6 +92,9 @@ DownloadEngine::DownloadEngine(const SharedHandle<EventPoll>& eventPoll)
 #ifdef ENABLE_BITTORRENT
     btRegistry_(new BtRegistry()),
 #endif // ENABLE_BITTORRENT
+#ifdef HAVE_ARES_ADDR_NODE
+    asyncDNSServers_(0),
+#endif // HAVE_ARES_ADDR_NODE
     dnsCache_(new DNSCache())
 {
   unsigned char sessionId[20];
@@ -104,6 +104,9 @@ DownloadEngine::DownloadEngine(const SharedHandle<EventPoll>& eventPoll)
 
 DownloadEngine::~DownloadEngine() {
   cleanQueue();
+#ifdef HAVE_ARES_ADDR_NODE
+  setAsyncDNSServers(0);
+#endif // HAVE_ARES_ADDR_NODE
 }
 
 void DownloadEngine::cleanQueue() {
@@ -140,10 +143,12 @@ void DownloadEngine::run()
   Timer cp;
   cp.reset(0);
   while(!commands_.empty() || !routineCommands_.empty()) {
-    global::wallclock.reset();
-    if(cp.differenceInMillis(global::wallclock) >= refreshInterval_) {
+    global::wallclock().reset();
+    calculateStatistics();
+    if(cp.differenceInMillis(global::wallclock())+A2_DELTA_MILLIS >=
+       refreshInterval_) {
       refreshInterval_ = DEFAULT_REFRESH_INTERVAL;
-      cp = global::wallclock;
+      cp = global::wallclock();
       executeCommand(commands_, Command::STATUS_ALL);
     } else {
       executeCommand(commands_, Command::STATUS_ACTIVE);
@@ -154,7 +159,6 @@ void DownloadEngine::run()
       waitData();
     }
     noWait_ = false;
-    calculateStatistics();
   }
   onEndOfRun();
 }
@@ -165,8 +169,9 @@ void DownloadEngine::waitData()
   if(noWait_) {
     tv.tv_sec = tv.tv_usec = 0;
   } else {
-    tv.tv_sec = 0;
-    tv.tv_usec = refreshInterval_*1000;
+    lldiv_t qr = lldiv(refreshInterval_*1000, 1000000);
+    tv.tv_sec = qr.quot;
+    tv.tv_usec = qr.rem;
   }
   eventPoll_->poll(tv);
 }
@@ -280,10 +285,10 @@ void DownloadEngine::poolSocket(const std::string& key,
   std::multimap<std::string, SocketPoolEntry>::value_type p(key, entry);
   socketPool_.insert(p);
 
-  if(lastSocketPoolScan_.difference(global::wallclock) >= 60) {
+  if(lastSocketPoolScan_.difference(global::wallclock()) >= 60) {
     std::multimap<std::string, SocketPoolEntry> newPool;
     A2_LOG_DEBUG("Scaning SocketPool and erasing timed out entry.");
-    lastSocketPoolScan_ = global::wallclock;
+    lastSocketPoolScan_ = global::wallclock();
     for(std::multimap<std::string, SocketPoolEntry>::iterator i =
           socketPool_.begin(), eoi = socketPool_.end(); i != eoi; ++i) {
       if(!(*i).second.isTimeout()) {
@@ -489,7 +494,7 @@ DownloadEngine::SocketPoolEntry::~SocketPoolEntry() {}
 
 bool DownloadEngine::SocketPoolEntry::isTimeout() const
 {
-  return registeredTime_.difference(global::wallclock) >= timeout_;
+  return registeredTime_.difference(global::wallclock()) >= timeout_;
 }
 
 cuid_t DownloadEngine::newCUID()
@@ -560,4 +565,17 @@ void DownloadEngine::setCheckIntegrityMan
   checkIntegrityMan_ = ciman;
 }
 
+#ifdef HAVE_ARES_ADDR_NODE
+void DownloadEngine::setAsyncDNSServers(ares_addr_node* asyncDNSServers)
+{
+  ares_addr_node* node = asyncDNSServers_;
+  while(node) {
+    ares_addr_node* next = node->next;
+    delete node;
+    node = next;
+  }
+  asyncDNSServers_ = asyncDNSServers;
+}
+#endif // HAVE_ARES_ADDR_NODE
+
 } // namespace aria2
