@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -44,90 +44,91 @@ namespace aria2 {
 
 #ifdef ENABLE_METALINK
 
-MemoryBufferPreDownloadHandlerHandle
-DownloadHandlerFactory::_metalinkPreDownloadHandler;
+SharedHandle<MemoryBufferPreDownloadHandler>
+DownloadHandlerFactory::metalinkPreDownloadHandler_;
 
-MetalinkPostDownloadHandlerHandle
-DownloadHandlerFactory::_metalinkPostDownloadHandler;
+SharedHandle<MetalinkPostDownloadHandler>
+DownloadHandlerFactory::metalinkPostDownloadHandler_;
 
 #endif // ENABLE_METALINK
 
 #ifdef ENABLE_BITTORRENT
 
-MemoryBufferPreDownloadHandlerHandle
-DownloadHandlerFactory::_btPreDownloadHandler;
+SharedHandle<MemoryBufferPreDownloadHandler>
+DownloadHandlerFactory::btPreDownloadHandler_;
 
-BtPostDownloadHandlerHandle
-DownloadHandlerFactory::_btPostDownloadHandler;
+SharedHandle<BtPostDownloadHandler>
+DownloadHandlerFactory::btPostDownloadHandler_;
 
 SharedHandle<UTMetadataPostDownloadHandler>
-DownloadHandlerFactory::_btMetadataPostDownloadHandler;
+DownloadHandlerFactory::btMetadataPostDownloadHandler_;
 #endif // ENABLE_BITTORRENT
 
 #ifdef ENABLE_METALINK
 
-MemoryBufferPreDownloadHandlerHandle
+SharedHandle<MemoryBufferPreDownloadHandler>
 DownloadHandlerFactory::getMetalinkPreDownloadHandler()
 {
-  if(_metalinkPreDownloadHandler.isNull()) {
-    _metalinkPreDownloadHandler.reset(new MemoryBufferPreDownloadHandler());
+  if(!metalinkPreDownloadHandler_) {
+    metalinkPreDownloadHandler_.reset(new MemoryBufferPreDownloadHandler());
 
     RequestGroupCriteriaHandle criteria
       (new ContentTypeRequestGroupCriteria
        (DownloadHandlerConstants::getMetalinkContentTypes().begin(),
-	DownloadHandlerConstants::getMetalinkContentTypes().end(),
-	DownloadHandlerConstants::getMetalinkExtensions().begin(),
-	DownloadHandlerConstants::getMetalinkExtensions().end()));
-    _metalinkPreDownloadHandler->setCriteria(criteria);
+        DownloadHandlerConstants::getMetalinkContentTypes().end(),
+        DownloadHandlerConstants::getMetalinkExtensions().begin(),
+        DownloadHandlerConstants::getMetalinkExtensions().end()));
+    metalinkPreDownloadHandler_->setCriteria(criteria);
   }
-  return _metalinkPreDownloadHandler;
+  return metalinkPreDownloadHandler_;
 }
 
-MetalinkPostDownloadHandlerHandle
+SharedHandle<MetalinkPostDownloadHandler>
 DownloadHandlerFactory::getMetalinkPostDownloadHandler()
 {
-  if(_metalinkPostDownloadHandler.isNull()) {
-    _metalinkPostDownloadHandler.reset(new MetalinkPostDownloadHandler());
+  if(!metalinkPostDownloadHandler_) {
+    metalinkPostDownloadHandler_.reset(new MetalinkPostDownloadHandler());
   }
-  return _metalinkPostDownloadHandler;
+  return metalinkPostDownloadHandler_;
 }
 
 #endif // ENABLE_METALINK
 
 #ifdef ENABLE_BITTORRENT
 
-MemoryBufferPreDownloadHandlerHandle
+SharedHandle<MemoryBufferPreDownloadHandler>
 DownloadHandlerFactory::getBtPreDownloadHandler()
 {
-  if(_btPreDownloadHandler.isNull()) {
-    _btPreDownloadHandler.reset(new MemoryBufferPreDownloadHandler());
+  if(!btPreDownloadHandler_) {
+    btPreDownloadHandler_.reset(new MemoryBufferPreDownloadHandler());
 
     RequestGroupCriteriaHandle criteria
       (new ContentTypeRequestGroupCriteria
        (DownloadHandlerConstants::getBtContentTypes().begin(),
-	DownloadHandlerConstants::getBtContentTypes().end(),
-	DownloadHandlerConstants::getBtExtensions().begin(),
-	DownloadHandlerConstants::getBtExtensions().end()));
-    _btPreDownloadHandler->setCriteria(criteria);
+        DownloadHandlerConstants::getBtContentTypes().end(),
+        DownloadHandlerConstants::getBtExtensions().begin(),
+        DownloadHandlerConstants::getBtExtensions().end()));
+    btPreDownloadHandler_->setCriteria(criteria);
   }
-  return _btPreDownloadHandler;
+  return btPreDownloadHandler_;
 }
 
-BtPostDownloadHandlerHandle DownloadHandlerFactory::getBtPostDownloadHandler()
+SharedHandle<BtPostDownloadHandler>
+DownloadHandlerFactory::getBtPostDownloadHandler()
 {
-  if(_btPostDownloadHandler.isNull()) {
-    _btPostDownloadHandler.reset(new BtPostDownloadHandler());
+  if(!btPostDownloadHandler_) {
+    btPostDownloadHandler_.reset(new BtPostDownloadHandler());
   }
-  return _btPostDownloadHandler;
+  return btPostDownloadHandler_;
 }
 
 SharedHandle<UTMetadataPostDownloadHandler>
 DownloadHandlerFactory::getUTMetadataPostDownloadHandler()
 {
-  if(_btMetadataPostDownloadHandler.isNull()) {
-    _btMetadataPostDownloadHandler.reset(new UTMetadataPostDownloadHandler());
+  if(!btMetadataPostDownloadHandler_) {
+    btMetadataPostDownloadHandler_.reset(new UTMetadataPostDownloadHandler());
   }
-  return _btMetadataPostDownloadHandler;
+  return btMetadataPostDownloadHandler_;
 }
 
 #endif // ENABLE_BITTORRENT
