@@ -34,8 +34,7 @@
 /* copyright --> */
 #include "ExpatMetalinkProcessor.h"
 
-#include <iostream>
-#include <fstream>
+#include <cstdio>
 
 #include "DefaultDiskWriter.h"
 #include "MetalinkParserStateMachine.h"
@@ -46,6 +45,9 @@
 #include "DlAbortEx.h"
 #include "MetalinkParserState.h"
 #include "A2STR.h"
+#include "error_code.h"
+#include "BufferedFile.h"
+#include "a2functional.h"
 
 namespace aria2 {
 
@@ -158,62 +160,77 @@ namespace {
 void checkError(XML_Parser parser)
 {
   if(XML_Parse(parser, 0, 0, 1) == XML_STATUS_ERROR) {
-    throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
+    throw DL_ABORT_EX2(MSG_CANNOT_PARSE_METALINK,
+                       error_code::METALINK_PARSE_ERROR);
   }
   SessionData* sessionData =
     reinterpret_cast<SessionData*>(XML_GetUserData(parser));
   const SharedHandle<MetalinkParserStateMachine>& stm = sessionData->stm_;
   if(!stm->finished()) {
-    throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
+    throw DL_ABORT_EX2(MSG_CANNOT_PARSE_METALINK,
+                       error_code::METALINK_PARSE_ERROR);
   }
   if(!stm->getErrors().empty()) {
-    throw DL_ABORT_EX(stm->getErrorString());
+    throw DL_ABORT_EX2(stm->getErrorString(),
+                       error_code::METALINK_PARSE_ERROR);
   }
 }
 } // namespace
 
 SharedHandle<Metalinker>
-MetalinkProcessor::parseFile(const std::string& filename)
+MetalinkProcessor::parseFile
+(const std::string& filename,
+ const std::string& baseUri)
 {
+  BufferedFile* fp = 0;
+  auto_delete_d<BufferedFile*> deleter(fp);
   if(filename == DEV_STDIN) {
-    return parseFile(std::cin);
+    fp = new BufferedFile(stdin);
+    return parseFile(*fp);
   } else {
-    std::ifstream infile(filename.c_str(), std::ios::binary);
-    return parseFile(infile);
+    fp = new BufferedFile(filename, BufferedFile::READ);
+    return parseFile(*fp, baseUri);
   }
 }
 
 SharedHandle<Metalinker>
-MetalinkProcessor::parseFile(std::istream& stream)
+MetalinkProcessor::parseFile
+(BufferedFile& fp, const std::string& baseUri)
 {
   stm_.reset(new MetalinkParserStateMachine());
+  stm_->setBaseUri(baseUri);
   char buf[4096];
 
   SharedHandle<SessionData> sessionData(new SessionData(stm_));
   XML_Parser parser = createParser(sessionData);
   auto_delete<XML_Parser> deleter(parser, XML_ParserFree);
-  while(stream) {
-    stream.read(buf, sizeof(buf));
-    if(XML_Parse(parser, buf, stream.gcount(), 0) == XML_STATUS_ERROR) {
-      throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
+  while(1) {
+    size_t res = fp.read(buf, sizeof(buf));
+    if(XML_Parse(parser, buf, res, 0) == XML_STATUS_ERROR) {
+      throw DL_ABORT_EX2(MSG_CANNOT_PARSE_METALINK,
+                         error_code::METALINK_PARSE_ERROR);
+    }
+    if(res < sizeof(buf)) {
+      break;
     }
-  }
-  if(stream.bad()) {
-    throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
   }
   checkError(parser);
   return stm_->getResult();
 }
          
 SharedHandle<Metalinker>
-MetalinkProcessor::parseFromBinaryStream(const SharedHandle<BinaryStream>& binaryStream)
+MetalinkProcessor::parseFromBinaryStream
+(const SharedHandle<BinaryStream>& binaryStream,
+ const std::string& baseUri)
 {
   stm_.reset(new MetalinkParserStateMachine());
+  stm_->setBaseUri(baseUri);
   ssize_t bufSize = 4096;
   unsigned char buf[bufSize];
 
   SharedHandle<SessionData> sessionData(new SessionData(stm_));
   XML_Parser parser = createParser(sessionData);
+  auto_delete<XML_Parser> deleter(parser, XML_ParserFree);
   off_t readOffset = 0;
   while(1) {
     ssize_t res = binaryStream->readData(buf, bufSize, readOffset);
@@ -222,7 +239,8 @@ MetalinkProcessor::parseFromBinaryStream(const SharedHandle<BinaryStream>& binar
     }
     if(XML_Parse(parser, reinterpret_cast<const char*>(buf), res, 0) ==
        XML_STATUS_ERROR) {
-      throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
+      throw DL_ABORT_EX2(MSG_CANNOT_PARSE_METALINK,
+                         error_code::METALINK_PARSE_ERROR);
     }
     readOffset += res;
   }
