@@ -42,23 +42,30 @@
 #include <expat.h>
 
 #include "SharedHandle.h"
+#include "A2STR.h"
 
 namespace aria2 {
 
 class Metalinker;
 class BinaryStream;
 class MetalinkParserStateMachine;
+class BufferedFile;
 
 class MetalinkProcessor {
 private:
   SharedHandle<MetalinkParserStateMachine> stm_;
-public:
-  SharedHandle<Metalinker> parseFile(const std::string& filename);
 
-  SharedHandle<Metalinker> parseFile(std::istream& stream);
+  SharedHandle<Metalinker> parseFile
+  (BufferedFile& fp,
+   const std::string& baseUri = A2STR::NIL);
+public:
+  SharedHandle<Metalinker> parseFile
+  (const std::string& filename,
+   const std::string& baseUri = A2STR::NIL);
 
   SharedHandle<Metalinker> parseFromBinaryStream
-  (const SharedHandle<BinaryStream>& binaryStream);
+  (const SharedHandle<BinaryStream>& binaryStream,
+   const std::string& baseUri = A2STR::NIL);
 };
 
 } // namespace aria2
