@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_FALLOC_FILE_ALLOCATION_ITERATOR_H_
-#define _D_FALLOC_FILE_ALLOCATION_ITERATOR_H_
+#ifndef D_FALLOC_FILE_ALLOCATION_ITERATOR_H
+#define D_FALLOC_FILE_ALLOCATION_ITERATOR_H
 
 #include "FileAllocationIterator.h"
 #include "BinaryStream.h"
@@ -43,9 +43,9 @@ namespace aria2 {
 // Allocate disk space using posix_fallocate() system call.
 class FallocFileAllocationIterator:public FileAllocationIterator {
 private:
-  BinaryStream* _stream;
-  off_t _offset;
-  uint64_t _totalLength;
+  BinaryStream* stream_;
+  off_t offset_;
+  uint64_t totalLength_;
 public:
   FallocFileAllocationIterator(BinaryStream* stream, off_t offset,
                                uint64_t totalLength);
@@ -61,5 +61,5 @@ public:
 
 } // namespace aria2
 
-#endif // _D_FALLOC_FILE_ALLOCATION_ITERATOR_H_
+#endif // D_FALLOC_FILE_ALLOCATION_ITERATOR_H
 
