@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_FILE_H_
-#define _D_FILE_H_
+#ifndef D_FILE_H
+#define D_FILE_H
 
 #include "common.h"
 
@@ -57,8 +57,13 @@ private:
   int fillStat(a2_struct_stat& fstat);
 public:
   File(const std::string& name);
+
+  File(const File& c);
+
   ~File();
 
+  File& operator=(const File& c);
+
   /**
    * Tests whether the file or directory denoted by name exists.
    */
@@ -114,8 +119,10 @@ public:
   // directory cannot be retrieved or its length is larger than 2048,
   // returns ".".
   static std::string getCurrentDir();
+  // Returns possible path separators for the underlining platform.
+  static const std::string& getPathSeparators();
 };
 
 } // namespace aria2
 
-#endif // _D_FILE_H_
+#endif // D_FILE_H
