@@ -38,6 +38,7 @@
 #include "DownloadEngine.h"
 #include "RequestGroup.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "message.h"
 #include "prefs.h"
 #include "util.h"
@@ -46,6 +47,8 @@
 #include "a2functional.h"
 #include "RecoverableException.h"
 #include "wallclock.h"
+#include "RequestGroupMan.h"
+#include "fmt.h"
 
 namespace aria2 {
 
@@ -53,47 +56,47 @@ FileAllocationCommand::FileAllocationCommand
 (cuid_t cuid, RequestGroup* requestGroup, DownloadEngine* e,
  const SharedHandle<FileAllocationEntry>& fileAllocationEntry):
   RealtimeCommand(cuid, requestGroup, e),
-  _fileAllocationEntry(fileAllocationEntry) {}
+  fileAllocationEntry_(fileAllocationEntry) {}
 
 FileAllocationCommand::~FileAllocationCommand() {}
 
 bool FileAllocationCommand::executeInternal()
 {
-  if(_requestGroup->isHaltRequested()) {
-    _e->_fileAllocationMan->dropPickedEntry();
+  if(getRequestGroup()->isHaltRequested()) {
+    getDownloadEngine()->getFileAllocationMan()->dropPickedEntry();
     return true;
   }
-  _fileAllocationEntry->allocateChunk();
-  if(_fileAllocationEntry->finished()) {
-    if(logger->debug()) {
-      logger->debug(MSG_ALLOCATION_COMPLETED,
-                    _timer.difference(global::wallclock),
-                    util::itos(_requestGroup->getTotalLength(), true).c_str());
-    }
-    _e->_fileAllocationMan->dropPickedEntry();
+  fileAllocationEntry_->allocateChunk();
+  if(fileAllocationEntry_->finished()) {
+    A2_LOG_DEBUG
+      (fmt(MSG_ALLOCATION_COMPLETED,
+           static_cast<long int>(timer_.difference(global::wallclock())),
+           util::itos(getRequestGroup()->getTotalLength(), true).c_str()));
+    getDownloadEngine()->getFileAllocationMan()->dropPickedEntry();
     
-    std::vector<Command*> commands;
-    try {
-      _fileAllocationEntry->prepareForNextAction(commands, _e);
-    } catch(RecoverableException& e) {
-      std::for_each(commands.begin(), commands.end(), Deleter());
-      throw;
-    }
-    _e->addCommand(commands);
-    _e->setNoWait(true);
+    std::vector<Command*>* commands = new std::vector<Command*>();
+    auto_delete_container<std::vector<Command*> > commandsDel(commands);
+    fileAllocationEntry_->prepareForNextAction(*commands, getDownloadEngine());
+    getDownloadEngine()->addCommand(*commands);
+    commands->clear();
+    getDownloadEngine()->setNoWait(true);
     return true;
   } else {
-    _e->commands.push_back(this);
+    getDownloadEngine()->addCommand(this);
     return false;
   }
 }
 
 bool FileAllocationCommand::handleException(Exception& e)
 {
-  _e->_fileAllocationMan->dropPickedEntry();
-  logger->error(MSG_FILE_ALLOCATION_FAILURE, e, util::itos(cuid).c_str());
-  logger->error(MSG_DOWNLOAD_NOT_COMPLETE, util::itos(cuid).c_str(),
-                _requestGroup->getDownloadContext()->getBasePath().c_str());
+  getDownloadEngine()->getFileAllocationMan()->dropPickedEntry();
+  A2_LOG_ERROR_EX(fmt(MSG_FILE_ALLOCATION_FAILURE,
+                      getCuid()),
+                  e);
+  A2_LOG_ERROR
+    (fmt(MSG_DOWNLOAD_NOT_COMPLETE,
+         getCuid(),
+         getRequestGroup()->getDownloadContext()->getBasePath().c_str()));
   return true;
 }
 
