@@ -38,24 +38,16 @@
 #include "RequestGroup.h"
 #include "PieceStorage.h"
 #include "DiskAdaptor.h"
-#include "DownloadContext.h"
-#include "RequestGroupMan.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
 
 namespace aria2 {
 
 FileAllocationEntry::FileAllocationEntry(RequestGroup* requestGroup, Command* nextCommand):
   RequestGroupEntry(requestGroup, nextCommand),
   fileAllocationIterator_(requestGroup->getPieceStorage()->getDiskAdaptor()->fileAllocationIterator())
-{
-  getRequestGroup()->getPieceStorage()->getDiskAdaptor()->enableDirectIO();
-}
+{}
 
 FileAllocationEntry:: ~FileAllocationEntry()
-{
-  getRequestGroup()->getPieceStorage()->getDiskAdaptor()->disableDirectIO();
-}
+{}
 
 off_t FileAllocationEntry::getCurrentLength()
 {
@@ -77,9 +69,4 @@ void FileAllocationEntry::allocateChunk()
   fileAllocationIterator_->allocateChunk();
 }
 
-void FileAllocationEntry::disableDirectIO()
-{
-  getRequestGroup()->getPieceStorage()->getDiskAdaptor()->disableDirectIO();
-}
-
 } // namespace aria2
