@@ -155,7 +155,7 @@ FileEntry::getRequest
             req.reset();
             continue;
           }
-          req->setReferer(referer);
+          req->setReferer(util::percentEncodeMini(referer));
           req->setMethod(method);
           spentUris_.push_back(uri);
           inFlightRequests_.push_back(req);
@@ -174,13 +174,13 @@ FileEntry::getRequest
     }
   } else {
     // Skip Request object if it is still
-    // sleeping(Request::getWakeTime() < global::wallclock).  If all
+    // sleeping(Request::getWakeTime() < global::wallclock()).  If all
     // pooled objects are sleeping, return first one.  Caller should
     // inspect returned object's getWakeTime().
     std::deque<SharedHandle<Request> >::iterator i = requestPool_.begin();
     std::deque<SharedHandle<Request> >::iterator eoi = requestPool_.end();
     for(; i != eoi; ++i) {
-      if((*i)->getWakeTime() <= global::wallclock) {
+      if((*i)->getWakeTime() <= global::wallclock()) {
         break;
       }
     }
@@ -200,7 +200,7 @@ FileEntry::findFasterRequest(const SharedHandle<Request>& base)
 {
   const int startupIdleTime = 10;
   if(requestPool_.empty() ||
-     lastFasterReplace_.difference(global::wallclock) < startupIdleTime) {
+     lastFasterReplace_.difference(global::wallclock()) < startupIdleTime) {
     return SharedHandle<Request>();
   }
   const SharedHandle<PeerStat>& fastest = requestPool_.front()->getPeerStat();
@@ -211,13 +211,13 @@ FileEntry::findFasterRequest(const SharedHandle<Request>& base)
   // TODO hard coded value. See PREF_STARTUP_IDLE_TIME
   if(!basestat ||
      (basestat->getDownloadStartTime().
-      difference(global::wallclock) >= startupIdleTime &&
+      difference(global::wallclock()) >= startupIdleTime &&
       fastest->getAvgDownloadSpeed()*0.8 > basestat->calculateDownloadSpeed())){
     // TODO we should consider that "fastest" is very slow.
     SharedHandle<Request> fastestRequest = requestPool_.front();
     requestPool_.pop_front();
     inFlightRequests_.push_back(fastestRequest);
-    lastFasterReplace_ = global::wallclock;
+    lastFasterReplace_ = global::wallclock();
     return fastestRequest;
   }
   return SharedHandle<Request>();
@@ -231,7 +231,7 @@ FileEntry::findFasterRequest
 {
   const int startupIdleTime = 10;
   const unsigned int SPEED_THRESHOLD = 20*1024;
-  if(lastFasterReplace_.difference(global::wallclock) < startupIdleTime) {
+  if(lastFasterReplace_.difference(global::wallclock()) < startupIdleTime) {
     return SharedHandle<Request>();
   }
   std::vector<std::string> inFlightHosts;
@@ -280,7 +280,7 @@ FileEntry::findFasterRequest
     uris_.erase(std::find(uris_.begin(), uris_.end(), uri));
     spentUris_.push_back(uri);
     inFlightRequests_.push_back(fastestRequest);
-    lastFasterReplace_ = global::wallclock;
+    lastFasterReplace_ = global::wallclock();
     return fastestRequest;
   }
   A2_LOG_DEBUG("No faster server found.");
@@ -518,8 +518,9 @@ size_t FileEntry::setUris(const std::vector<std::string>& uris)
 bool FileEntry::addUri(const std::string& uri)
 {
   uri::UriStruct us;
-  if(uri::parse(us, uri)) {
-    uris_.push_back(uri);
+  std::string peUri = util::percentEncodeMini(uri);
+  if(uri::parse(us, peUri)) {
+    uris_.push_back(peUri);
     return true;
   } else {
     return false;
@@ -529,9 +530,10 @@ bool FileEntry::addUri(const std::string& uri)
 bool FileEntry::insertUri(const std::string& uri, size_t pos)
 {
   uri::UriStruct us;
-  if(uri::parse(us, uri)) {
+  std::string peUri = util::percentEncodeMini(uri);
+  if(uri::parse(us, peUri)) {
     pos = std::min(pos, uris_.size());
-    uris_.insert(uris_.begin()+pos, uri);
+    uris_.insert(uris_.begin()+pos, peUri);
     return true;
   } else {
     return false;
@@ -568,4 +570,26 @@ bool FileEntry::emptyRequestUri() const
   return uris_.empty() && inFlightRequests_.empty() && requestPool_.empty();
 }
 
+void writeFilePath
+(std::ostream& o,
+ const SharedHandle<FileEntry>& entry,
+ bool memory)
+{
+  if(entry->getPath().empty()) {
+    std::vector<std::string> uris;
+    entry->getUris(uris);
+    if(uris.empty()) {
+      o << "n/a";
+    } else {
+      o << uris.front();
+    }
+  } else {
+    if(memory) {
+      o << "[MEMORY]" << File(entry->getPath()).getBasename();
+    } else {
+      o << entry->getPath();
+    }
+  }
+}
+
 } // namespace aria2
