@@ -46,15 +46,15 @@
 #include "File.h"
 #include "Request.h"
 #include "URIResult.h"
-#include "DownloadResultCode.h"
+#include "error_code.h"
 #include "A2STR.h"
 #include "TimerA2.h"
+#include "util.h"
 
 namespace aria2 {
 
 class URISelector;
-
-class Logger;
+class ServerStatMan;
 
 class FileEntry {
 private:
@@ -74,7 +74,6 @@ private:
   size_t maxConnectionPerServer_;
   std::string originalName_;
   Timer lastFasterReplace_;
-  Logger* logger_;
 
   void storePool(const SharedHandle<Request>& request);
 public:
@@ -93,7 +92,7 @@ public:
 
   const std::string& getPath() const { return path_; }
 
-  void setPath(const std::string& path) { path_ = path; }
+  void setPath(const std::string& path);
 
   uint64_t getLength() const { return length_; }
 
@@ -109,8 +108,6 @@ public:
 
   void setRequested(bool flag) { requested_ = flag; }
 
-  void setupDir();
-
   const std::deque<std::string>& getRemainingUris() const
   {
     return uris_;
@@ -147,10 +144,7 @@ public:
   // Inserts uris_ and spentUris_ into uris.
   void getUris(std::vector<std::string>& uris) const;
 
-  void setContentType(const std::string& contentType)
-  {
-    contentType_ = contentType;
-  }
+  void setContentType(const std::string& contentType);
 
   const std::string& getContentType() const { return contentType_; }
 
@@ -177,19 +171,20 @@ public:
   // removed from the pool and returned.
   SharedHandle<Request> findFasterRequest(const SharedHandle<Request>& base);
 
+  // Finds faster server using ServerStatMan.
+  SharedHandle<Request>
+  findFasterRequest
+  (const SharedHandle<Request>& base,
+   const std::vector<std::pair<size_t, std::string> >& usedHosts,
+   const SharedHandle<ServerStatMan>& serverStatMan);
+
   void poolRequest(const SharedHandle<Request>& request);
 
   bool removeRequest(const SharedHandle<Request>& request);
 
-  size_t countInFlightRequest() const
-  {
-    return inFlightRequests_.size();
-  }
+  size_t countInFlightRequest() const;
 
-  size_t countPooledRequest() const
-  {
-    return requestPool_.size();
-  }
+  size_t countPooledRequest() const;
 
   const std::deque<SharedHandle<Request> >& getInFlightRequests() const
   {
@@ -207,7 +202,7 @@ public:
 
   void removeIdenticalURI(const std::string& uri);
 
-  void addURIResult(std::string uri, downloadresultcode::RESULT result);
+  void addURIResult(std::string uri, error_code::Value result);
 
   const std::deque<URIResult>& getURIResults() const
   {
@@ -217,7 +212,7 @@ public:
   // Extracts URIResult whose _result is r and stores them into res.
   // The extracted URIResults are removed from uriResults_.
   void extractURIResult
-  (std::deque<URIResult>& res, downloadresultcode::RESULT r);
+  (std::deque<URIResult>& res, error_code::Value r);
 
   void setMaxConnectionPerServer(size_t n)
   {
@@ -236,10 +231,7 @@ public:
 
   void releaseRuntimeResource();
 
-  void setOriginalName(const std::string& originalName)
-  {
-    originalName_ = originalName;
-  }
+  void setOriginalName(const std::string& originalName);
 
   const std::string& getOriginalName() const
   {
@@ -248,10 +240,7 @@ public:
 
   bool removeUri(const std::string& uri);
 
-  bool emptyRequestUri() const
-  {
-    return uris_.empty() && inFlightRequests_.empty() && requestPool_.empty();
-  }
+  bool emptyRequestUri() const;
 
   void setUniqueProtocol(bool f)
   {
@@ -305,6 +294,14 @@ bool isUriSuppliedForRequsetFileEntry(InputIterator first, InputIterator last)
   return false;
 }
 
+// Writes filename to given o.  If memory is true, the output is
+// "[MEMORY]" plus the basename of the filename.  If there is no
+// FileEntry, writes "n/a" to o.
+void writeFilePath
+(std::ostream& o,
+ const SharedHandle<FileEntry>& entry,
+ bool memory);
+
 // Writes first filename to given o.  If memory is true, the output is
 // "[MEMORY]" plus the basename of the first filename.  If there is no
 // FileEntry, writes "n/a" to o.  If more than 1 FileEntry are in the
@@ -315,23 +312,11 @@ void writeFilePath
 (InputIterator first, InputIterator last, std::ostream& o, bool memory)
 {
   SharedHandle<FileEntry> e = getFirstRequestedFileEntry(first, last);
-  if(e.isNull()) {
+  if(!e) {
     o << "n/a";
   } else {
-    if(e->getPath().empty()) {
-      std::vector<std::string> uris;
-      e->getUris(uris);
-      if(uris.empty()) {
-        o << "n/a";
-      } else {
-        o << uris.front();
-      }
-    } else {
-      if(memory) {
-        o << "[MEMORY]" << File(e->getPath()).getBasename();
-      } else {
-        o << e->getPath();
-      }
+    writeFilePath(o, e, memory);
+    if(!e->getPath().empty()) {
       size_t count = countRequestedFileEntry(first, last);
       if(count > 1) {
         o << " (" << count-1 << "more)";
@@ -340,6 +325,6 @@ void writeFilePath
   }
 }
 
-}
+} // namespace aria2
 
 #endif // D_FILE_ENTRY_H
