@@ -32,11 +32,13 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_FTP_NEGOTIATION_COMMAND_H_
-#define _D_FTP_NEGOTIATION_COMMAND_H_
+#ifndef D_FTP_NEGOTIATION_COMMAND_H
+#define D_FTP_NEGOTIATION_COMMAND_H
 
 #include "AbstractCommand.h"
 
+#include <deque>
+
 namespace aria2 {
 
 class FtpConnection;
@@ -167,4 +169,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_FTP_NEGOTIATION_COMMAND_H_
+#endif // D_FTP_NEGOTIATION_COMMAND_H
