@@ -66,6 +66,9 @@ HttpServerCommand::HttpServerCommand
   e_->addSocketForReadCheck(socket_, this);
   httpServer_->setUsernamePassword(e_->getOption()->get(PREF_RPC_USER),
                                    e_->getOption()->get(PREF_RPC_PASSWD));
+  if(e_->getOption()->getAsBool(PREF_RPC_ALLOW_ORIGIN_ALL)) {
+    httpServer_->setAllowOrigin("*");
+  }
 #ifdef HAVE_ZLIB
   httpServer_->enableGZip();
 #else // !HAVE_ZLIB
@@ -109,7 +112,7 @@ bool HttpServerCommand::execute()
   try {
     if(socket_->isReadable(0) ||
        !httpServer_->getSocketRecvBuffer()->bufferEmpty()) {
-      timeoutTimer_ = global::wallclock;
+      timeoutTimer_ = global::wallclock();
       SharedHandle<HttpHeader> header;
 
       header = httpServer_->receiveRequest();
@@ -144,7 +147,7 @@ bool HttpServerCommand::execute()
       e_->setNoWait(true);
       return true;
     } else {
-      if(timeoutTimer_.difference(global::wallclock) >= 30) {
+      if(timeoutTimer_.difference(global::wallclock()) >= 30) {
         A2_LOG_INFO("HTTP request timeout.");
         return true;
       } else {
