@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_MULTI_DISK_ADAPTOR_H_
-#define _D_MULTI_DISK_ADAPTOR_H_
+#ifndef D_MULTI_DISK_ADAPTOR_H
+#define D_MULTI_DISK_ADAPTOR_H
 
 #include "DiskAdaptor.h"
 
@@ -48,7 +48,6 @@ private:
   SharedHandle<FileEntry> fileEntry_;
   SharedHandle<DiskWriter> diskWriter_;
   bool open_;
-  bool directIO_;
   bool needsFileAllocation_;
 public:
   DiskWriterEntry(const SharedHandle<FileEntry>& fileEntry);
@@ -86,16 +85,6 @@ public:
 
   bool operator<(const DiskWriterEntry& entry) const;
 
-  // Set directIO_ to true.
-  // Additionally, if diskWriter is opened, diskWriter->enableDirectIO() is
-  // called.
-  void enableDirectIO();
-
-  // Set directIO_ to false.
-  // Additionally, if diskWriter is opened, diskWriter->disableDirectIO() is
-  // called.
-  void disableDirectIO();
-
   bool needsFileAllocation() const
   {
     return needsFileAllocation_;
@@ -122,14 +111,10 @@ private:
 
   size_t maxOpenFiles_;
 
-  bool directIOAllowed_;
-
   bool readOnly_;
 
   void resetDiskWriterEntries();
 
-  void mkdir() const;
-
   void openIfNot(const SharedHandle<DiskWriterEntry>& entry,
                  void (DiskWriterEntry::*f)());
  
@@ -137,6 +122,7 @@ private:
 
 public:
   MultiDiskAdaptor();
+  ~MultiDiskAdaptor();
 
   virtual void initAndOpenFile();
 
@@ -157,10 +143,6 @@ public:
 
   virtual SharedHandle<FileAllocationIterator> fileAllocationIterator();
 
-  virtual void enableDirectIO();
-
-  virtual void disableDirectIO();
-
   virtual void enableReadOnly();
 
   virtual void disableReadOnly();
@@ -176,11 +158,6 @@ public:
     return pieceLength_;
   }
 
-  void allowDirectIO()
-  {
-    directIOAllowed_ = true;
-  }
-
   virtual void cutTrailingGarbage();
 
   void setMaxOpenFiles(size_t maxOpenFiles);
@@ -199,4 +176,4 @@ typedef SharedHandle<MultiDiskAdaptor> MultiDiskAdaptorHandle;
 
 } // namespace aria2
 
-#endif // _D_MULTI_DISK_ADAPTOR_H_
+#endif // D_MULTI_DISK_ADAPTOR_H
