@@ -36,6 +36,7 @@
 
 #include <signal.h>
 
+#include <cstring>
 #include <ostream>
 
 #include "RequestGroupMan.h"
@@ -54,14 +55,12 @@
 #include "File.h"
 #include "Netrc.h"
 #include "AuthConfigFactory.h"
-#include "DownloadContext.h"
 #include "SessionSerializer.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
 #include "TimeA2.h"
+#include "fmt.h"
+#include "SocketCore.h"
+#include "OutputFile.h"
 #ifdef ENABLE_SSL
-# include "SocketCore.h"
 # include "TLSContext.h"
 #endif // ENABLE_SSL
 
@@ -93,55 +92,105 @@ void handler(int signal) {
 }
 } // namespace
 
+#ifdef HAVE_ARES_ADDR_NODE
+
+namespace {
+
+ares_addr_node* parseAsyncDNSServers(const std::string& serversOpt)
+{
+  std::vector<std::string> servers;
+  util::split(serversOpt,
+              std::back_inserter(servers),
+              A2STR::COMMA_C,
+              true /* doStrip */);
+  ares_addr_node root;
+  root.next = 0;
+  ares_addr_node* tail = &root;
+  for(std::vector<std::string>::const_iterator i = servers.begin(),
+        eoi = servers.end(); i != eoi; ++i) {
+    struct addrinfo* res;
+    int s = callGetaddrinfo(&res, (*i).c_str(), 0, AF_UNSPEC,
+                            0, AI_NUMERICHOST, 0);
+    if(s != 0) {
+      continue;
+    }
+    WSAAPI_AUTO_DELETE<struct addrinfo*> resDeleter(res, freeaddrinfo);
+    if(res) {
+      ares_addr_node* node = new ares_addr_node();
+      node->next = 0;
+      node->family = res->ai_family;
+      if(node->family == AF_INET) {
+        struct sockaddr_in* in =
+          reinterpret_cast<struct sockaddr_in*>(res->ai_addr);
+        memcpy(&node->addr.addr4, &(in->sin_addr), 4);
+      } else {
+        struct sockaddr_in6* in =
+          reinterpret_cast<struct sockaddr_in6*>(res->ai_addr);
+        memcpy(&node->addr.addr6, &(in->sin6_addr), 16);
+      }
+      tail->next = node;
+      tail = node;
+    }
+  }
+  return root.next;
+}
+
+} // namespace
+
+#endif // HAVE_ARES_ADDR_NODE
+
 MultiUrlRequestInfo::MultiUrlRequestInfo
 (const std::vector<SharedHandle<RequestGroup> >& requestGroups,
  const SharedHandle<Option>& op,
  const SharedHandle<StatCalc>& statCalc,
- std::ostream& summaryOut)
-  :
-  requestGroups_(requestGroups),
-  option_(op),
-  statCalc_(statCalc),
-  summaryOut_(summaryOut),
-  logger_(LogFactory::getInstance())
+ const SharedHandle<OutputFile>& summaryOut)
+  : requestGroups_(requestGroups),
+    option_(op),
+    statCalc_(statCalc),
+    summaryOut_(summaryOut)
 {}
 
 MultiUrlRequestInfo::~MultiUrlRequestInfo() {}
 
 void MultiUrlRequestInfo::printMessageForContinue()
 {
-  summaryOut_ << "\n"
-              << _("aria2 will resume download if the transfer is restarted.")
-              << "\n"
-              << _("If there are any errors, then see the log file. See '-l' option in help/man page for details.")
-              << "\n";
+  summaryOut_->printf
+    ("\n%s\n%s\n",
+     _("aria2 will resume download if the transfer is restarted."),
+     _("If there are any errors, then see the log file. See '-l' option in help/man page for details."));
 }
 
-downloadresultcode::RESULT MultiUrlRequestInfo::execute()
+error_code::Value MultiUrlRequestInfo::execute()
 {
-  downloadresultcode::RESULT returnValue = downloadresultcode::FINISHED;
+  error_code::Value returnValue = error_code::FINISHED;
   try {
     DownloadEngineHandle e =
       DownloadEngineFactory().newDownloadEngine(option_.get(), requestGroups_);
 
     if(!option_->blank(PREF_LOAD_COOKIES)) {
       File cookieFile(option_->get(PREF_LOAD_COOKIES));
-      if(cookieFile.isFile()) {
-        e->getCookieStorage()->load(cookieFile.getPath(), Time().getTime());
-        logger_->info("Loaded cookies from '%s'.",
-                      cookieFile.getPath().c_str());
+      if(cookieFile.isFile() &&
+         e->getCookieStorage()->load(cookieFile.getPath(), Time().getTime())) {
+        A2_LOG_INFO(fmt("Loaded cookies from '%s'.",
+                        cookieFile.getPath().c_str()));
       } else {
-        logger_->error(MSG_LOADING_COOKIE_FAILED, cookieFile.getPath().c_str());
+        A2_LOG_ERROR(fmt(MSG_LOADING_COOKIE_FAILED,
+                         cookieFile.getPath().c_str()));
       }
     }
 
     SharedHandle<AuthConfigFactory> authConfigFactory(new AuthConfigFactory());
     File netrccf(option_->get(PREF_NETRC_PATH));
     if(!option_->getAsBool(PREF_NO_NETRC) && netrccf.isFile()) {
+#ifdef __MINGW32__
+      // Windows OS does not have permission, so set it to 0.
+      mode_t mode  = 0;
+#else // !__MINGW32__
       mode_t mode = netrccf.mode();
+#endif // !__MINGW32__
       if(mode&(S_IRWXG|S_IRWXO)) {
-        logger_->notice(MSG_INCORRECT_NETRC_PERMISSION,
-                        option_->get(PREF_NETRC_PATH).c_str());
+        A2_LOG_NOTICE(fmt(MSG_INCORRECT_NETRC_PERMISSION,
+                          option_->get(PREF_NETRC_PATH).c_str()));
       } else {
         SharedHandle<Netrc> netrc(new Netrc());
         netrc->parse(option_->get(PREF_NETRC_PATH));
@@ -159,19 +208,24 @@ downloadresultcode::RESULT MultiUrlRequestInfo::execute()
     }
     if(!option_->blank(PREF_CA_CERTIFICATE)) {
       if(!tlsContext->addTrustedCACertFile(option_->get(PREF_CA_CERTIFICATE))) {
-        logger_->info(MSG_WARN_NO_CA_CERT);
+        A2_LOG_INFO(MSG_WARN_NO_CA_CERT);
       }
     } else if(option_->getAsBool(PREF_CHECK_CERTIFICATE)) {
-      logger_->info(MSG_WARN_NO_CA_CERT);
+      A2_LOG_INFO(MSG_WARN_NO_CA_CERT);
     }
     if(option_->getAsBool(PREF_CHECK_CERTIFICATE)) {
       tlsContext->enablePeerVerification();
     }
     SocketCore::setTLSContext(tlsContext);
 #endif
+#ifdef HAVE_ARES_ADDR_NODE
+    ares_addr_node* asyncDNSServers =
+      parseAsyncDNSServers(option_->get(PREF_ASYNC_DNS_SERVER));
+    e->setAsyncDNSServers(asyncDNSServers);
+#endif // HAVE_ARES_ADDR_NODE
     if(!Timer::monotonicClock()) {
-      logger_->warn("Don't change system time while aria2c is running."
-                    " Doing this may make aria2c hang for long time.");
+      A2_LOG_WARN("Don't change system time while aria2c is running."
+                  " Doing this may make aria2c hang for long time.");
     }
 
     std::string serverStatIf = option_->get(PREF_SERVER_STAT_IF);
@@ -197,16 +251,17 @@ downloadresultcode::RESULT MultiUrlRequestInfo::execute()
     if(!serverStatOf.empty()) {
       e->getRequestGroupMan()->saveServerStat(serverStatOf);
     }
-    e->getRequestGroupMan()->showDownloadResults(summaryOut_);
-    summaryOut_ << std::flush;
+    e->getRequestGroupMan()->showDownloadResults
+      (*summaryOut_, option_->get(PREF_DOWNLOAD_RESULT) == A2_V_FULL);
+    summaryOut_->flush();
 
     RequestGroupMan::DownloadStat s =
       e->getRequestGroupMan()->getDownloadStat();
     if(!s.allCompleted()) {
       printMessageForContinue();
-      if(s.getLastErrorResult() == downloadresultcode::FINISHED &&
+      if(s.getLastErrorResult() == error_code::FINISHED &&
          s.getInProgress() > 0) {
-        returnValue = downloadresultcode::IN_PROGRESS;
+        returnValue = error_code::IN_PROGRESS;
       } else {
         returnValue = s.getLastErrorResult();
       }
@@ -216,18 +271,18 @@ downloadresultcode::RESULT MultiUrlRequestInfo::execute()
     if(!option_->blank(PREF_SAVE_SESSION)) {
       const std::string& filename = option_->get(PREF_SAVE_SESSION);
       if(sessionSerializer.save(filename)) {
-        logger_->notice("Serialized session to '%s' successfully.",
-                        filename.c_str());
+        A2_LOG_NOTICE(fmt("Serialized session to '%s' successfully.",
+                          filename.c_str()));
       } else {
-        logger_->notice("Failed to serialize session to '%s'.",
-                        filename.c_str());
+        A2_LOG_NOTICE(fmt("Failed to serialize session to '%s'.",
+                          filename.c_str()));
       }
     }
   } catch(RecoverableException& e) {
-    if(returnValue == downloadresultcode::FINISHED) {
-      returnValue = downloadresultcode::UNKNOWN_ERROR;
+    if(returnValue == error_code::FINISHED) {
+      returnValue = error_code::UNKNOWN_ERROR;
     }
-    logger_->error(EX_EXCEPTION_CAUGHT, e);
+    A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
   }
 #ifdef SIGHUP
   util::setGlobalSignalHandler(SIGHUP, SIG_DFL, 0);
