@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_OPTION_H_
-#define _D_OPTION_H_
+#ifndef D_OPTION_H
+#define D_OPTION_H
 
 #include "common.h"
 #include <string>
@@ -43,7 +43,7 @@ namespace aria2 {
 
 class Option {
 private:
-  std::map<std::string, std::string> table;
+  std::map<std::string, std::string> table_;
 public:
   Option();
   ~Option();
@@ -65,17 +65,15 @@ public:
 
   void clear();
 
-  std::map<std::string, std::string>::const_iterator begin() const
-  {
-    return table.begin();
-  }
+  std::map<std::string, std::string>::const_iterator begin() const;
 
-  std::map<std::string, std::string>::const_iterator end() const
-  {
-    return table.end();
-  }
+  std::map<std::string, std::string>::const_iterator end() const;
+
+  std::map<std::string, std::string>::iterator begin();
+
+  std::map<std::string, std::string>::iterator end();
 };
 
 } // namespace aria2
 
-#endif // _D_OPTION_H_
+#endif // D_OPTION_H
