@@ -131,6 +131,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BITTORRENT);
     op->addTag(TAG_METALINK);
     op->addTag(TAG_FILE);
+    op->addTag(TAG_CHECKSUM);
     handlers.push_back(op);
   }
 #endif // ENABLE_MESSAGE_DIGEST
@@ -205,6 +206,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->hide();
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new ParameterOptionHandler
+                                   (PREF_DOWNLOAD_RESULT,
+                                    TEXT_DOWNLOAD_RESULT,
+                                    A2_V_DEFAULT,
+                                    A2_V_DEFAULT,
+                                    A2_V_FULL));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
 #ifdef ENABLE_ASYNC_DNS
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
@@ -296,6 +307,21 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
   }
+#ifdef ENABLE_MESSAGE_DIGEST
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+                                   (PREF_HASH_CHECK_ONLY,
+                                    TEXT_HASH_CHECK_ONLY,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_BITTORRENT);
+    op->addTag(TAG_METALINK);
+    op->addTag(TAG_FILE);
+    op->addTag(TAG_CHECKSUM);
+    handlers.push_back(op);
+  }
+#endif // ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_HUMAN_READABLE,
@@ -531,6 +557,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     A2_V_TRUE,
                                     OptionHandler::OPT_ARG));
     op->addTag(TAG_METALINK);
+    op->addTag(TAG_CHECKSUM);
     handlers.push_back(op);
   }
 #endif // ENABLE_MESSAGE_DIGEST
@@ -595,6 +622,15 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+                                   (PREF_RPC_ALLOW_ORIGIN_ALL,
+                                    TEXT_RPC_ALLOW_ORIGIN_ALL,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
+    op->addTag(TAG_RPC);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_RPC_LISTEN_ALL,
@@ -639,6 +675,17 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   // HTTP/FTP options
+#ifdef ENABLE_MESSAGE_DIGEST
+  {
+    SharedHandle<OptionHandler> op(new ChecksumOptionHandler
+                                   (PREF_CHECKSUM,
+                                    TEXT_CHECKSUM));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    op->addTag(TAG_CHECKSUM);
+    handlers.push_back(op);
+  }
+#endif // ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
                                    (PREF_CONNECT_TIMEOUT,
@@ -715,6 +762,18 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<UnitNumberOptionHandler> op(new UnitNumberOptionHandler
+                                             (PREF_PIECE_LENGTH,
+                                              TEXT_PIECE_LENGTH,
+                                              "1M",
+                                              1024*1024,
+                                              1024*1024*1024));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_REMOTE_TIME,
@@ -746,15 +805,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
-  {
-    SharedHandle<UnitNumberOptionHandler> op(new UnitNumberOptionHandler
-                                             (PREF_SEGMENT_SIZE,
-                                              NO_DESCRIPTION,
-                                              "1M",
-                                              1024, -1));
-    op->hide();
-    handlers.push_back(op);
-  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_SERVER_STAT_IF,
@@ -812,7 +862,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     TEXT_STREAM_PIECE_SELECTOR,
                                     A2_V_DEFAULT,
                                     A2_V_DEFAULT,
-                                    V_INORDER));
+                                    V_INORDER,
+                                    A2_V_GEOM));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
@@ -1279,6 +1330,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
                                     A2_V_TRUE,
                                     OptionHandler::OPT_ARG));
     op->addTag(TAG_BITTORRENT);
+    op->addTag(TAG_CHECKSUM);
     handlers.push_back(op);
   }
   {
@@ -1752,6 +1804,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
       TAG_HOOK,
       TAG_FILE,
       TAG_RPC,
+      TAG_CHECKSUM,
       TAG_EXPERIMENTAL,
       TAG_DEPRECATED,
       TAG_HELP,
