@@ -42,6 +42,7 @@
 #include <string>
 
 #include "SharedHandle.h"
+#include "Command.h"
 
 namespace aria2 {
 
@@ -49,7 +50,7 @@ class BitfieldMan;
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-class MessageDigestContext;
+class MessageDigest;
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -59,17 +60,21 @@ private:
   size_t length_;
   size_t blockLength_;
   BitfieldMan* bitfield_;
-
+  std::vector<cuid_t> users_;
+  bool usedBySegment_;
 #ifdef ENABLE_MESSAGE_DIGEST
 
   size_t nextBegin_;
 
-  std::string hashAlgo_;
+  std::string hashType_;
 
-  SharedHandle<MessageDigestContext> mdctx_;
+  SharedHandle<MessageDigest> mdctx_;
 
 #endif // ENABLE_MESSAGE_DIGEST
 
+  Piece(const Piece& piece);
+
+  Piece& operator=(const Piece& piece);  
 public:
 
   static const size_t BLOCK_LENGTH  = 16*1024;
@@ -78,12 +83,8 @@ public:
 
   Piece(size_t index, size_t length, size_t blockLength = BLOCK_LENGTH);
 
-  Piece(const Piece& piece);
-
   ~Piece();
 
-  Piece& operator=(const Piece& piece);
-  
   bool operator==(const Piece& piece) const
   {
     return index_ == piece.index_;
@@ -154,7 +155,7 @@ public:
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-  void setHashAlgo(const std::string& algo);
+  void setHashType(const std::string& hashType);
 
   // Updates hash value. This function compares begin and private variable
   // nextBegin_ and only when they are equal, hash is updated eating data and
@@ -177,6 +178,22 @@ public:
    * Loses current bitfield state.
    */
   void reconfigure(size_t length);
+
+  void addUser(cuid_t cuid);
+  void removeUser(cuid_t cuid);
+  bool getUsed() const
+  {
+    return !users_.empty();
+  }
+  bool usedBy(cuid_t cuid) const;
+  bool getUsedBySegment() const
+  {
+    return usedBySegment_;
+  }
+  void setUsedBySegment(bool f)
+  {
+    usedBySegment_ = f;
+  }
 };
 
 } // namespace aria2
