@@ -32,11 +32,13 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_POST_DOWNLOAD_HANDLER_H_
-#define _D_POST_DOWNLOAD_HANDLER_H_
+#ifndef D_POST_DOWNLOAD_HANDLER_H
+#define D_POST_DOWNLOAD_HANDLER_H
 
 #include "DownloadHandler.h"
 
+#include <vector>
+
 namespace aria2 {
 
 class PostDownloadHandler:public DownloadHandler
@@ -47,13 +49,10 @@ public:
   virtual ~PostDownloadHandler() {}
 
   virtual void
-  getNextRequestGroups(std::deque<SharedHandle<RequestGroup> >& groups,
+  getNextRequestGroups(std::vector<SharedHandle<RequestGroup> >& groups,
                        RequestGroup* requestGroup) = 0;
 };
 
-typedef SharedHandle<PostDownloadHandler> PostDownloadHandlerHandle;
-typedef std::deque<PostDownloadHandlerHandle> PostDownloadHandlers;
-
 } // namespace aria2
 
-#endif // _D_POST_DOWNLOAD_HANDLER_H_
+#endif // D_POST_DOWNLOAD_HANDLER_H
