@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -32,10 +32,13 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_RANDOMIZER_H_
-#define _D_RANDOMIZER_H_
+#ifndef D_RANDOMIZER_H
+#define D_RANDOMIZER_H
 
 #include "common.h"
+#include "SharedHandle.h"
+
+namespace aria2 {
 
 class Randomizer {
 public:
@@ -52,4 +55,7 @@ public:
 };
 
 typedef SharedHandle<Randomizer> RandomizerHandle;
-#endif // _D_RANDOMIZER_H_
+
+} // namespace aria2
+
+#endif // D_RANDOMIZER_H
