@@ -68,17 +68,16 @@
 #include "FileEntry.h"
 #include "Request.h"
 #include "FileAllocationIterator.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "A2STR.h"
 #include "URISelector.h"
-#include "InOrderURISelector.h"
+#include "InorderURISelector.h"
 #include "PieceSelector.h"
 #include "a2functional.h"
 #include "SocketCore.h"
 #include "SimpleRandomizer.h"
-#include "ServerStatMan.h"
 #include "Segment.h"
-#include "FileAllocationEntry.h"
+#include "SocketRecvBuffer.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "CheckIntegrityCommand.h"
 # include "ChecksumCheckIntegrityEntry.h"
@@ -121,33 +120,37 @@
 
 namespace aria2 {
 
-gid_t RequestGroup::gidCounter_ = 0;
+a2_gid_t RequestGroup::gidCounter_ = 0;
 
-RequestGroup::RequestGroup(const SharedHandle<Option>& option):
-  gid_(newGID()),
-  option_(new Option(*option.get())),
-  numConcurrentCommand_(option->getAsInt(PREF_SPLIT)),
-  numStreamConnection_(0),
-  numStreamCommand_(0),
-  numCommand_(0),
-  saveControlFile_(true),
-  progressInfoFile_(new NullProgressInfoFile()),
-  preLocalFileCheckEnabled_(true),
-  haltRequested_(false),
-  forceHaltRequested_(false),
-  haltReason_(RequestGroup::NONE),
-  pauseRequested_(false),
-  uriSelector_(new InOrderURISelector()),
-  lastModifiedTime_(Time::null()),
-  fileNotFoundCount_(0),
-  timeout_(option->getAsInt(PREF_TIMEOUT)),
-  inMemoryDownload_(false),
-  maxDownloadSpeedLimit_(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT)),
-  maxUploadSpeedLimit_(option->getAsInt(PREF_MAX_UPLOAD_LIMIT)),
-  belongsToGID_(0),
-  requestGroupMan_(0),
-  resumeFailureCount_(0),
-  logger_(LogFactory::getInstance())
+RequestGroup::RequestGroup(const SharedHandle<Option>& option)
+  : gid_(newGID()),
+    option_(new Option(*option.get())),
+    numConcurrentCommand_(option->getAsInt(PREF_SPLIT)),
+    numStreamConnection_(0),
+    numStreamCommand_(0),
+    numCommand_(0),
+    saveControlFile_(true),
+    progressInfoFile_(new NullProgressInfoFile()),
+    preLocalFileCheckEnabled_(true),
+    haltRequested_(false),
+    forceHaltRequested_(false),
+    haltReason_(RequestGroup::NONE),
+    pauseRequested_(false),
+    uriSelector_(new InorderURISelector()),
+    lastModifiedTime_(Time::null()),
+    fileNotFoundCount_(0),
+    timeout_(option->getAsInt(PREF_TIMEOUT)),
+#ifdef ENABLE_BITTORRENT
+    btRuntime_(0),
+    peerStorage_(0),
+#endif // ENABLE_BITTORRENT
+    inMemoryDownload_(false),
+    maxDownloadSpeedLimit_(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT)),
+    maxUploadSpeedLimit_(option->getAsInt(PREF_MAX_UPLOAD_LIMIT)),
+    lastErrorCode_(error_code::UNDEFINED),
+    belongsToGID_(0),
+    requestGroupMan_(0),
+    resumeFailureCount_(0)
 {
   fileAllocationEnabled_ = option_->get(PREF_FILE_ALLOCATION) != V_NONE;
   // Add types to be sent as a Accept header value here.
@@ -173,7 +176,7 @@ bool RequestGroup::isCheckIntegrityReady() const
 
 bool RequestGroup::downloadFinished() const
 {
-  if(pieceStorage_.isNull()) {
+  if(!pieceStorage_) {
     return false;
   } else {
     return pieceStorage_->downloadFinished();
@@ -182,34 +185,35 @@ bool RequestGroup::downloadFinished() const
 
 bool RequestGroup::allDownloadFinished() const
 {
-  if(pieceStorage_.isNull()) {
+  if(!pieceStorage_) {
     return false;
   } else {
     return pieceStorage_->allDownloadFinished();
   }
 }
 
-downloadresultcode::RESULT RequestGroup::downloadResult() const
+error_code::Value RequestGroup::downloadResult() const
 {
   if(downloadFinished() && !downloadContext_->isChecksumVerificationNeeded())
-    return downloadresultcode::FINISHED;
+    return error_code::FINISHED;
   else {
-    if (lastUriResult_.isNull()) {
-      if(haltReason_ == RequestGroup::USER_REQUEST ||
-         haltReason_ == RequestGroup::SHUTDOWN_SIGNAL) {
-        return downloadresultcode::IN_PROGRESS;
+    if(haltReason_ == RequestGroup::USER_REQUEST) {
+      return error_code::REMOVED;
+    } else if(lastErrorCode_ == error_code::UNDEFINED) {
+      if(haltReason_ == RequestGroup::SHUTDOWN_SIGNAL) {
+        return error_code::IN_PROGRESS;
       } else {
-        return downloadresultcode::UNKNOWN_ERROR;
+        return error_code::UNKNOWN_ERROR;
       }
     } else {
-      return lastUriResult_->getResult();
+      return lastErrorCode_;
     }
   }    
 }
 
 void RequestGroup::closeFile()
 {
-  if(!pieceStorage_.isNull()) {
+  if(pieceStorage_) {
     pieceStorage_->getDiskAdaptor()->closeFile();
   }
 }
@@ -233,20 +237,19 @@ SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
     if(downloadFinished()) {
 #ifdef ENABLE_MESSAGE_DIGEST
       if(downloadContext_->isChecksumVerificationNeeded()) {
-        if(logger_->info()) {
-          logger_->info(MSG_HASH_CHECK_NOT_DONE);
-        }
-        SharedHandle<ChecksumCheckIntegrityEntry> tempEntry
-          (new ChecksumCheckIntegrityEntry(this));
+        A2_LOG_INFO(MSG_HASH_CHECK_NOT_DONE);
+        ChecksumCheckIntegrityEntry* tempEntry =
+          new ChecksumCheckIntegrityEntry(this);
         tempEntry->setRedownload(true);
-        checkEntry = tempEntry;
+        checkEntry.reset(tempEntry);
       } else
 #endif // ENABLE_MESSAGE_DIGEST
         {
           downloadContext_->setChecksumVerified(true);
-          logger_->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                          util::itos(gid_).c_str(),
-                          downloadContext_->getBasePath().c_str());
+          A2_LOG_NOTICE
+            (fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
+                 util::itos(gid_).c_str(),
+                 downloadContext_->getBasePath().c_str()));
         }
     } else {
       checkEntry.reset(new StreamCheckIntegrityEntry(this));
@@ -257,17 +260,18 @@ SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
     if(option_->getAsBool(PREF_CHECK_INTEGRITY) &&
        downloadContext_->isChecksumVerificationAvailable()) {
       loadAndOpenFile(infoFile);
-      SharedHandle<ChecksumCheckIntegrityEntry> tempEntry
-        (new ChecksumCheckIntegrityEntry(this));
+      ChecksumCheckIntegrityEntry* tempEntry =
+        new ChecksumCheckIntegrityEntry(this);
       tempEntry->setRedownload(true);
-      checkEntry = tempEntry;
+      checkEntry.reset(tempEntry);
     } else
 #endif // ENABLE_MESSAGE_DIGEST
       {
         downloadContext_->setChecksumVerified(true);
-        logger_->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                        util::itos(gid_).c_str(),
-                        downloadContext_->getBasePath().c_str());
+        A2_LOG_NOTICE
+          (fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
+               util::itos(gid_).c_str(),
+               downloadContext_->getBasePath().c_str()));
       }
   } else {
     loadAndOpenFile(infoFile);
@@ -295,22 +299,23 @@ void RequestGroup::createInitialCommand
           ("Cancel BitTorrent download in dry-run context.");
       }
       SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
-      if(!btRegistry->getDownloadContext(torrentAttrs->infoHash).isNull()) {
+      if(btRegistry->getDownloadContext(torrentAttrs->infoHash)) {
         // TODO If metadataGetMode == false and each FileEntry has
         // URI, then go without BT.
-        throw DOWNLOAD_FAILURE_EXCEPTION
-          (StringFormat
-           ("InfoHash %s is already registered.",
-            bittorrent::getInfoHashString(downloadContext_).c_str()).str());
+        throw DOWNLOAD_FAILURE_EXCEPTION2
+          (fmt("InfoHash %s is already registered.",
+               bittorrent::getInfoHashString(downloadContext_).c_str()),
+           error_code::DUPLICATE_INFO_HASH);
       }
       if(metadataGetMode) {
         // Use UnknownLengthPieceStorage.
         initPieceStorage();
       } else {
         if(e->getRequestGroupMan()->isSameFileBeingDownloaded(this)) {
-          throw DOWNLOAD_FAILURE_EXCEPTION
-            (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-                          downloadContext_->getBasePath().c_str()).str());
+          throw DOWNLOAD_FAILURE_EXCEPTION2
+            (fmt(EX_DUPLICATE_FILE_DOWNLOAD,
+                 downloadContext_->getBasePath().c_str()),
+             error_code::DUPLICATE_DOWNLOAD);
         }
         initPieceStorage();
         if(downloadContext_->getFileEntries().size() > 1) {
@@ -327,16 +332,16 @@ void RequestGroup::createInitialCommand
         
       BtRuntimeHandle btRuntime(new BtRuntime());
       btRuntime->setMaxPeers(option_->getAsInt(PREF_BT_MAX_PEERS));
-      btRuntime_ = btRuntime;
-      if(!progressInfoFile.isNull()) {
+      btRuntime_ = btRuntime.get();
+      if(progressInfoFile) {
         progressInfoFile->setBtRuntime(btRuntime);
       }
 
       SharedHandle<DefaultPeerStorage> peerStorage(new DefaultPeerStorage());
       peerStorage->setBtRuntime(btRuntime);
       peerStorage->setPieceStorage(pieceStorage_);
-      peerStorage_ = peerStorage;
-      if(!progressInfoFile.isNull()) {
+      peerStorage_ = peerStorage.get();
+      if(progressInfoFile) {
         progressInfoFile->setPeerStorage(peerStorage);
       }
 
@@ -350,16 +355,16 @@ void RequestGroup::createInitialCommand
       btAnnounce->shuffleAnnounce();
       
       assert(btRegistry->get(gid_).isNull());
-      btRegistry->put(gid_,
-                      BtObject(downloadContext_,
-                               pieceStorage_,
-                               peerStorage,
-                               btAnnounce,
-                               btRuntime,
-                               (progressInfoFile.isNull()?
-                                progressInfoFile_:
-                                SharedHandle<BtProgressInfoFile>
-                                (progressInfoFile))));
+      btRegistry->put
+        (gid_, BtObject
+         (downloadContext_,
+          pieceStorage_,
+          peerStorage,
+          btAnnounce,
+          btRuntime,
+          (progressInfoFile ?
+           SharedHandle<BtProgressInfoFile>(progressInfoFile) :
+           progressInfoFile_)));
       if(metadataGetMode) {
         if(option_->getAsBool(PREF_ENABLE_DHT) ||
            (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
@@ -376,8 +381,8 @@ void RequestGroup::createInitialCommand
             e->addCommand(dhtCommands);
           }
         } else {
-          logger_->notice("For BitTorrent Magnet URI, enabling DHT is strongly"
-                          " recommended. See --enable-dht option.");
+          A2_LOG_NOTICE("For BitTorrent Magnet URI, enabling DHT is strongly"
+                        " recommended. See --enable-dht option.");
         }
 
         SharedHandle<CheckIntegrityEntry> entry
@@ -396,12 +401,11 @@ void RequestGroup::createInitialCommand
         } else {
           // Open file in writable mode to allow the program
           // truncate the file to downloadContext_->getTotalLength()
-          if(logger_->debug()) {
-            logger_->debug("File size not match. File is opened in writable"
-                           " mode. Expected:%s Actual:%s",
-                           util::uitos(downloadContext_->getTotalLength()).c_str(),
-                           util::uitos(actualFileSize).c_str());
-          }
+          A2_LOG_DEBUG
+            (fmt("File size not match. File is opened in writable"
+                 " mode. Expected:%s Actual:%s",
+                 util::uitos(downloadContext_->getTotalLength()).c_str(),
+                 util::uitos(actualFileSize).c_str()));
         }
       }
       // Call Load, Save and file allocation command here
@@ -415,10 +419,10 @@ void RequestGroup::createInitialCommand
              !option_->getAsBool(PREF_ALLOW_OVERWRITE) &&
              !option_->getAsBool(PREF_BT_SEED_UNVERIFIED)) {
             // TODO we need this->haltRequested = true?
-            throw DOWNLOAD_FAILURE_EXCEPTION
-              (StringFormat
-               (MSG_FILE_ALREADY_EXISTS,
-                downloadContext_->getBasePath().c_str()).str());
+            throw DOWNLOAD_FAILURE_EXCEPTION2
+              (fmt(MSG_FILE_ALREADY_EXISTS,
+                   downloadContext_->getBasePath().c_str()),
+               error_code::FILE_ALREADY_EXISTS);
           } else {
             pieceStorage_->getDiskAdaptor()->openFile();
           }
@@ -480,19 +484,19 @@ void RequestGroup::createInitialCommand
       createNextCommand(commands, e, 1);
     } else {
       if(e->getRequestGroupMan()->isSameFileBeingDownloaded(this)) {
-        throw DOWNLOAD_FAILURE_EXCEPTION
-          (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-                        downloadContext_->getBasePath().c_str()).str());
+        throw DOWNLOAD_FAILURE_EXCEPTION2
+          (fmt(EX_DUPLICATE_FILE_DOWNLOAD,
+               downloadContext_->getBasePath().c_str()),
+           error_code::DUPLICATE_DOWNLOAD);
       }
-      adjustFilename
-        (SharedHandle<BtProgressInfoFile>(new DefaultBtProgressInfoFile
-                                          (downloadContext_,
-                                           SharedHandle<PieceStorage>(),
-                                           option_.get())));
+      SharedHandle<BtProgressInfoFile> progressInfoFile
+        (new DefaultBtProgressInfoFile
+         (downloadContext_, SharedHandle<PieceStorage>(), option_.get()));
+      adjustFilename(progressInfoFile);
       initPieceStorage();
       SharedHandle<CheckIntegrityEntry> checkEntry =
         createCheckIntegrityEntry();
-      if(!checkEntry.isNull()) {
+      if(checkEntry) {
         processCheckIntegrityEntry(commands, checkEntry, e);
       }
     }
@@ -507,9 +511,10 @@ void RequestGroup::createInitialCommand
     // In this context, multiple FileEntry objects are in
     // DownloadContext.
     if(e->getRequestGroupMan()->isSameFileBeingDownloaded(this)) {
-      throw DOWNLOAD_FAILURE_EXCEPTION
-        (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-                      downloadContext_->getBasePath().c_str()).str());
+      throw DOWNLOAD_FAILURE_EXCEPTION2
+        (fmt(EX_DUPLICATE_FILE_DOWNLOAD,
+             downloadContext_->getBasePath().c_str()),
+         error_code::DUPLICATE_DOWNLOAD);
     }
     initPieceStorage();
     if(downloadContext_->getFileEntries().size() > 1) {
@@ -530,10 +535,10 @@ void RequestGroup::createInitialCommand
         if(!isCheckIntegrityReady() &&
            !option_->getAsBool(PREF_ALLOW_OVERWRITE)) {
           // TODO we need this->haltRequested = true?
-          throw DOWNLOAD_FAILURE_EXCEPTION
-            (StringFormat
-             (MSG_FILE_ALREADY_EXISTS,
-              downloadContext_->getBasePath().c_str()).str());
+          throw DOWNLOAD_FAILURE_EXCEPTION2
+            (fmt(MSG_FILE_ALREADY_EXISTS,
+                 downloadContext_->getBasePath().c_str()),
+             error_code::FILE_ALREADY_EXISTS);
         } else {
           pieceStorage_->getDiskAdaptor()->openFile();
         }
@@ -589,8 +594,9 @@ void RequestGroup::initPieceStorage()
 #endif // ENABLE_BITTORRENT
       )) {
 #ifdef ENABLE_BITTORRENT
-    SharedHandle<DefaultPieceStorage> ps
-      (new DefaultPieceStorage(downloadContext_, option_.get()));
+    DefaultPieceStorage* ps =
+      new DefaultPieceStorage(downloadContext_, option_.get());
+    SharedHandle<PieceStorage> psHolder(ps);
     if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
       if(isUriSuppliedForRequsetFileEntry
          (downloadContext_->getFileEntries().begin(),
@@ -598,11 +604,10 @@ void RequestGroup::initPieceStorage()
         // Use LongestSequencePieceSelector when HTTP/FTP/BitTorrent
         // integrated downloads. Currently multi-file integrated
         // download is not supported.
-        if(logger_->debug()) {
-          logger_->debug("Using LongestSequencePieceSelector");
-        }
-        ps->setPieceSelector
-          (SharedHandle<PieceSelector>(new LongestSequencePieceSelector()));
+        A2_LOG_DEBUG("Using LongestSequencePieceSelector");
+        SharedHandle<PieceSelector> longestPieceSelector
+          (new LongestSequencePieceSelector());
+        ps->setPieceSelector(longestPieceSelector);
       }
       if(option_->defined(PREF_BT_PRIORITIZE_PIECE)) {
         std::vector<size_t> result;
@@ -621,27 +626,29 @@ void RequestGroup::initPieceStorage()
       }
     }
 #else // !ENABLE_BITTORRENT
-    SharedHandle<DefaultPieceStorage> ps
-      (new DefaultPieceStorage(downloadContext_, option_.get()));
+    DefaultPieceStorage* ps =
+      new DefaultPieceStorage(downloadContext_, option_.get());
+    SharedHandle<PieceStorage> psHolder(ps);
 #endif // !ENABLE_BITTORRENT
-    if(!diskWriterFactory_.isNull()) {
+    if(diskWriterFactory_) {
       ps->setDiskWriterFactory(diskWriterFactory_);
     }
-    tempPieceStorage = ps;
+    tempPieceStorage.swap(psHolder);
   } else {
-    UnknownLengthPieceStorageHandle ps
-      (new UnknownLengthPieceStorage(downloadContext_, option_.get()));
-    if(!diskWriterFactory_.isNull()) {
+    UnknownLengthPieceStorage* ps =
+      new UnknownLengthPieceStorage(downloadContext_, option_.get());
+    SharedHandle<PieceStorage> psHolder(ps);
+    if(diskWriterFactory_) {
       ps->setDiskWriterFactory(diskWriterFactory_);
     }
-    tempPieceStorage = ps;
+    tempPieceStorage.swap(psHolder);
   }
   tempPieceStorage->initStorage();
   SharedHandle<SegmentMan> tempSegmentMan
     (new SegmentMan(option_.get(), downloadContext_, tempPieceStorage));
 
-  pieceStorage_ = tempPieceStorage;
-  segmentMan_ = tempSegmentMan;
+  pieceStorage_.swap(tempPieceStorage);
+  segmentMan_.swap(tempSegmentMan);
 }
 
 void RequestGroup::dropPieceStorage()
@@ -679,8 +686,9 @@ void RequestGroup::adjustFilename
      option_->getAsBool(PREF_REMOVE_CONTROL_FILE) &&
      infoFile->exists()) {
     infoFile->removeFile();
-    logger_->notice("Removed control file for %s because it is requested by"
-                    " user.", infoFile->getFilename().c_str());
+    A2_LOG_NOTICE(fmt("Removed control file for %s because it is requested by"
+                      " user.",
+                      infoFile->getFilename().c_str()));
   }
   if(infoFile->exists()) {
     // Use current filename
@@ -712,9 +720,9 @@ void RequestGroup::removeDefunctControlFile
   if(progressInfoFile->exists() &&
      !pieceStorage_->getDiskAdaptor()->fileExists()) {
     progressInfoFile->removeFile();
-    logger_->notice(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
-                    progressInfoFile->getFilename().c_str(),
-                    downloadContext_->getBasePath().c_str());
+    A2_LOG_NOTICE(fmt(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
+                      progressInfoFile->getFilename().c_str(),
+                      downloadContext_->getBasePath().c_str()));
   }
 }
 
@@ -749,8 +757,7 @@ void RequestGroup::loadAndOpenFile(const BtProgressInfoFileHandle& progressInfoF
     }
     setProgressInfoFile(progressInfoFile);
   } catch(RecoverableException& e) {
-    throw DOWNLOAD_FAILURE_EXCEPTION2
-      (StringFormat(EX_DOWNLOAD_ABORTED).str(), e);
+    throw DOWNLOAD_FAILURE_EXCEPTION2(fmt(EX_DOWNLOAD_ABORTED), e);
   }
 }
 
@@ -764,16 +771,16 @@ void RequestGroup::shouldCancelDownloadForSafety()
   if(outfile.exists()) {
     if(option_->getAsBool(PREF_AUTO_FILE_RENAMING)) {
       if(tryAutoFileRenaming()) {
-        logger_->notice(MSG_FILE_RENAMED, getFirstFilePath().c_str());
+        A2_LOG_NOTICE(fmt(MSG_FILE_RENAMED, getFirstFilePath().c_str()));
       } else {
-        throw DOWNLOAD_FAILURE_EXCEPTION
-          (StringFormat("File renaming failed: %s",
-                        getFirstFilePath().c_str()).str());
+        throw DOWNLOAD_FAILURE_EXCEPTION2
+          (fmt("File renaming failed: %s", getFirstFilePath().c_str()),
+           error_code::FILE_RENAMING_FAILED);
       }
     } else {
-      throw DOWNLOAD_FAILURE_EXCEPTION
-        (StringFormat(MSG_FILE_ALREADY_EXISTS,
-                      getFirstFilePath().c_str()).str());
+      throw DOWNLOAD_FAILURE_EXCEPTION2
+        (fmt(MSG_FILE_ALREADY_EXISTS, getFirstFilePath().c_str()),
+         error_code::FILE_ALREADY_EXISTS);
     }
   }
 }
@@ -849,7 +856,7 @@ void RequestGroup::createNextCommand(std::vector<Command*>& commands,
 
 std::string RequestGroup::getFirstFilePath() const
 {
-  assert(!downloadContext_.isNull());
+  assert(downloadContext_);
   if(inMemoryDownload()) {
     static const std::string DIR_MEMORY("[MEMORY]");
     return DIR_MEMORY+File(downloadContext_->getFirstFileEntry()->getPath()).getBasename();
@@ -860,7 +867,7 @@ std::string RequestGroup::getFirstFilePath() const
 
 uint64_t RequestGroup::getTotalLength() const
 {
-  if(pieceStorage_.isNull()) {
+  if(!pieceStorage_) {
     return 0;
   } else {
     if(pieceStorage_->isSelectiveDownloadingMode()) {
@@ -873,7 +880,7 @@ uint64_t RequestGroup::getTotalLength() const
 
 uint64_t RequestGroup::getCompletedLength() const
 {
-  if(pieceStorage_.isNull()) {
+  if(!pieceStorage_) {
     return 0;
   } else {
     if(pieceStorage_->isSelectiveDownloadingMode()) {
@@ -891,9 +898,9 @@ void RequestGroup::validateFilename(const std::string& expectedFilename,
     return;
   }
   if(expectedFilename != actualFilename) {
-    throw DL_ABORT_EX(StringFormat(EX_FILENAME_MISMATCH,
-                                   expectedFilename.c_str(),
-                                   actualFilename.c_str()).str());
+    throw DL_ABORT_EX(fmt(EX_FILENAME_MISMATCH,
+                          expectedFilename.c_str(),
+                          actualFilename.c_str()));
   }
 }
 
@@ -905,9 +912,9 @@ void RequestGroup::validateTotalLength(uint64_t expectedTotalLength,
   }
   if(expectedTotalLength != actualTotalLength) {
     throw DL_ABORT_EX
-      (StringFormat(EX_SIZE_MISMATCH,
-                    util::itos(expectedTotalLength, true).c_str(),
-                    util::itos(actualTotalLength, true).c_str()).str());
+      (fmt(EX_SIZE_MISMATCH,
+           util::itos(expectedTotalLength, true).c_str(),
+           util::itos(actualTotalLength, true).c_str()));
   }
 }
 
@@ -945,7 +952,7 @@ unsigned int RequestGroup::getNumConnection() const
 {
   unsigned int numConnection = numStreamConnection_;
 #ifdef ENABLE_BITTORRENT
-  if(!btRuntime_.isNull()) {
+  if(btRuntime_) {
     numConnection += btRuntime_->getConnections();
   }
 #endif // ENABLE_BITTORRENT
@@ -961,9 +968,7 @@ void RequestGroup::decreaseNumCommand()
 {
   --numCommand_;
   if(!numCommand_ && requestGroupMan_) {
-    if(logger_->debug()) {
-      logger_->debug("GID#%s - Request queue check", util::itos(gid_).c_str());
-    }
+    A2_LOG_DEBUG(fmt("GID#%s - Request queue check", util::itos(gid_).c_str()));
     requestGroupMan_->requestQueueCheck();
   }
 }
@@ -973,11 +978,11 @@ TransferStat RequestGroup::calculateStat() const
 {
   TransferStat stat;
 #ifdef ENABLE_BITTORRENT
-  if(!peerStorage_.isNull()) {
+  if(peerStorage_) {
     stat = peerStorage_->calculateStat();
   }
 #endif // ENABLE_BITTORRENT
-  if(!segmentMan_.isNull()) {
+  if(segmentMan_) {
     stat.setDownloadSpeed
       (stat.getDownloadSpeed()+segmentMan_->calculateDownloadSpeed());
     stat.setSessionDownloadLength
@@ -995,7 +1000,7 @@ void RequestGroup::setHaltRequested(bool f, HaltReason haltReason)
     haltReason_ = haltReason;
   }
 #ifdef ENABLE_BITTORRENT
-  if(!btRuntime_.isNull()) {
+  if(btRuntime_) {
     btRuntime_->setHalt(f);
   }
 #endif // ENABLE_BITTORRENT
@@ -1016,22 +1021,22 @@ void RequestGroup::releaseRuntimeResource(DownloadEngine* e)
 {
 #ifdef ENABLE_BITTORRENT
   e->getBtRegistry()->remove(gid_);
+  btRuntime_ = 0;
+  peerStorage_ = 0;
 #endif // ENABLE_BITTORRENT
-  if(!pieceStorage_.isNull()) {
+  if(pieceStorage_) {
     pieceStorage_->removeAdvertisedPiece(0);
   }
   // Don't reset segmentMan_ and pieceStorage_ here to provide
-  // progress information via XML-RPC
+  // progress information via RPC
   progressInfoFile_.reset();
   downloadContext_->releaseRuntimeResource();
 }
 
 void RequestGroup::preDownloadProcessing()
 {
-  if(logger_->debug()) {
-    logger_->debug("Finding PreDownloadHandler for path %s.",
-                   getFirstFilePath().c_str());
-  }
+  A2_LOG_DEBUG(fmt("Finding PreDownloadHandler for path %s.",
+                   getFirstFilePath().c_str()));
   try {
     for(std::vector<SharedHandle<PreDownloadHandler> >::const_iterator itr =
           preDownloadHandlers_.begin(), eoi = preDownloadHandlers_.end();
@@ -1042,22 +1047,18 @@ void RequestGroup::preDownloadProcessing()
       }
     }
   } catch(RecoverableException& ex) {
-    logger_->error(EX_EXCEPTION_CAUGHT, ex);
+    A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
     return;
   }
-  if(logger_->debug()) {
-    logger_->debug("No PreDownloadHandler found.");
-  }
+  A2_LOG_DEBUG("No PreDownloadHandler found.");
   return;
 }
 
 void RequestGroup::postDownloadProcessing
 (std::vector<SharedHandle<RequestGroup> >& groups)
 {
-  if(logger_->debug()) {
-    logger_->debug("Finding PostDownloadHandler for path %s.",
-                   getFirstFilePath().c_str());
-  }
+  A2_LOG_DEBUG(fmt("Finding PostDownloadHandler for path %s.",
+                   getFirstFilePath().c_str()));
   try {
     for(std::vector<SharedHandle<PostDownloadHandler> >::const_iterator itr =
           postDownloadHandlers_.begin(), eoi = postDownloadHandlers_.end();
@@ -1068,11 +1069,9 @@ void RequestGroup::postDownloadProcessing
       }
     }
   } catch(RecoverableException& ex) {
-    logger_->error(EX_EXCEPTION_CAUGHT, ex);
-  }
-  if(logger_->debug()) {
-    logger_->debug("No PostDownloadHandler found.");
+    A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
   }
+  A2_LOG_DEBUG("No PostDownloadHandler found.");
 }
 
 void RequestGroup::initializePreDownloadHandler()
@@ -1107,7 +1106,7 @@ void RequestGroup::initializePostDownloadHandler()
 
 bool RequestGroup::isDependencyResolved()
 {
-  if(dependency_.isNull()) {
+  if(!dependency_) {
     return true;
   }
   return dependency_->resolve();
@@ -1164,10 +1163,8 @@ bool RequestGroup::needsFileAllocation() const
 
 DownloadResultHandle RequestGroup::createDownloadResult() const
 {
-  if(logger_->debug()) {
-    logger_->debug("GID#%s - Creating DownloadResult.",
-                   util::itos(gid_).c_str());
-  }
+  A2_LOG_DEBUG(fmt("GID#%s - Creating DownloadResult.",
+                   util::itos(gid_).c_str()));
   TransferStat st = calculateStat();
   SharedHandle<DownloadResult> res(new DownloadResult());
   res->gid = gid_;
@@ -1183,7 +1180,7 @@ DownloadResultHandle RequestGroup::createDownloadResult() const
   res->totalLength = getTotalLength();
   res->completedLength = getCompletedLength();
   res->uploadLength = st.getAllTimeUploadLength();
-  if(!pieceStorage_.isNull()) {
+  if(pieceStorage_) {
     if(pieceStorage_->getBitfieldLength() > 0) {
       res->bitfieldStr = util::toHex(pieceStorage_->getBitfield(),
                                      pieceStorage_->getBitfieldLength());
@@ -1196,27 +1193,28 @@ DownloadResultHandle RequestGroup::createDownloadResult() const
 #endif // ENABLE_BITTORRENT
   res->pieceLength = downloadContext_->getPieceLength();
   res->numPieces = downloadContext_->getNumPieces();
-  res->dir = downloadContext_->getDir();
+  res->dir = option_->get(PREF_DIR);
   return res;
 }
   
 void RequestGroup::reportDownloadFinished()
 {
-  logger_->notice(MSG_FILE_DOWNLOAD_COMPLETED,
-                  downloadContext_->getBasePath().c_str());
+  A2_LOG_NOTICE(fmt(MSG_FILE_DOWNLOAD_COMPLETED,
+                    downloadContext_->getBasePath().c_str()));
   uriSelector_->resetCounters();
 #ifdef ENABLE_BITTORRENT
   if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
     TransferStat stat = calculateStat();
-    double shareRatio =
-      ((stat.getAllTimeUploadLength()*10)/getCompletedLength())/10.0;
+    uint64_t completedLength = getCompletedLength();
+    double shareRatio = completedLength == 0 ? 0.0 :
+      1.0*stat.getAllTimeUploadLength()/completedLength;
     SharedHandle<TorrentAttribute> attrs =
       bittorrent::getTorrentAttrs(downloadContext_);
     if(!attrs->metadata.empty()) {
-      logger_->notice(MSG_SHARE_RATIO_REPORT,
-                      shareRatio,
-                      util::abbrevSize(stat.getAllTimeUploadLength()).c_str(),
-                      util::abbrevSize(getCompletedLength()).c_str());
+      A2_LOG_NOTICE(fmt(MSG_SHARE_RATIO_REPORT,
+                        shareRatio,
+                        util::abbrevSize(stat.getAllTimeUploadLength()).c_str(),
+                        util::abbrevSize(completedLength).c_str()));
     }
   }
 #endif // ENABLE_BITTORRENT
@@ -1242,13 +1240,13 @@ void RequestGroup::setURISelector(const SharedHandle<URISelector>& uriSelector)
 
 void RequestGroup::applyLastModifiedTimeToLocalFiles()
 {
-  if(!pieceStorage_.isNull() && lastModifiedTime_.good()) {
-    logger_->info("Applying Last-Modified time: %s",
-                  lastModifiedTime_.toHTTPDate().c_str());
+  if(pieceStorage_ && lastModifiedTime_.good()) {
+    A2_LOG_INFO(fmt("Applying Last-Modified time: %s",
+                    lastModifiedTime_.toHTTPDate().c_str()));
     size_t n =
       pieceStorage_->getDiskAdaptor()->utime(Time(), lastModifiedTime_);
-    logger_->info("Last-Modified attrs of %lu files were updated.",
-                  static_cast<unsigned long>(n));
+    A2_LOG_INFO(fmt("Last-Modified attrs of %lu files were updated.",
+                    static_cast<unsigned long>(n)));
   }
 }
 
@@ -1264,11 +1262,11 @@ void RequestGroup::increaseAndValidateFileNotFoundCount()
   ++fileNotFoundCount_;
   const unsigned int maxCount = option_->getAsInt(PREF_MAX_FILE_NOT_FOUND);
   if(maxCount > 0 && fileNotFoundCount_ >= maxCount &&
-     (segmentMan_.isNull() ||
+     (!segmentMan_ ||
       segmentMan_->calculateSessionDownloadLength() == 0)) {
     throw DOWNLOAD_FAILURE_EXCEPTION2
-      (StringFormat("Reached max-file-not-found count=%u", maxCount).str(),
-       downloadresultcode::MAX_FILE_NOT_FOUND);
+      (fmt("Reached max-file-not-found count=%u", maxCount),
+       error_code::MAX_FILE_NOT_FOUND);
   }
 }
 
@@ -1294,12 +1292,6 @@ bool RequestGroup::doesUploadSpeedExceed()
     maxUploadSpeedLimit_ < calculateStat().getUploadSpeed();
 }
 
-void RequestGroup::setLastUriResult
-(const std::string uri, downloadresultcode::RESULT result)
-{
-  lastUriResult_.reset(new URIResult(uri, result));
-}
-
 void RequestGroup::saveControlFile() const
 {
   if(saveControlFile_) {
@@ -1316,7 +1308,7 @@ void RequestGroup::setDownloadContext
 (const SharedHandle<DownloadContext>& downloadContext)
 {
   downloadContext_ = downloadContext;
-  if(!downloadContext_.isNull()) {
+  if(downloadContext_) {
     downloadContext_->setOwnerRequestGroup(this);
   }
 }
@@ -1330,7 +1322,7 @@ bool RequestGroup::p2pInvolved() const
 #endif // !ENABLE_BITTORRENT
 }
 
-gid_t RequestGroup::newGID()
+a2_gid_t RequestGroup::newGID()
 {
   if(gidCounter_ == INT64_MAX) {
     gidCounter_ = 0;
