@@ -38,8 +38,6 @@
 #include <cstring>
 #include <iomanip>
 #include <sstream>
-#include <ostream>
-#include <fstream>
 #include <numeric>
 #include <algorithm>
 #include <utility>
@@ -59,7 +57,7 @@
 #include "PeerStat.h"
 #include "SegmentMan.h"
 #include "FeedbackURISelector.h"
-#include "InOrderURISelector.h"
+#include "InorderURISelector.h"
 #include "AdaptiveURISelector.h"
 #include "Option.h"
 #include "prefs.h"
@@ -75,6 +73,7 @@
 #include "uri.h"
 #include "Triplet.h"
 #include "Signature.h"
+#include "OutputFile.h"
 
 namespace aria2 {
 
@@ -116,27 +115,21 @@ void RequestGroupMan::addRequestGroup
 void RequestGroupMan::addReservedGroup
 (const std::vector<SharedHandle<RequestGroup> >& groups)
 {
-  if(reservedGroups_.empty()) {
-    requestQueueCheck();
-  }
+  requestQueueCheck();
   reservedGroups_.insert(reservedGroups_.end(), groups.begin(), groups.end());
 }
 
 void RequestGroupMan::addReservedGroup
 (const SharedHandle<RequestGroup>& group)
 {
-  if(reservedGroups_.empty()) {
-    requestQueueCheck();
-  }
+  requestQueueCheck();
   reservedGroups_.push_back(group);
 }
 
 void RequestGroupMan::insertReservedGroup
 (size_t pos, const std::vector<SharedHandle<RequestGroup> >& groups)
 {
-  if(reservedGroups_.empty()) {
-    requestQueueCheck();
-  }
+  requestQueueCheck();
   reservedGroups_.insert
     (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos),
      groups.begin(), groups.end());
@@ -145,9 +138,7 @@ void RequestGroupMan::insertReservedGroup
 void RequestGroupMan::insertReservedGroup
 (size_t pos, const SharedHandle<RequestGroup>& group)
 {
-  if(reservedGroups_.empty()) {
-    requestQueueCheck();
-  }
+  requestQueueCheck();
   reservedGroups_.insert
     (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos), group);
 }
@@ -270,6 +261,7 @@ void executeStopHook
      !option->blank(PREF_ON_DOWNLOAD_COMPLETE)) {
     util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_COMPLETE);
   } else if(result != error_code::IN_PROGRESS &&
+            result != error_code::REMOVED &&
             !option->blank(PREF_ON_DOWNLOAD_ERROR)) {
     util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_ERROR);
   } else if(!option->blank(PREF_ON_DOWNLOAD_STOP)) {
@@ -467,7 +459,7 @@ void RequestGroupMan::configureRequestGroup
     SharedHandle<URISelector> sel(new FeedbackURISelector(serverStatMan_));
     requestGroup->setURISelector(sel);
   } else if(uriSelectorValue == V_INORDER) {
-    SharedHandle<URISelector> sel(new InOrderURISelector());
+    SharedHandle<URISelector> sel(new InorderURISelector());
     requestGroup->setURISelector(sel);
   } else if(uriSelectorValue == V_ADAPTIVE) {
     SharedHandle<URISelector> sel(new AdaptiveURISelector(serverStatMan_,
@@ -499,7 +491,8 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
     reservedGroups_.pop_front();
     std::vector<Command*> commands;
     try {
-      if(groupToAdd->isPauseRequested()||!groupToAdd->isDependencyResolved()) {
+      if((rpc_ && groupToAdd->isPauseRequested()) ||
+         !groupToAdd->isDependencyResolved()) {
         temp.push_back(groupToAdd);
         continue;
       }
@@ -571,6 +564,7 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   size_t finished = 0;
   size_t error = removedErrorResult_;
   size_t inprogress = 0;
+  size_t removed = 0;
   error_code::Value lastError = removedLastErrorResult_;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
@@ -580,38 +574,52 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
     }
     if((*itr)->result == error_code::FINISHED) {
       ++finished;
+    } else if((*itr)->result == error_code::IN_PROGRESS) {
+      ++inprogress;
+    } else if((*itr)->result == error_code::REMOVED) {
+      ++removed;
     } else {
       ++error;
       lastError = (*itr)->result;
     }
   }
-  return DownloadStat(finished, error, inprogress, reservedGroups_.size(),
+  return DownloadStat(finished, error, inprogress, removed,
+                      reservedGroups_.size(),
                       lastError);
 }
 
-void RequestGroupMan::showDownloadResults(std::ostream& o) const
+void RequestGroupMan::showDownloadResults(OutputFile& o, bool full) const
 {
   static const std::string MARK_OK("OK");
   static const std::string MARK_ERR("ERR");
   static const std::string MARK_INPR("INPR");
+  static const std::string MARK_RM("RM");
 
-  // Download Results:
-  // idx|stat|path/length
-  // ===+====+=======================================================================
-  o << "\n"
-    <<_("Download Results:") << "\n"
-    << "gid|stat|avg speed  |path/URI" << "\n"
-    << "===+====+===========+";
 #ifdef __MINGW32__
   int pathRowSize = 58;
 #else // !__MINGW32__
   int pathRowSize = 59;
 #endif // !__MINGW32__
-  o << std::setfill('=') << std::setw(pathRowSize) << '=' << "\n";
+  // Download Results:
+  // idx|stat|path/length
+  // ===+====+=======================================================================
+  o.printf("\n%s"
+           "\ngid|stat|avg speed  |",
+           _("Download Results:"));
+  if(full) {
+    o.write("  %|path/URI"
+            "\n===+====+===========+===+");
+    pathRowSize -= 4;
+  } else {
+    o.write("path/URI"
+            "\n===+====+===========+");
+  }
+  std::string line(pathRowSize, '=');
+  o.printf("%s\n", line.c_str());
   int ok = 0;
   int err = 0;
   int inpr = 0;
-
+  int rm = 0;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
       itr != eoi; ++itr) {
@@ -625,32 +633,44 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
     } else if((*itr)->result == error_code::IN_PROGRESS) {
       status = MARK_INPR;
       ++inpr;
+    } else if((*itr)->result == error_code::REMOVED) {
+      status = MARK_RM;
+      ++rm;
     } else {
       status = MARK_ERR;
       ++err;
     }
-    o << formatDownloadResult(status, *itr) << "\n";
+    if(full) {
+      formatDownloadResultFull(o, status, *itr);
+    } else {
+      o.write(formatDownloadResult(status, *itr).c_str());
+      o.write("\n");
+    }
   }
-  if(ok > 0 || err > 0 || inpr > 0) {
-    o << "\n"
-      << _("Status Legend:") << "\n";
-
+  if(ok > 0 || err > 0 || inpr > 0 || rm > 0) {
+    o.printf("\n%s\n", _("Status Legend:"));
     if(ok > 0) {
-      o << " (OK):download completed.";
+      o.write(" (OK):download completed.");
     }
     if(err > 0) {
-      o << "(ERR):error occurred.";
+      o.write("(ERR):error occurred.");
     }
     if(inpr > 0) {
-      o << "(INPR):download in-progress.";
+      o.write("(INPR):download in-progress.");
     }
-    o << "\n";
+    if(rm > 0) {
+      o.write("(RM):download removed.");
+    }
+    o.write("\n");
   }
 }
 
-std::string RequestGroupMan::formatDownloadResult(const std::string& status, const DownloadResultHandle& downloadResult) const
+namespace {
+void formatDownloadResultCommon
+(std::ostream& o,
+ const std::string& status,
+ const DownloadResultHandle& downloadResult)
 {
-  std::stringstream o;
   o << std::setw(3) << downloadResult->gid << "|"
     << std::setw(4) << status << "|"
     << std::setw(11);
@@ -662,6 +682,58 @@ std::string RequestGroupMan::formatDownloadResult(const std::string& status, con
     o << "n/a";
   }
   o << "|";
+}
+} // namespace
+
+void RequestGroupMan::formatDownloadResultFull
+(OutputFile& out,
+ const std::string& status,
+ const DownloadResultHandle& downloadResult) const
+{
+  BitfieldMan bt(downloadResult->pieceLength, downloadResult->totalLength);
+  bt.setBitfield(reinterpret_cast<const unsigned char*>
+                 (util::fromHex(downloadResult->bitfieldStr).data()),
+                 downloadResult->bitfieldStr.size()/2);
+  bool head = true;
+  const std::vector<SharedHandle<FileEntry> >& fileEntries =
+    downloadResult->fileEntries;
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
+        fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
+    if(!(*i)->isRequested()) {
+      continue;
+    }
+    std::stringstream o;
+    if(head) {
+      formatDownloadResultCommon(o, status, downloadResult);
+      head = false;
+    } else {
+      o << "   |    |           |";
+    }
+    if((*i)->getLength() == 0 || downloadResult->bitfieldStr.empty()) {
+      o << "  -|";
+    } else {
+      uint64_t completedLength =
+        bt.getOffsetCompletedLength((*i)->getOffset(), (*i)->getLength());
+      o << std::setw(3) << 100*completedLength/(*i)->getLength() << "|";
+    }
+    writeFilePath(o, *i, downloadResult->inMemoryDownload);
+    o << "\n";
+    out.write(o.str().c_str());
+  }
+  if(head) {
+    std::stringstream o;
+    formatDownloadResultCommon(o, status, downloadResult);
+    o << "  -|n/a\n";
+    out.write(o.str().c_str());
+  }
+}
+
+std::string RequestGroupMan::formatDownloadResult
+(const std::string& status,
+ const DownloadResultHandle& downloadResult) const
+{
+  std::stringstream o;
+  formatDownloadResultCommon(o, status, downloadResult);
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
     downloadResult->fileEntries;
   writeFilePath(fileEntries.begin(), fileEntries.end(), o,
@@ -827,44 +899,12 @@ bool RequestGroupMan::addServerStat(const SharedHandle<ServerStat>& serverStat)
 
 bool RequestGroupMan::loadServerStat(const std::string& filename)
 {
-  std::ifstream in(filename.c_str(), std::ios::binary);
-  if(!in) {
-    A2_LOG_ERROR(fmt(MSG_OPENING_READABLE_SERVER_STAT_FILE_FAILED,
-                     filename.c_str()));
-    return false;
-  }
-  if(serverStatMan_->load(in)) {
-    A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_LOADED, filename.c_str()));
-    return true;
-  } else {
-    A2_LOG_ERROR(fmt(MSG_READING_SERVER_STAT_FILE_FAILED, filename.c_str()));
-    return false;
-  }
+  return serverStatMan_->load(filename);
 }
 
 bool RequestGroupMan::saveServerStat(const std::string& filename) const
 {
-  std::string tempfile = filename;
-  tempfile += "__temp";
-  {
-    std::ofstream out(tempfile.c_str(), std::ios::binary);
-    if(!out) {
-      A2_LOG_ERROR(fmt(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
-                       filename.c_str()));
-      return false;
-    }
-    if(!serverStatMan_->save(out)) {
-      A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
-      return false;
-    }
-  }
-  if(File(tempfile).renameTo(filename)) {
-    A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_SAVED, filename.c_str()));
-    return true;
-  } else {
-    A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
-    return false;
-  }
+  return serverStatMan_->save(filename);
 }
 
 void RequestGroupMan::removeStaleServerStat(time_t timeout)
