@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_REQUEST_SLOT_H_
-#define _D_REQUEST_SLOT_H_
+#ifndef D_REQUEST_SLOT_H
+#define D_REQUEST_SLOT_H
 
 #include "common.h"
 #include "TimerA2.h"
@@ -71,7 +71,7 @@ public:
   
   RequestSlot(size_t index, uint32_t begin, size_t length, size_t blockIndex,
               const SharedHandle<Piece>& piece = SharedHandle<Piece>()):
-    dispatchedTime_(global::wallclock),
+    dispatchedTime_(global::wallclock()),
     index_(index), begin_(begin), length_(length), blockIndex_(blockIndex),
     piece_(piece) {}
 
@@ -145,4 +145,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_REQUEST_SLOT_H_
+#endif // D_REQUEST_SLOT_H
