@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_SEGMENT_MAN_H_
-#define _D_SEGMENT_MAN_H_
+#ifndef D_SEGMENT_MAN_H
+#define D_SEGMENT_MAN_H
 
 #include "common.h"
 
@@ -49,7 +49,6 @@
 namespace aria2 {
 
 class Segment;
-class Logger;
 class Option;
 class PeerStat;
 class DownloadContext;
@@ -60,8 +59,13 @@ class FileEntry;
 struct SegmentEntry {
   cuid_t cuid;
   SharedHandle<Segment> segment;
-  SegmentEntry(cuid_t cuid, const SharedHandle<Segment>& segment):
-    cuid(cuid), segment(segment) {}
+
+  SegmentEntry(cuid_t cuid, const SharedHandle<Segment>& segment);
+  ~SegmentEntry();
+
+  // Don't allow copying
+  SegmentEntry(const SegmentEntry&);
+  SegmentEntry& operator=(const SegmentEntry&);
 };
 
 typedef SharedHandle<SegmentEntry> SegmentEntryHandle;
@@ -74,8 +78,6 @@ class SegmentMan {
 private:
   const Option* option_;
 
-  Logger* logger_;
-
   SharedHandle<DownloadContext> downloadContext_;
 
   SharedHandle<PieceStorage> pieceStorage_;
@@ -104,7 +106,7 @@ private:
   SharedHandle<Segment> checkoutSegment(cuid_t cuid,
                                         const SharedHandle<Piece>& piece);
 
-  void cancelSegment(const SharedHandle<Segment>& segment);
+  void cancelSegmentInternal(cuid_t cuid, const SharedHandle<Segment>& segment);
 public:
   SegmentMan(const Option* option,
              const SharedHandle<DownloadContext>& downloadContext,
@@ -251,4 +253,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_SEGMENT_MAN_H_
+#endif // D_SEGMENT_MAN_H
