@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -32,19 +32,18 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_SERVER_STAT_H_
-#define _D_SERVER_STAT_H_
+#ifndef D_SERVER_STAT_H
+#define D_SERVER_STAT_H
 #include "common.h"
 
 #include <string>
 #include <iosfwd>
 
+#include "SharedHandle.h"
 #include "TimeA2.h"
 
 namespace aria2 {
 
-class Logger;
-
 // ServerStatMan: has many ServerStat
 // URISelector: interface
 // ServerStatURISelector: Has a reference of ServerStatMan
@@ -53,7 +52,7 @@ class ServerStat {
 public:
   enum STATUS {
     OK = 0,
-    ERROR
+    A2_ERROR
   };
   
   static const std::string STATUS_STRING[];
@@ -64,17 +63,17 @@ public:
 
   const std::string& getHostname() const
   {
-    return _hostname;
+    return hostname_;
   }
 
   const std::string& getProtocol() const
   {
-    return _protocol;
+    return protocol_;
   }
 
   const Time& getLastUpdated() const
   {
-    return _lastUpdated;
+    return lastUpdated_;
   }
 
   // This method doesn't update _lastUpdate.
@@ -82,10 +81,10 @@ public:
 
   unsigned int getDownloadSpeed() const
   {
-    return _downloadSpeed;
+    return downloadSpeed_;
   }
 
-  // update download speed and update _lastUpdated
+  // update download speed and update lastUpdated_
   void updateDownloadSpeed(unsigned int downloadSpeed);
 
   // set download speed. This method doesn't update _lastUpdate.
@@ -93,7 +92,7 @@ public:
 
   unsigned int getSingleConnectionAvgSpeed() const
   {
-    return _singleConnectionAvgSpeed;
+    return singleConnectionAvgSpeed_;
   }
 
   void updateSingleConnectionAvgSpeed(unsigned int downloadSpeed);
@@ -101,7 +100,7 @@ public:
 
   unsigned int getMultiConnectionAvgSpeed() const
   {
-    return _multiConnectionAvgSpeed;
+    return multiConnectionAvgSpeed_;
   }
 
   void updateMultiConnectionAvgSpeed(unsigned int downloadSpeed);
@@ -109,7 +108,7 @@ public:
 
   unsigned int getCounter() const
   {
-    return _counter;
+    return counter_;
   }
 
   void increaseCounter();
@@ -125,52 +124,60 @@ public:
 
   STATUS getStatus() const
   {
-    return _status;
+    return status_;
   }
 
   bool isOK() const
   {
-    return _status == OK;
+    return status_ == OK;
   }
 
-  // set status OK and update _lastUpdated
+  // set status OK and update lastUpdated_
   void setOK();
 
   bool isError() const
   {
-    return _status == ERROR;
+    return status_ == A2_ERROR;
   }
 
-  // set status ERROR and update _lastUpdated
+  // set status ERROR and update lastUpdated_
   void setError();
 
   bool operator<(const ServerStat& serverStat) const;
 
   bool operator==(const ServerStat& serverStat) const;
+
+  std::string toString() const;
 private:
-  std::string _hostname;
+  std::string hostname_;
   
-  std::string _protocol;
+  std::string protocol_;
 
-  unsigned int _downloadSpeed;
+  unsigned int downloadSpeed_;
   
-  unsigned int _singleConnectionAvgSpeed;
+  unsigned int singleConnectionAvgSpeed_;
   
-  unsigned int _multiConnectionAvgSpeed;
+  unsigned int multiConnectionAvgSpeed_;
 
-  unsigned int _counter;
+  unsigned int counter_;
 
-  Logger* _logger;
+  STATUS status_;
 
-  STATUS _status;
-
-  Time _lastUpdated;
+  Time lastUpdated_;
 
   void setStatusInternal(STATUS status);
 };
 
-std::ostream& operator<<(std::ostream& o, const ServerStat& serverStat);
+class ServerStatFaster {
+public:
+  bool operator()
+  (const std::pair<SharedHandle<ServerStat>, std::string> lhs,
+   const std::pair<SharedHandle<ServerStat>, std::string> rhs) const
+  {
+    return lhs.first->getDownloadSpeed() > rhs.first->getDownloadSpeed();
+  }
+};
 
 } // namespace aria2
 
-#endif // _D_SERVER_STAT_H_
+#endif // D_SERVER_STAT_H
