@@ -34,8 +34,9 @@
 /* copyright --> */
 #include "ServerStatMan.h"
 
+#include <cstring>
+#include <cstdio>
 #include <algorithm>
-#include <ostream>
 #include <iterator>
 #include <map>
 #include <vector>
@@ -43,6 +44,12 @@
 #include "ServerStat.h"
 #include "util.h"
 #include "RecoverableException.h"
+#include "a2functional.h"
+#include "BufferedFile.h"
+#include "message.h"
+#include "fmt.h"
+#include "LogFactory.h"
+#include "File.h"
 
 namespace aria2 {
 
@@ -55,7 +62,8 @@ SharedHandle<ServerStat> ServerStatMan::find(const std::string& hostname,
 {
   SharedHandle<ServerStat> ss(new ServerStat(hostname, protocol));
   std::deque<SharedHandle<ServerStat> >::const_iterator i =
-    std::lower_bound(serverStats_.begin(), serverStats_.end(), ss);
+    std::lower_bound(serverStats_.begin(), serverStats_.end(), ss,
+                     DerefLess<SharedHandle<ServerStat> >());
   if(i != serverStats_.end() &&
      (*i)->getHostname() == hostname && (*i)->getProtocol() == protocol) {
     return *i;
@@ -67,9 +75,10 @@ SharedHandle<ServerStat> ServerStatMan::find(const std::string& hostname,
 bool ServerStatMan::add(const SharedHandle<ServerStat>& serverStat)
 {
   std::deque<SharedHandle<ServerStat> >::iterator i =
-    std::lower_bound(serverStats_.begin(), serverStats_.end(), serverStat);
+    std::lower_bound(serverStats_.begin(), serverStats_.end(), serverStat,
+                     DerefLess<SharedHandle<ServerStat> >());
 
-  if(i != serverStats_.end() && (*i) == serverStat) {
+  if(i != serverStats_.end() && *(*i) == *serverStat) {
     return false;
   } else {
     serverStats_.insert(i, serverStat);
@@ -77,15 +86,41 @@ bool ServerStatMan::add(const SharedHandle<ServerStat>& serverStat)
   } 
 }
 
-bool ServerStatMan::save(std::ostream& out) const
+bool ServerStatMan::save(const std::string& filename) const
 {
-  std::copy(serverStats_.begin(), serverStats_.end(),
-            std::ostream_iterator<SharedHandle<ServerStat> >(out, "\n"));
-  out.flush();
-  return !out.bad();
+  std::string tempfile = filename;
+  tempfile += "__temp";
+  {
+    BufferedFile fp(tempfile, BufferedFile::WRITE);
+    if(!fp) {
+      A2_LOG_ERROR(fmt(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
+                       filename.c_str()));
+      return false;
+    }
+    for(std::deque<SharedHandle<ServerStat> >::const_iterator i =
+          serverStats_.begin(), eoi = serverStats_.end(); i != eoi; ++i) {
+      std::string l = (*i)->toString();
+      l += "\n";
+      if(fp.write(l.data(), l.size()) != l.size()) {
+        A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED,
+                         filename.c_str()));
+      }
+    }
+    if(fp.close() == EOF) {
+      A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
+      return false;
+    }
+  }
+  if(File(tempfile).renameTo(filename)) {
+    A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_SAVED, filename.c_str()));
+    return true;
+  } else {
+    A2_LOG_ERROR(fmt(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str()));
+    return false;
+  }
 }
 
-bool ServerStatMan::load(std::istream& in)
+bool ServerStatMan::load(const std::string& filename)
 {
   static const std::string S_HOST = "host";
   static const std::string S_PROTOCOL = "protocol";
@@ -96,9 +131,24 @@ bool ServerStatMan::load(std::istream& in)
   static const std::string S_COUNTER = "counter";
   static const std::string S_STATUS = "status";
 
-  std::string line;
-  while(getline(in, line)) {
-    util::trimSelf(line);
+  BufferedFile fp(filename, BufferedFile::READ);
+  if(!fp) {
+    A2_LOG_ERROR(fmt(MSG_OPENING_READABLE_SERVER_STAT_FILE_FAILED,
+                     filename.c_str()));
+    return false;
+  }
+  char buf[4096];
+  while(1) {
+    if(!fp.getsn(buf, sizeof(buf))) {
+      if(fp.eof()) {
+        break;
+      } else {
+        A2_LOG_ERROR(fmt(MSG_READING_SERVER_STAT_FILE_FAILED,
+                         filename.c_str()));
+        return false;
+      }
+    }
+    std::string line = util::stripIter(&buf[0], &buf[strlen(buf)]);
     if(line.empty()) {
       continue;
     }
@@ -107,9 +157,10 @@ bool ServerStatMan::load(std::istream& in)
     std::map<std::string, std::string> m;
     for(std::vector<std::string>::const_iterator i = items.begin(),
           eoi = items.end(); i != eoi; ++i) {
-      std::pair<std::string, std::string> p = util::split(*i, "=");
-      util::trimSelf(p.first);
-      util::trimSelf(p.second);
+      std::pair<std::string, std::string> p;
+      util::divide(p, *i, '=');
+      p.first = util::strip(p.first);
+      p.second = util::strip(p.second);
       m[p.first] = p.second;
     }
     if(m[S_HOST].empty() || m[S_PROTOCOL].empty()) {
@@ -137,7 +188,8 @@ bool ServerStatMan::load(std::istream& in)
       continue;
     }
   }
-  return !in.bad();
+  A2_LOG_NOTICE(fmt(MSG_SERVER_STAT_LOADED, filename.c_str()));
+  return true;
 }
 
 namespace {
@@ -153,7 +205,7 @@ public:
     return ss->getLastUpdated().difference(time_) >= timeout_;
   }
 };
-}
+} // namespace
 
 void ServerStatMan::removeStaleServerStat(time_t timeout)
 {
