@@ -33,23 +33,30 @@
  */
 /* copyright --> */
 #include "SingleFileAllocationIterator.h"
+
+#include <cstring>
+#include <cstdlib>
+
 #include "BinaryStream.h"
 #include "util.h"
 #include "a2io.h"
-#include <cstring>
-#include <cstdlib>
+#include "Logger.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
 #define BUFSIZE (256*1024)
 #define ALIGNMENT 512
 
-SingleFileAllocationIterator::SingleFileAllocationIterator(BinaryStream* stream, off_t offset, uint64_t totalLength):stream_(stream), offset_(offset), totalLength_(totalLength), buffer_(0)
-{
-  if(offset_%ALIGNMENT != 0) {
-    stream_->disableDirectIO();
-  }
-}
+SingleFileAllocationIterator::SingleFileAllocationIterator
+(BinaryStream* stream,
+ off_t offset,
+ uint64_t totalLength)
+  : stream_(stream),
+    offset_(offset),
+    totalLength_(totalLength),
+    buffer_(0)
+{}
 
 SingleFileAllocationIterator::~SingleFileAllocationIterator()
 {
@@ -62,6 +69,13 @@ SingleFileAllocationIterator::~SingleFileAllocationIterator()
 
 void SingleFileAllocationIterator::init()
 {
+  static bool noticeDone = false;
+  if(!noticeDone) {
+    noticeDone = true;
+    A2_LOG_NOTICE("Allocating disk space. Use --file-allocation=none to"
+                  " disable it. See --file-allocation option in man page for"
+                  " more details.");
+  }
 #ifdef HAVE_POSIX_MEMALIGN
   buffer_ = reinterpret_cast<unsigned char*>
     (util::allocateAlignedMemory(ALIGNMENT, BUFSIZE));
@@ -84,7 +98,7 @@ void SingleFileAllocationIterator::allocateChunk()
 
 bool SingleFileAllocationIterator::finished()
 {
-  return (uint64_t)offset_ >= totalLength_;
+  return (uint64_t)offset_ == totalLength_;
 }
 
 } // namespace aria2
