@@ -53,6 +53,7 @@
 #include "Option.h"
 #include "DlAbortEx.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "A2STR.h"
 #include "SocketCore.h"
 #include "Request.h"
@@ -61,18 +62,15 @@
 #include "bittorrent_helper.h"
 #include "a2functional.h"
 #include "util.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
+#include "fmt.h"
 
 namespace aria2 {
 
 TrackerWatcherCommand::TrackerWatcherCommand
-(cuid_t cuid, RequestGroup* requestGroup, DownloadEngine* e):
-  Command(cuid),
-  requestGroup_(requestGroup),
-  e_(e)
+(cuid_t cuid, RequestGroup* requestGroup, DownloadEngine* e)
+  : Command(cuid),
+    requestGroup_(requestGroup),
+    e_(e)
 {
   requestGroup_->increaseNumCommand();
 }
@@ -84,37 +82,34 @@ TrackerWatcherCommand::~TrackerWatcherCommand()
 
 bool TrackerWatcherCommand::execute() {
   if(requestGroup_->isForceHaltRequested()) {
-    if(trackerRequestGroup_.isNull()) {
+    if(!trackerRequestGroup_) {
       return true;
     } else if(trackerRequestGroup_->getNumCommand() == 0 ||
               trackerRequestGroup_->downloadFinished()) {
       return true;
     } else {
       trackerRequestGroup_->setForceHaltRequested(true);
+      e_->setRefreshInterval(0);
       e_->addCommand(this);
       return false;
     }
   }
   if(btAnnounce_->noMoreAnnounce()) {
-    if(getLogger()->debug()) {
-      getLogger()->debug("no more announce");
-    }
+    A2_LOG_DEBUG("no more announce");
     return true;
   }
-  if(trackerRequestGroup_.isNull()) {
+  if(!trackerRequestGroup_) {
     trackerRequestGroup_ = createAnnounce();
-    if(!trackerRequestGroup_.isNull()) {
+    if(trackerRequestGroup_) {
       try {
         std::vector<Command*>* commands = new std::vector<Command*>();
         auto_delete_container<std::vector<Command*> > commandsDel(commands);
         trackerRequestGroup_->createInitialCommand(*commands, e_);
         e_->addCommand(*commands);
         commands->clear();
-        if(getLogger()->debug()) {
-          getLogger()->debug("added tracker request command");
-        }
+        A2_LOG_DEBUG("added tracker request command");
       } catch(RecoverableException& ex) {
-        getLogger()->error(EX_EXCEPTION_CAUGHT, ex);
+        A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
       }
     }
   } else if(trackerRequestGroup_->downloadFinished()){
@@ -125,7 +120,7 @@ bool TrackerWatcherCommand::execute() {
       btAnnounce_->announceSuccess();
       btAnnounce_->resetAnnounce();
     } catch(RecoverableException& ex) {
-      getLogger()->error(EX_EXCEPTION_CAUGHT, ex);      
+      A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);      
       btAnnounce_->announceFailure();
       if(btAnnounce_->isAllAnnounceFailed()) {
         btAnnounce_->resetAnnounce();
@@ -170,7 +165,7 @@ void TrackerWatcherCommand::processTrackerResponse
      trackerResponse.size());
   while(!btRuntime_->isHalt() && btRuntime_->lessThanMinPeers()) {
     SharedHandle<Peer> peer = peerStorage_->getUnusedPeer();
-    if(peer.isNull()) {
+    if(!peer) {
       break;
     }
     peer->usedBy(e_->newCUID());
@@ -180,11 +175,9 @@ void TrackerWatcherCommand::processTrackerResponse
     command->setPeerStorage(peerStorage_);
     command->setPieceStorage(pieceStorage_);
     e_->addCommand(command);
-    if(getLogger()->debug()) {
-      getLogger()->debug("CUID#%s - Adding new command CUID#%s",
-                         util::itos(getCuid()).c_str(),
-                         util::itos(peer->usedBy()).c_str());
-    }
+    A2_LOG_DEBUG(fmt("CUID#%lld - Adding new command CUID#%s",
+                     getCuid(),
+                     util::itos(peer->usedBy()).c_str()));
   }
 }
 
@@ -197,7 +190,8 @@ SharedHandle<RequestGroup> TrackerWatcherCommand::createAnnounce() {
   return rg;
 }
 
-static bool backupTrackerIsAvailable
+namespace {
+bool backupTrackerIsAvailable
 (const SharedHandle<DownloadContext>& context)
 {
   SharedHandle<TorrentAttribute> torrentAttrs =
@@ -214,6 +208,7 @@ static bool backupTrackerIsAvailable
     return false;
   }
 }
+} // namespace
 
 SharedHandle<RequestGroup>
 TrackerWatcherCommand::createRequestGroup(const std::string& uri)
@@ -222,13 +217,9 @@ TrackerWatcherCommand::createRequestGroup(const std::string& uri)
   uris.push_back(uri);
   SharedHandle<RequestGroup> rg(new RequestGroup(getOption()));
   if(backupTrackerIsAvailable(requestGroup_->getDownloadContext())) {
-    if(getLogger()->debug()) {
-      getLogger()->debug("This is multi-tracker announce.");
-    }
+    A2_LOG_DEBUG("This is multi-tracker announce.");
   } else {
-    if(getLogger()->debug()) {
-      getLogger()->debug("This is single-tracker announce.");
-    }
+    A2_LOG_DEBUG("This is single-tracker announce.");
   }
   rg->setNumConcurrentCommand(1);
   // If backup tracker is available, try 2 times for each tracker
@@ -236,19 +227,18 @@ TrackerWatcherCommand::createRequestGroup(const std::string& uri)
   rg->getOption()->put(PREF_MAX_TRIES, "2");
   // TODO When dry-run mode becomes available in BitTorrent, set
   // PREF_DRY_RUN=false too.
-  rg->getOption()->put(PREF_USE_HEAD, V_FALSE);
+  rg->getOption()->put(PREF_USE_HEAD, A2_V_FALSE);
   // Setting tracker timeouts
   rg->setTimeout(rg->getOption()->getAsInt(PREF_BT_TRACKER_TIMEOUT));
   rg->getOption()->put(PREF_CONNECT_TIMEOUT,
                        rg->getOption()->get(PREF_BT_TRACKER_CONNECT_TIMEOUT));
-  rg->getOption()->put(PREF_REUSE_URI, V_FALSE);
-  rg->getOption()->put(PREF_SELECT_LEAST_USED_HOST, V_FALSE);
+  rg->getOption()->put(PREF_REUSE_URI, A2_V_FALSE);
+  rg->getOption()->put(PREF_SELECT_LEAST_USED_HOST, A2_V_FALSE);
   static const std::string TRACKER_ANNOUNCE_FILE("[tracker.announce]");
   SharedHandle<DownloadContext> dctx
-    (new DownloadContext(getOption()->getAsInt(PREF_SEGMENT_SIZE),
+    (new DownloadContext(getOption()->getAsInt(PREF_PIECE_LENGTH),
                          0,
                          TRACKER_ANNOUNCE_FILE));
-  dctx->setDir(A2STR::NIL);
   dctx->getFileEntries().front()->setUris(uris);
   rg->setDownloadContext(dctx);
   SharedHandle<DiskWriterFactory> dwf(new ByteArrayDiskWriterFactory());
@@ -256,10 +246,8 @@ TrackerWatcherCommand::createRequestGroup(const std::string& uri)
   rg->setFileAllocationEnabled(false);
   rg->setPreLocalFileCheckEnabled(false);
   util::removeMetalinkContentTypes(rg);
-  if(getLogger()->info()) {
-    getLogger()->info("Creating tracker request group GID#%s",
-                      util::itos(rg->getGID()).c_str());
-  }
+  A2_LOG_INFO(fmt("Creating tracker request group GID#%s",
+                  util::itos(rg->getGID()).c_str()));
   return rg;
 }
 
