@@ -39,13 +39,22 @@
 #include "PeerStorage.h"
 #include "DlAbortEx.h"
 #include "message.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "bencode2.h"
 #include "a2functional.h"
 #include "wallclock.h"
 
 namespace aria2 {
 
+namespace {
+
+// This is the hard limit of the number of "fresh peer" and "dropped
+// peer".  This number is treated as the sum of IPv4 and IPv6 peers.
+const size_t DEFAULT_MAX_FRESH_PEER = 50;
+const size_t DEFAULT_MAX_DROPPED_PEER = 50;
+
+} // namespace
+
 const std::string UTPexExtensionMessage::EXTENSION_NAME = "ut_pex";
 
 UTPexExtensionMessage::UTPexExtensionMessage(uint8_t extensionMessageID):
@@ -100,10 +109,10 @@ UTPexExtensionMessage::createCompactPeerListAndFlag
       (compact, (*itr)->getIPAddress(), (*itr)->getPort());
     if(compactlen == COMPACT_LEN_IPV4) {
       addrstring.append(&compact[0], &compact[compactlen]);
-      flagstring += (*itr)->isSeeder() ? 0x02 : 0x00;
+      flagstring += (*itr)->isSeeder() ? 0x02u : 0x00u;
     } else if(compactlen == COMPACT_LEN_IPV6) {
       addrstring6.append(&compact[0], &compact[compactlen]);
-      flagstring6 += (*itr)->isSeeder() ? 0x02 : 0x00;
+      flagstring6 += (*itr)->isSeeder() ? 0x02u : 0x00u;
     }
   }
   return std::make_pair(std::make_pair(addrstring, flagstring),
@@ -125,7 +134,7 @@ void UTPexExtensionMessage::doReceivedAction()
 bool UTPexExtensionMessage::addFreshPeer(const SharedHandle<Peer>& peer)
 {
   if(!peer->isIncomingPeer() &&
-     peer->getFirstContactTime().difference(global::wallclock) < interval_) {
+     peer->getFirstContactTime().difference(global::wallclock()) < interval_) {
     freshPeers_.push_back(peer);
     return true;
   } else {
@@ -142,7 +151,7 @@ bool UTPexExtensionMessage::addDroppedPeer(const SharedHandle<Peer>& peer)
 {
   if(!peer->isIncomingPeer() &&
      peer->getBadConditionStartTime().
-     difference(global::wallclock) < interval_) {
+     difference(global::wallclock()) < interval_) {
     droppedPeers_.push_back(peer);
     return true;
   } else {
@@ -175,30 +184,31 @@ UTPexExtensionMessageHandle
 UTPexExtensionMessage::create(const unsigned char* data, size_t len)
 {
   if(len < 1) {
-    throw DL_ABORT_EX(StringFormat(MSG_TOO_SMALL_PAYLOAD_SIZE,
-                                   EXTENSION_NAME.c_str(), len).str());
+    throw DL_ABORT_EX(fmt(MSG_TOO_SMALL_PAYLOAD_SIZE,
+                          EXTENSION_NAME.c_str(),
+                          static_cast<unsigned long>(len)));
   }
   UTPexExtensionMessageHandle msg(new UTPexExtensionMessage(*data));
 
   SharedHandle<ValueBase> decoded = bencode2::decode(data+1, len-1);
-  const Dict* dict = asDict(decoded);
+  const Dict* dict = downcast<Dict>(decoded);
   if(dict) {
-    const String* added = asString(dict->get("added"));
+    const String* added = downcast<String>(dict->get("added"));
     if(added) {
       bittorrent::extractPeer
         (added, AF_INET,  std::back_inserter(msg->freshPeers_));
     }
-    const String* dropped = asString(dict->get("dropped"));
+    const String* dropped = downcast<String>(dict->get("dropped"));
     if(dropped) {
       bittorrent::extractPeer
         (dropped, AF_INET, std::back_inserter(msg->droppedPeers_));
     }
-    const String* added6 = asString(dict->get("added6"));
+    const String* added6 = downcast<String>(dict->get("added6"));
     if(added6) {
       bittorrent::extractPeer
         (added6, AF_INET6, std::back_inserter(msg->freshPeers_));
     }
-    const String* dropped6 = asString(dict->get("dropped6"));
+    const String* dropped6 = downcast<String>(dict->get("dropped6"));
     if(dropped6) {
       bittorrent::extractPeer
         (dropped6, AF_INET6, std::back_inserter(msg->droppedPeers_));
