@@ -81,7 +81,8 @@ bool UnknownLengthPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
 void UnknownLengthPieceStorage::getMissingPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const SharedHandle<Peer>& peer,
+ cuid_t cuid)
 {
   abort();
 }
@@ -90,15 +91,8 @@ void UnknownLengthPieceStorage::getMissingPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
  const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
-{
-  abort();
-}
-
-void UnknownLengthPieceStorage::getMissingFastPiece
-(std::vector<SharedHandle<Piece> >& pieces,
- size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   abort();
 }
@@ -107,18 +101,32 @@ void UnknownLengthPieceStorage::getMissingFastPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
  const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
+ cuid_t cuid)
 {
   abort();
 }
 
-SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+void UnknownLengthPieceStorage::getMissingFastPiece
+(std::vector<SharedHandle<Piece> >& pieces,
+ size_t minMissingBlocks,
+ const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   abort();
 }
 
 SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+(const SharedHandle<Peer>& peer,
+ cuid_t cuid)
+{
+  abort();
+}
+
+SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece
+(const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   abort();
 }
@@ -129,13 +137,16 @@ bool UnknownLengthPieceStorage::hasMissingUnusedPiece()
   abort();
 }
 
-SharedHandle<Piece> UnknownLengthPieceStorage::getSparseMissingUnusedPiece
-(size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length)
+SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece
+(size_t minSplitSize,
+ const unsigned char* ignoreBitfield,
+ size_t length,
+ cuid_t cuid)
 {
   if(downloadFinished_) {
     return SharedHandle<Piece>();
   }
-  if(piece_.isNull()) {
+  if(!piece_) {
     piece_.reset(new Piece());
     return piece_;
   } else {
@@ -143,10 +154,12 @@ SharedHandle<Piece> UnknownLengthPieceStorage::getSparseMissingUnusedPiece
   }
 }
 
-SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece(size_t index)
+SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece
+(size_t index,
+ cuid_t cuid)
 {
   if(index == 0) {
-    return getSparseMissingUnusedPiece(0, 0, 0);
+    return getMissingPiece(0, 0, 0, cuid);
   } else {
     return SharedHandle<Piece>();
   }
@@ -155,7 +168,7 @@ SharedHandle<Piece> UnknownLengthPieceStorage::getMissingPiece(size_t index)
 SharedHandle<Piece> UnknownLengthPieceStorage::getPiece(size_t index)
 {
   if(index == 0) {
-    if(piece_.isNull()) {
+    if(!piece_) {
       return SharedHandle<Piece>(new Piece());
     } else {
       return piece_;
@@ -167,7 +180,7 @@ SharedHandle<Piece> UnknownLengthPieceStorage::getPiece(size_t index)
 
 void UnknownLengthPieceStorage::completePiece(const SharedHandle<Piece>& piece)
 {
-  if(piece_ == piece) {
+  if(*piece_ == *piece) {
     downloadFinished_ = true;
     totalLength_ = piece_->getLength();
     diskAdaptor_->setTotalLength(totalLength_);
@@ -175,9 +188,11 @@ void UnknownLengthPieceStorage::completePiece(const SharedHandle<Piece>& piece)
   }
 }
 
-void UnknownLengthPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
+void UnknownLengthPieceStorage::cancelPiece
+(const SharedHandle<Piece>& piece,
+ cuid_t cuid)
 {
-  if(piece_ == piece) {
+  if(*piece_ == *piece) {
     piece_.reset();
   }
 }
@@ -193,7 +208,7 @@ bool UnknownLengthPieceStorage::hasPiece(size_t index)
 
 bool UnknownLengthPieceStorage::isPieceUsed(size_t index)
 {
-  if(index == 0 && !piece_.isNull()) {
+  if(index == 0 && piece_) {
     return true;
   } else {
     return false;
@@ -216,7 +231,7 @@ size_t UnknownLengthPieceStorage::getPieceLength(size_t index)
 
 void UnknownLengthPieceStorage::markAllPiecesDone()
 {
-  if(!piece_.isNull()) {
+  if(piece_) {
     totalLength_ = piece_->getLength();
     piece_.reset();
   }
