@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_A2_ALGO_H_
-#define _D_A2_ALGO_H_
+#ifndef D_A2_ALGO_H
+#define D_A2_ALGO_H
 
 #include <algorithm>
 
@@ -120,4 +120,4 @@ InputIterator find_wrap_if
 
 } // namespace aria2
 
-#endif // _D_A2_ALGO_H_
+#endif // D_A2_ALGO_H
