@@ -206,6 +206,33 @@ public:
   }
 };
 
+template<typename T>
+class auto_delete_d {
+private:
+  T obj_;
+public:
+  auto_delete_d(T obj):obj_(obj) {}
+
+  ~auto_delete_d()
+  {
+    delete obj_;
+  }
+};
+
+template<typename T, typename R>
+class auto_delete_r {
+private:
+  T obj_;
+  R (*deleter_)(T);
+public:
+  auto_delete_r(T obj, R (*deleter)(T)):obj_(obj), deleter_(deleter) {}
+
+  ~auto_delete_r()
+  {
+    (void)deleter_(obj_);
+  }
+};
+
 template<class Container>
 class auto_delete_container {
 private:
@@ -431,6 +458,39 @@ bool in(T x, S s, S t)
   return s <= x && x <= t;
 }
 
+template<typename T>
+struct DerefLess {
+  bool operator()(const T& lhs, const T& rhs) const
+  {
+    return *lhs < *rhs;
+  }
+};
+
+template<typename T>
+struct DerefEqualTo {
+  bool operator()(const T& lhs, const T& rhs) const
+  {
+    return *lhs == *rhs;
+  }
+};
+
+template<typename T>
+struct DerefEqual {
+  T target;
+
+  DerefEqual(const T& t):target(t) {}
+  bool operator()(const T& other) const
+  {
+    return *target == *other;
+  }
+};
+
+template<typename T>
+struct DerefEqual<T> derefEqual(const T& t)
+{
+  return DerefEqual<T>(t);
+}
+
 } // namespace aria2
 
 #endif // D_A2_FUNCTIONAL_H
