@@ -35,7 +35,6 @@
 #include "download_helper.h"
 
 #include <iostream>
-#include <fstream>
 #include <algorithm>
 #include <sstream>
 
@@ -169,7 +168,10 @@ const std::set<std::string>& listRequestOptions()
     PREF_RETRY_WAIT,
     PREF_METALINK_BASE_URI,
     PREF_PAUSE,
-    PREF_STREAM_PIECE_SELECTOR
+    PREF_STREAM_PIECE_SELECTOR,
+    PREF_HASH_CHECK_ONLY,
+    PREF_CHECKSUM,
+    PREF_PIECE_LENGTH
   };
   static std::set<std::string> requestOptions
     (vbegin(REQUEST_OPTIONS), vend(REQUEST_OPTIONS));
@@ -222,13 +224,23 @@ SharedHandle<RequestGroup> createRequestGroup
   SharedHandle<RequestGroup> rg(new RequestGroup(option));
   SharedHandle<DownloadContext> dctx
     (new DownloadContext
-     (option->getAsInt(PREF_SEGMENT_SIZE),
+     (option->getAsInt(PREF_PIECE_LENGTH),
       0,
       useOutOption&&!option->blank(PREF_OUT)?
       util::applyDir(option->get(PREF_DIR), option->get(PREF_OUT)):A2STR::NIL));
   dctx->getFirstFileEntry()->setUris(uris);
   dctx->getFirstFileEntry()->setMaxConnectionPerServer
     (option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER));
+#ifdef ENABLE_MESSAGE_DIGEST
+  const std::string& checksum = option->get(PREF_CHECKSUM);
+  if(!checksum.empty()) {
+    std::pair<std::string, std::string> p;
+    util::divide(p, checksum, '=');
+    util::lowercase(p.first);
+    util::lowercase(p.second);
+    dctx->setDigest(p.first, p.second);
+  }
+#endif // ENABLE_MESSAGE_DIGEST
   rg->setDownloadContext(dctx);
   rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
   removeOneshotOption(rg->getOption());
@@ -506,9 +518,9 @@ namespace {
 void createRequestGroupForUriList
 (std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
- std::istream& in)
+ const std::string& filename)
 {
-  UriListParser p(in);
+  UriListParser p(filename);
   while(p.hasNext()) {
     std::vector<std::string> uris;
     SharedHandle<Option> tempOption(new Option());
@@ -518,6 +530,7 @@ void createRequestGroupForUriList
     }
 
     SharedHandle<Option> requestOption(new Option(*option.get()));
+    requestOption->remove(PREF_OUT);
     for(std::set<std::string>::const_iterator i =
           listRequestOptions().begin(), eoi = listRequestOptions().end();
         i != eoi; ++i) {
@@ -536,15 +549,14 @@ void createRequestGroupForUriList
  const SharedHandle<Option>& option)
 {
   if(option->get(PREF_INPUT_FILE) == "-") {
-    createRequestGroupForUriList(result, option, std::cin);
+    createRequestGroupForUriList(result, option, DEV_STDIN);
   } else {
     if(!File(option->get(PREF_INPUT_FILE)).isFile()) {
       throw DL_ABORT_EX
         (fmt(EX_FILE_OPEN, option->get(PREF_INPUT_FILE).c_str(),
              "No such file"));
     }
-    std::ifstream f(option->get(PREF_INPUT_FILE).c_str(), std::ios::binary);
-    createRequestGroupForUriList(result, option, f);
+    createRequestGroupForUriList(result, option, option->get(PREF_INPUT_FILE));
   }
 }
 
