@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DOWNLOAD_HELPER_H_
-#define _D_DOWNLOAD_HELPER_H_
+#ifndef D_DOWNLOAD_HELPER_H
+#define D_DOWNLOAD_HELPER_H
 
 #include "common.h"
 
@@ -87,13 +87,18 @@ void createRequestGroupForUriList
  const SharedHandle<Option>& option);
 
 // Create RequestGroup object using provided uris.  If ignoreLocalPath
-// is true, a path to torrent file abd metalink file are ignored.
+// is true, a path to torrent file abd metalink file are ignored.  If
+// throwOnError is true, exception will be thrown when Metalink
+// Document or .torrent file cannot be parsed or unrecognized URI is
+// given. If throwOnError is false, these errors are just logged as
+// error.
 void createRequestGroupForUri
 (std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
  const std::vector<std::string>& uris,
  bool ignoreForceSequential = false,
- bool ignoreLocalPath = false);
+ bool ignoreLocalPath = false,
+ bool throwOnError = false);
 
 template<typename InputIterator>
 void setMetadataInfo
@@ -107,6 +112,10 @@ void setMetadataInfo
 SharedHandle<MetadataInfo>
 createMetadataInfoFromFirstFileEntry(const SharedHandle<DownloadContext>& dctx);
 
+// Removes option value which is only effective at the first
+// construction time.
+void removeOneshotOption(const SharedHandle<Option>& option);
+
 } // namespace aria2
 
-#endif // _D_DOWNLOAD_HELPER_H_
+#endif // D_DOWNLOAD_HELPER_H
