@@ -45,6 +45,8 @@ const std::string A2_V_DEFAULT("default");
 const std::string V_NONE("none");
 const std::string V_MEM("mem");
 const std::string V_ALL("all");
+const std::string A2_V_FULL("full");
+const std::string A2_V_GEOM("geom");
 
 /**
  * General preferences
@@ -74,7 +76,7 @@ const std::string PREF_REFERER("referer");
 // value: 1*digit
 const std::string PREF_LOWEST_SPEED_LIMIT("lowest-speed-limit");
 // value: 1*digit
-const std::string PREF_SEGMENT_SIZE("segment-size");
+const std::string PREF_PIECE_LENGTH("piece-length");
 // value: 1*digit
 const std::string PREF_MAX_OVERALL_DOWNLOAD_LIMIT("max-overall-download-limit");
 // value: 1*digit
@@ -169,17 +171,7 @@ const std::string PREF_RPC_MAX_REQUEST_SIZE("rpc-max-request-size");
 // value: true | false
 const std::string PREF_RPC_LISTEN_ALL("rpc-listen-all");
 // value: true | false
-const std::string PREF_ENABLE_XML_RPC("enable-xml-rpc");
-// value: 1*digit
-const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
-// value: string
-const std::string PREF_XML_RPC_USER("xml-rpc-user");
-// value: string
-const std::string PREF_XML_RPC_PASSWD("xml-rpc-passwd");
-// value: 1*digit
-const std::string PREF_XML_RPC_MAX_REQUEST_SIZE("xml-rpc-max-request-size");
-// value: true | false
-const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
+const std::string PREF_RPC_ALLOW_ORIGIN_ALL("rpc-allow-origin-all");
 // value: true | false
 const std::string PREF_DRY_RUN("dry-run");
 // value: true | false
@@ -228,6 +220,12 @@ const std::string PREF_STREAM_PIECE_SELECTOR("stream-piece-selector");
 const std::string PREF_TRUNCATE_CONSOLE_READOUT("truncate-console-readout");
 // value: true | false
 const std::string PREF_PAUSE("pause");
+// value: default | full
+const std::string PREF_DOWNLOAD_RESULT("download-result");
+// value: true | false
+const std::string PREF_HASH_CHECK_ONLY("hash-check-only");
+// values: hashType=digest
+const std::string PREF_CHECKSUM("checksum");
 
 /**
  * FTP related preferences
