@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_PREFS_H_
-#define _D_PREFS_H_
+#ifndef D_PREFS_H
+#define D_PREFS_H
 
 #include "common.h"
 #include <string>
@@ -43,11 +43,15 @@ namespace aria2 {
 /**
  * Constants
  */
-extern const std::string V_TRUE;
-extern const std::string V_FALSE;
+extern const std::string A2_V_TRUE;
+extern const std::string A2_V_FALSE;
+extern const std::string A2_V_DEFAULT;
 extern const std::string V_NONE;
 extern const std::string V_MEM;
 extern const std::string V_ALL;
+extern const std::string A2_V_FULL;
+extern const std::string A2_V_GEOM;
+
 /**
  * General preferences
  */
@@ -76,7 +80,7 @@ extern const std::string PREF_REFERER;
 // value: 1*digit
 extern const std::string PREF_LOWEST_SPEED_LIMIT;
 // value: 1*digit
-extern const std::string PREF_SEGMENT_SIZE;
+extern const std::string PREF_PIECE_LENGTH;
 // value: 1*digit
 extern const std::string PREF_MAX_DOWNLOAD_LIMIT;
 // value: 1*digit
@@ -158,28 +162,30 @@ extern const std::string V_KQUEUE;
 extern const std::string V_PORT;
 extern const std::string V_POLL;
 extern const std::string V_SELECT;
-// value: 1*digit
-extern const std::string PREF_XML_RPC_LISTEN_PORT;
 // value: true | false
-extern const std::string PREF_ENABLE_XML_RPC;
+extern const std::string PREF_ENABLE_RPC;
+// value: 1*digit
+extern const std::string PREF_RPC_LISTEN_PORT;
+// value: string
+extern const std::string PREF_RPC_USER;
+// value: string
+extern const std::string PREF_RPC_PASSWD;
+// value: 1*digit
+extern const std::string PREF_RPC_MAX_REQUEST_SIZE;
+// value: true | false
+extern const std::string PREF_RPC_LISTEN_ALL;
+// value: true | false
+extern const std::string PREF_RPC_ALLOW_ORIGIN_ALL;
 // value: true | false
 extern const std::string PREF_DRY_RUN;
 // value: true | false
 extern const std::string PREF_REUSE_URI;
 // value: string
-extern const std::string PREF_XML_RPC_USER;
-// value: string
-extern const std::string PREF_XML_RPC_PASSWD;
-// value: 1*digit
-extern const std::string PREF_XML_RPC_MAX_REQUEST_SIZE;
-// value: string
 extern const std::string PREF_ON_DOWNLOAD_START;
 extern const std::string PREF_ON_DOWNLOAD_PAUSE;
 extern const std::string PREF_ON_DOWNLOAD_STOP;
 extern const std::string PREF_ON_DOWNLOAD_COMPLETE;
 extern const std::string PREF_ON_DOWNLOAD_ERROR;
-// value: true | false
-extern const std::string PREF_XML_RPC_LISTEN_ALL;
 // value: string
 extern const std::string PREF_INTERFACE;
 // value: true | false
@@ -202,6 +208,26 @@ extern const std::string PREF_MIN_SPLIT_SIZE;
 extern const std::string PREF_CONDITIONAL_GET;
 // value: true | false
 extern const std::string PREF_SELECT_LEAST_USED_HOST;
+// value: true | false
+extern const std::string PREF_ENABLE_ASYNC_DNS6;
+// value: 1*digit
+extern const std::string PREF_MAX_DOWNLOAD_RESULT;
+// value: 1*digit
+extern const std::string PREF_RETRY_WAIT;
+// value: string
+extern const std::string PREF_ASYNC_DNS_SERVER;
+// value: true | false
+extern const std::string PREF_SHOW_CONSOLE_READOUT;
+// value: default | inorder | geom
+extern const std::string PREF_STREAM_PIECE_SELECTOR;
+// value: true | false
+extern const std::string PREF_TRUNCATE_CONSOLE_READOUT;
+// value: true | false
+extern const std::string PREF_PAUSE;
+// value: default | full
+extern const std::string PREF_DOWNLOAD_RESULT;
+// value: true | false
+extern const std::string PREF_HASH_CHECK_ONLY;
 
 /**
  * FTP related preferences
@@ -216,6 +242,8 @@ extern const std::string V_ASCII;
 extern const std::string PREF_FTP_PASV;
 // values: true | false
 extern const std::string PREF_FTP_REUSE_CONNECTION;
+// values: hashType=digest
+extern const std::string PREF_CHECKSUM;
 
 /**
  * HTTP related preferences
@@ -310,6 +338,8 @@ extern const std::string PREF_PEER_ID_PREFIX;
 extern const std::string PREF_ENABLE_PEER_EXCHANGE;
 // values: true | false
 extern const std::string PREF_ENABLE_DHT;
+// values: a string
+extern const std::string PREF_DHT_LISTEN_ADDR;
 // values: 1*digit
 extern const std::string PREF_DHT_LISTEN_PORT;
 // values: a string
@@ -320,6 +350,18 @@ extern const std::string PREF_DHT_ENTRY_POINT_PORT;
 extern const std::string PREF_DHT_ENTRY_POINT;
 // values: a string
 extern const std::string PREF_DHT_FILE_PATH;
+// values: true | false
+extern const std::string PREF_ENABLE_DHT6;
+// values: a string
+extern const std::string PREF_DHT_LISTEN_ADDR6;
+// values: a string
+extern const std::string PREF_DHT_ENTRY_POINT_HOST6;
+// values: 1*digit
+extern const std::string PREF_DHT_ENTRY_POINT_PORT6;
+// values: a string (hostname:port)
+extern const std::string PREF_DHT_ENTRY_POINT6;
+// values: a string
+extern const std::string PREF_DHT_FILE_PATH6;
 // values: plain | arc4
 extern const std::string PREF_BT_MIN_CRYPTO_LEVEL;
 extern const std::string V_PLAIN;
@@ -362,6 +404,10 @@ extern const std::string PREF_BT_TRACKER_CONNECT_TIMEOUT;
 extern const std::string PREF_DHT_MESSAGE_TIMEOUT;
 // values: string
 extern const std::string PREF_ON_BT_DOWNLOAD_COMPLETE;
+// values: string
+extern const std::string PREF_BT_TRACKER;
+// values: string
+extern const std::string PREF_BT_EXCLUDE_TRACKER;
 
 /**
  * Metalink related preferences
@@ -387,7 +433,9 @@ extern const std::string V_HTTPS;
 extern const std::string V_FTP;
 // values: true | false
 extern const std::string PREF_METALINK_ENABLE_UNIQUE_PROTOCOL;
+// values: a string
+extern const std::string PREF_METALINK_BASE_URI;
 
 } // namespace aria2
 
-#endif // _D_PREFS_H_
+#endif // D_PREFS_H
