@@ -42,7 +42,7 @@
   _(" -l, --log=LOG                The file name of the log file. If '-' is\n" \
     "                              specified, log is written to stdout.")
 #define TEXT_DAEMON                                                     \
-  _(" -D, --daemon                 Run as daemon. The current working directory will\n" \
+  _(" -D, --daemon[=true|false]    Run as daemon. The current working directory will\n" \
     "                              be changed to \"/\" and standard input, standard\n" \
     "                              output and standard error will be redirected to\n" \
     "                              \"/dev/null\".")
@@ -59,8 +59,9 @@
     "                              Please note that in Metalink download, this\n" \
     "                              option has no effect and use -C option instead.")
 #define TEXT_RETRY_WAIT                                                 \
-  _(" --retry-wait=SEC             Set the seconds to wait to retry after an error\n" \
-    "                              has occured.")
+  _(" --retry-wait=SEC             Set the seconds to wait between retries. \n" \
+    "                              With SEC > 0, aria2 will retry download when the\n" \
+    "                              HTTP server returns 503 response.")
 #define TEXT_TIMEOUT                                            \
   _(" -t, --timeout=SEC            Set timeout in seconds.")
 #define TEXT_MAX_TRIES                                                  \
@@ -130,14 +131,15 @@
     "                              This may take some time depending on the size of\n" \
     "                              the file.\n"                         \
     "                              If you are using newer file systems such as ext4\n" \
-    "                              (with extents support), btrfs or xfs, 'falloc' is\n" \
-    "                              your best choice. It allocates large(few GiB)\n" \
-    "                              files almost instantly. Don't use 'falloc' with\n" \
-    "                              legacy file systems such as ext3 because it takes\n" \
-    "                              almost same time as 'prealloc' and it blocks aria2\n" \
-    "                              entirely until allocation finishes. 'falloc' may\n" \
-    "                              not be available if your system doesn't have\n" \
-    "                              posix_fallocate() function.")
+    "                              (with extents support), btrfs, xfs or NTFS\n" \
+    "                              (MinGW build only), 'falloc' is your best\n"   \
+    "                              choice. It allocates large(few GiB) files\n" \
+    "                              almost instantly. Don't use 'falloc' with legacy\n" \
+    "                              file systems such as ext3 and FAT32 because it\n" \
+    "                              takes almost same time as 'prealloc' and it\n" \
+    "                              blocks aria2 entirely until allocation finishes.\n" \
+    "                              'falloc' may not be available if your system\n" \
+    "                              doesn't have posix_fallocate() function.")
 #define TEXT_NO_FILE_ALLOCATION_LIMIT                                   \
   _(" --no-file-allocation-limit=SIZE No file allocation is made for files whose\n" \
     "                              size is smaller than SIZE.\n"        \
@@ -147,14 +149,14 @@
     "                              allocating files.\n"                 \
     "                              Turn off if you encounter any error")
 #define TEXT_ALLOW_OVERWRITE                                            \
-  _(" --allow-overwrite=true|false Restart download from scratch if the\n" \
+  _(" --allow-overwrite[=true|false] Restart download from scratch if the\n" \
     "                              corresponding control file doesn't exist.  See\n" \
     "                              also --auto-file-renaming option.")
 #define TEXT_ALLOW_PIECE_LENGTH_CHANGE                                  \
-  _(" --allow-piece-length-change=true|false If false is given, aria2 aborts download\n" \
-    "                              when a piece length is different from one in\n" \
-    "                              a control file. If true is given, you can proceed\n" \
-    "                              but some download progress will be lost.")  
+  _(" --allow-piece-length-change[=true|false] If false is given, aria2 aborts\n" \
+    "                              download when a piece length is different from\n" \
+    "                              one in a control file. If true is given, you can\n" \
+    "                              proceed but some download progress will be lost.")  
 #define TEXT_FORCE_SEQUENTIAL                                           \
   _(" -Z, --force-sequential[=true|false] Fetch URIs in the command-line sequentially\n" \
     "                              and download each URI in a separate session, like\n" \
@@ -182,10 +184,18 @@
   _(" --enable-http-pipelining[=true|false] Enable HTTP/1.1 pipelining.")
 #define TEXT_CHECK_INTEGRITY                                            \
   _(" -V, --check-integrity[=true|false] Check file integrity by validating piece\n" \
-    "                              hashes. This option has effect only in BitTorrent\n" \
-    "                              and Metalink downloads with chunk checksums.\n" \
-    "                              Use this option to re-download a damaged portion\n" \
-    "                              of a file. See also --bt-hash-check-seed option.")
+    "                              hashes or a hash of entire file. This option has\n" \
+    "                              effect only in BitTorrent, Metalink downloads\n" \
+    "                              with checksums or HTTP(S)/FTP downloads with\n" \
+    "                              --checksum option. If piece hashes are provided,\n" \
+    "                              this option can detect damaged portions of a file\n" \
+    "                              and re-download them. If a hash of entire file is\n" \
+    "                              provided, hash check is only done when file has\n" \
+    "                              been already download. This is determined by file\n" \
+    "                              length. If hash check fails, file is\n" \
+    "                              re-downloaded from scratch. If both piece hashes\n" \
+    "                              and a hash of entire file are provided, only\n" \
+    "                              piece hashes are used.")
 #define TEXT_BT_HASH_CHECK_SEED                                         \
   _(" --bt-hash-check-seed[=true|false] If true is given, after hash check using\n" \
     "                              --check-integrity option and file is complete,\n" \
@@ -195,11 +205,11 @@
     "                              This option has effect only on BitTorrent\n" \
     "                              download.")
 #define TEXT_REALTIME_CHUNK_CHECKSUM                                    \
-  _(" --realtime-chunk-checksum=true|false  Validate chunk of data by calculating\n" \
+  _(" --realtime-chunk-checksum[=true|false]  Validate chunk of data by calculating\n" \
     "                              checksum while downloading a file if chunk\n" \
     "                              checksums are provided.")
 #define TEXT_CONTINUE                                                   \
-  _(" -c, --continue               Continue downloading a partially downloaded\n" \
+  _(" -c, --continue[=true|false]  Continue downloading a partially downloaded\n" \
     "                              file. Use this option to resume a download\n" \
     "                              started by a web browser or another program\n" \
     "                              which downloads files sequentially from the\n" \
@@ -208,16 +218,17 @@
 #define TEXT_USER_AGENT                                                 \
   _(" -U, --user-agent=USER_AGENT  Set user agent for http(s) downloads.")
 #define TEXT_NO_NETRC                                           \
-  _(" -n, --no-netrc               Disables netrc support.")
+  _(" -n, --no-netrc[=true|false]  Disables netrc support.")
 #define TEXT_INPUT_FILE                                                 \
   _(" -i, --input-file=FILE        Downloads URIs found in FILE. You can specify\n" \
     "                              multiple URIs for a single entity: separate\n" \
     "                              URIs on a single line using the TAB character.\n" \
     "                              Reads input from stdin when '-' is specified.\n" \
-    "                              The additional out and dir options can be\n" \
-    "                              specified after each line of URIs. This optional\n" \
-    "                              line must start with white space(s). See INPUT\n" \
-    "                              FILE section of man page for details.")
+    "                              Additionally, options can be specified after each\n" \
+    "                              line of URI. This optional line must start with\n" \
+    "                              one or more white spaces and have one option per\n" \
+    "                              single line. See INPUT FILE section of man page\n" \
+    "                              for details.")
 #define TEXT_MAX_CONCURRENT_DOWNLOADS                                   \
   _(" -j, --max-concurrent-downloads=N Set maximum number of parallel downloads for\n" \
     "                              every static (HTTP/FTP) URL, torrent and metalink.\n" \
@@ -231,9 +242,10 @@
     "                              overwritten. Session Cookies are also saved and\n" \
     "                              their expiry values are treated as 0.")
 #define TEXT_SHOW_FILES                                                 \
-  _(" -S, --show-files             Print file listing of .torrent or .metalink file\n" \
-    "                              and exit. More detailed information will be listed\n" \
-    "                              in case of torrent file.")
+  _(" -S, --show-files[=true|false] Print file listing of .torrent, .meta4 and\n" \
+    "                              .metalink file and exit. More detailed\n" \
+    "                              information will be listed in case of torrent\n" \
+    "                              file.")
 #define TEXT_SELECT_FILE                                                \
   _(" --select-file=INDEX...       Set file to download by specifying its index.\n" \
     "                              You can find the file index using the\n" \
@@ -255,9 +267,6 @@
     "                              written to the disk, but is just kept in memory.\n" \
     "                              If false is specified, the action mentioned above\n" \
     "                              is not taken.")
-#define TEXT_DIRECT_FILE_MAPPING                                        \
-  _(" --direct-file-mapping=true|false Directly read from and write to each file\n" \
-    "                              mentioned in .torrent file.")
 #define TEXT_LISTEN_PORT                                                \
   _(" --listen-port=PORT...        Set TCP port number for BitTorrent downloads.\n" \
     "                              Multiple ports can be specified by using ',',\n" \
@@ -315,7 +324,7 @@
     "                              peer, aria2 chooses the lowest one which satisfies\n" \
     "                              the given level.")
 #define TEXT_BT_REQUIRE_CRYPTO                                          \
-  _(" --bt-require-crypto=true|false If true is given, aria2 doesn't accept and\n" \
+  _(" --bt-require-crypto[=true|false] If true is given, aria2 doesn't accept and\n" \
     "                              establish connection with legacy BitTorrent\n" \
     "                              handshake. Thus aria2 always uses Obfuscation\n" \
     "                              handshake.")
@@ -338,8 +347,9 @@
     "                              0 means unlimited.\n"                \
     "                              See also --bt-request-peer-speed-limit option.")
 #define TEXT_METALINK_FILE                                              \
-  _(" -M, --metalink-file=METALINK_FILE The file path to the .metalink file. Reads\n" \
-    "                              input from stdin when '-' is specified.")
+  _(" -M, --metalink-file=METALINK_FILE The file path to the .meta4 and .metalink\n" \
+    "                              file. Reads input from stdin when '-' is\n" \
+    "                              specified.")
 #define TEXT_METALINK_SERVERS                                           \
   _(" -C, --metalink-servers=NUM_SERVERS The number of servers to connect to\n" \
     "                              simultaneously. Some Metalinks regulate the\n" \
@@ -364,7 +374,8 @@
     "                              if you don't have any preferred protocol.")
 #define TEXT_FOLLOW_METALINK                                            \
   _(" --follow-metalink=true|false|mem If true or mem is specified, when a file\n" \
-    "                              whose suffix is .metalink or content type of\n" \
+    "                              whose suffix is .meta4 or .metalink, or content\n" \
+    "                              type of application/metalink4+xml or\n" \
     "                              application/metalink+xml is downloaded, aria2\n" \
     "                              parses it as a metalink file and downloads files\n" \
     "                              mentioned in it.\n"                  \
@@ -373,7 +384,7 @@
     "                              If false is specified, the action mentioned above\n" \
     "                              is not taken.")
 #define TEXT_METALINK_ENABLE_UNIQUE_PROTOCOL                            \
-  _(" --metalink-enable-unique-protocol=true|false If true is given and several\n" \
+  _(" --metalink-enable-unique-protocol[=true|false] If true is given and several\n" \
     "                              protocols are available for a mirror in a metalink\n" \
     "                              file, aria2 uses one of them.\n"     \
     "                              Use --metalink-preferred-protocol option to\n" \
@@ -388,7 +399,7 @@
     "                              \"#http\". If non-tag word is given, print the usage\n" \
     "                              for the options whose name includes that word.")
 #define TEXT_NO_CONF                                                    \
-  _(" --no-conf                    Disable loading aria2.conf file.")
+  _(" --no-conf[=true|false]       Disable loading aria2.conf file.")
 #define TEXT_CONF_PATH                                                  \
   _(" --conf-path=PATH             Change the configuration file path to PATH.")
 #define TEXT_STOP                                                       \
@@ -491,22 +502,6 @@
     "                              server.")
 #define TEXT_EVENT_POLL                                                 \
   _(" --event-poll=POLL            Specify the method for polling events.")
-#define TEXT_XML_RPC_LISTEN_PORT                                        \
-  _(" --xml-rpc-listen-port=PORT   Specify a port number for XML-RPC server to listen\n" \
-    "                              to.")
-#define TEXT_ENABLE_XML_RPC                                             \
-  _(" --enable-xml-rpc[=true|false] Enable XML-RPC server.\n"           \
-    "                              It is strongly recommended to set username and\n" \
-    "                              password using --xml-rpc-user and --xml-rpc-passwd\n" \
-    "                              option. See also --xml-rpc-listen-port option.")
-#define TEXT_XML_RPC_MAX_REQUEST_SIZE                                   \
-  _(" --xml-rpc-max-request-size=SIZE Set max size of XML-RPC request. If aria2\n" \
-    "                              detects the request is more than SIZE bytes, it\n" \
-    "                              drops connection.")
-#define TEXT_XML_RPC_USER                               \
-  _(" --xml-rpc-user=USER          Set XML-RPC user.")
-#define TEXT_XML_RPC_PASSWD                                     \
-  _(" --xml-rpc-passwd=PASSWD      Set XML-RPC password.")
 #define TEXT_BT_EXTERNAL_IP                                             \
   _(" --bt-external-ip=IPADDRESS   Specify the external IP address to report to a\n" \
     "                              BitTorrent tracker. Although this function is\n" \
@@ -541,30 +536,31 @@
     "                              based on the response of tracker and the download\n" \
     "                              progress.")
 #define TEXT_ON_DOWNLOAD_COMPLETE                                       \
-  _(" --on-download-complete=COMMAND Set the command to be executed when download\n" \
-    "                              completes.\n"                        \
+  _(" --on-download-complete=COMMAND Set the command to be executed after download\n" \
+    "                              completed.\n"                        \
     "                              See --on-download-start option for the\n" \
     "                              requirement of COMMAND.\n"           \
     "                              See also --on-download-stop option.")
 #define TEXT_ON_DOWNLOAD_START                                          \
-  _(" --on-download-start=COMMAND  Set the command to be executed when download\n" \
-    "                              starts up. COMMAND must take just one argument and\n" \
-    "                              GID is passed to COMMAND as a first argument.")
+  _(" --on-download-start=COMMAND  Set the command to be executed after download\n" \
+    "                              got started. aria2 passes 3 arguments to COMMAND:\n" \
+    "                              GID, the nubmer of files and file path. See Event\n" \
+    "                              Hook in man page for more details.")
 #define TEXT_ON_DOWNLOAD_PAUSE                                          \
-  _(" --on-download-pause=COMMAND  Set the command to be executed when download\n" \
-    "                              is paused.\n"\
+  _(" --on-download-pause=COMMAND  Set the command to be executed after download\n" \
+    "                              was paused.\n"\
     "                              See --on-download-start option for the\n" \
     "                              requirement of COMMAND.")
 #define TEXT_ON_DOWNLOAD_ERROR                                          \
-  _(" --on-download-error=COMMAND  Set the command to be executed when download\n" \
-    "                              aborts due to error.\n"              \
+  _(" --on-download-error=COMMAND  Set the command to be executed after download\n" \
+    "                              aborted due to error.\n"              \
     "                              See --on-download-start option for the\n" \
     "                              requirement of COMMAND.\n"           \
     "                              See also --on-download-stop option.")
 #define TEXT_ON_DOWNLOAD_STOP                                           \
-  _(" --on-download-stop=COMMAND   Set the command to be executed when download\n" \
-    "                              stops. You can override the command to be executed\n" \
-    "                              for particular download result using\n" \
+  _(" --on-download-stop=COMMAND   Set the command to be executed after download\n" \
+    "                              stopped. You can override the command to be\n" \
+    "                              executed for particular download result using\n" \
     "                              --on-download-complete and --on-download-error. If\n" \
     "                              they are specified, command specified in this\n" \
     "                              option is not executed.\n"           \
@@ -574,10 +570,6 @@
   _(" --bt-stop-timeout=SEC        Stop BitTorrent download if download speed is 0 in\n" \
     "                              consecutive SEC seconds. If 0 is given, this\n" \
     "                              feature is disabled.")
-#define TEXT_XML_RPC_LISTEN_ALL                                         \
-  _(" --xml-rpc-listen-all[=true|false] Listen incoming XML-RPC requests on all\n" \
-    "                              network interfaces. If false is given, listen only\n" \
-    "                              on local loopback interface.")
 #define TEXT_BT_PRIORITIZE_PIECE                                        \
   _(" --bt-prioritize-piece=head[=SIZE],tail[=SIZE] Try to download first and last\n" \
     "                              pieces of each file first. This is useful for\n" \
@@ -685,7 +677,10 @@
     "                              You can pass this output file to aria2c with -i\n" \
     "                              option on restart. Please note that downloads\n" \
     "                              added by aria2.addTorrent and aria2.addMetalink\n" \
-    "                              XML-RPC method are not saved.")
+    "                              RPC method and whose metadata could not be saved\n" \
+    "                              as a file will not be saved. Downloads removed\n" \
+    "                              using aria2.remove and aria2.forceRemove will not\n" \
+    "                              be saved.")
 #define TEXT_MAX_CONNECTION_PER_SERVER          \
   _(" -x, --max-connection-per-server=NUM The maximum number of connections to one\n" \
     "                              server for each download.")
@@ -704,10 +699,10 @@
     "                              many limitations. See man page for details.")
 #define TEXT_ON_BT_DOWNLOAD_COMPLETE            \
   _(" --on-bt-download-complete=COMMAND For BitTorrent, a command specified in\n" \
-    "                              --on-download-complete is called when download\n" \
-    "                              completes and seeding is over. On the other hand,\n" \
-    "                              this option sets the command to be executed when\n" \
-    "                              download completes but before seeding.\n" \
+    "                              --on-download-complete is called after download\n" \
+    "                              completed and seeding is over. On the other hand,\n" \
+    "                              this option sets the command to be executed after\n" \
+    "                              download completed but before seeding.\n" \
     "                              See --on-download-start option for the\n" \
     "                              requirement of COMMAND.")
 #define TEXT_ENABLE_ASYNC_DNS6                  \
@@ -753,3 +748,109 @@
     "                              option may result high memory consumption after\n" \
     "                              thousands of downloads. Specifying 0 means no\n" \
     "                              download result is kept.")
+#define TEXT_ASYNC_DNS_SERVER                   \
+  _(" --async-dns-server=IPADDRESS[,...] Comma separated list of DNS server address\n" \
+    "                              used in asynchronous DNS resolver. Usually\n" \
+    "                              asynchronous DNS resolver reads DNS server\n" \
+    "                              addresses from /etc/resolv.conf. When this option\n" \
+    "                              is used, it uses DNS servers specified in this\n" \
+    "                              option instead of ones in /etc/resolv.conf. You\n" \
+    "                              can specify both IPv4 and IPv6 address. This\n" \
+    "                              option is useful when the system does not have\n" \
+    "                              /etc/resolv.conf and user does not have the\n" \
+    "                              permission to create it.")
+#define TEXT_ENABLE_RPC                                             \
+  _(" --enable-rpc[=true|false]    Enable JSON-RPC/XML-RPC server.\n"   \
+    "                              It is strongly recommended to set username and\n" \
+    "                              password using --rpc-user and --rpc-passwd\n" \
+    "                              option. See also --rpc-listen-port option.")
+#define TEXT_RPC_MAX_REQUEST_SIZE                                   \
+  _(" --rpc-max-request-size=SIZE  Set max size of JSON-RPC/XML-RPC request. If aria2\n" \
+    "                              detects the request is more than SIZE bytes, it\n" \
+    "                              drops connection.")
+#define TEXT_RPC_USER                               \
+  _(" --rpc-user=USER              Set JSON-RPC/XML-RPC user.")
+#define TEXT_RPC_PASSWD                                     \
+  _(" --rpc-passwd=PASSWD          Set JSON-RPC/XML-RPC password.")
+#define TEXT_RPC_LISTEN_ALL                                         \
+  _(" --rpc-listen-all[=true|false] Listen incoming JSON-RPC/XML-RPC requests on all\n" \
+    "                              network interfaces. If false is given, listen only\n" \
+    "                              on local loopback interface.")
+#define TEXT_RPC_LISTEN_PORT                                        \
+  _(" --rpc-listen-port=PORT       Specify a port number for JSON-RPC/XML-RPC server\n" \
+    "                              to listen to.")
+#define TEXT_SHOW_CONSOLE_READOUT                                       \
+  _(" --show-console-readout[=true|false] Show console readout.")
+#define TEXT_METALINK_BASE_URI                  \
+  _(" --metalink-base-uri=URI      Specify base URI to resolve relative URI in\n" \
+    "                              metalink:url and metalink:metaurl element in a\n" \
+    "                              metalink file stored in local disk. If URI points\n" \
+    "                              to a directory, URI must end with '/'.")
+#define TEXT_STREAM_PIECE_SELECTOR              \
+  _(" --stream-piece-selector=SELECTOR Specify piece selection algorithm\n" \
+    "                              used in HTTP/FTP download. Piece means fixed\n" \
+    "                              length segment which is downloaded in parallel\n" \
+    "                              in segmented download. If 'default' is given,\n" \
+    "                              aria2 selects piece so that it reduces the\n" \
+    "                              number of establishing connection. This is\n" \
+    "                              reasonable default behaviour because\n" \
+    "                              establishing connection is an expensive\n" \
+    "                              operation.\n"                        \
+    "                              If 'inorder' is given, aria2 selects piece which\n" \
+    "                              has minimum index. Index=0 means first of the\n" \
+    "                              file. This will be useful to view movie while\n" \
+    "                              downloading it. --enable-http-pipelining option\n" \
+    "                              may be useful to reduce reconnection overhead.\n" \
+    "                              Please note that aria2 honors\n"     \
+    "                              --min-split-size option, so it will be necessary\n" \
+    "                              to specify a reasonable value to\n"  \
+    "                              --min-split-size option.\n"          \
+    "                              If 'geom' is given, at the beginning aria2\n" \
+    "                              selects piece which has minimum index like\n" \
+    "                              'inorder', but it exponentially increasingly\n" \
+    "                              keeps space from previously selected piece. This\n" \
+    "                              will reduce the number of establishing connection\n" \
+    "                              and at the same time it will download the\n" \
+    "                              beginning part of the file first. This will be\n" \
+    "                              useful to view movie while downloading it.")
+#define TEXT_TRUNCATE_CONSOLE_READOUT                                   \
+  _(" --truncate-console-readout[=true|false] Truncate console readout to fit in\n"\
+    "                              a single line.")
+#define TEXT_PAUSE                              \
+  _(" --pause[=true|false]         Pause download after added. This option is\n" \
+    "                              effective only when --enable-rpc=true is given.")
+#define TEXT_RPC_ALLOW_ORIGIN_ALL                                       \
+  _(" --rpc-allow-origin-all[=true|false] Add Access-Control-Allow-Origin header\n" \
+    "                              field with value '*' to the RPC response.")
+#define TEXT_DOWNLOAD_RESULT                    \
+  _(" --download-result=OPT        This option changes the way \"Download Results\"\n" \
+    "                              is formatted. If OPT is 'default', print GID,\n" \
+    "                              status, average download speed and path/URI. If\n" \
+    "                              multiple files are involved, path/URI of first\n" \
+    "                              requested file is printed and remaining ones are\n" \
+    "                              omitted.\n"                          \
+    "                              If OPT is 'full', print GID, status, average\n" \
+    "                              download speed, percentage of progress and\n" \
+    "                              path/URI. The percentage of progress and\n" \
+    "                              path/URI are printed for each requested file in\n" \
+    "                              each row.")
+#define TEXT_HASH_CHECK_ONLY                    \
+  _(" --hash-check-only[=true|false] If true is given, after hash check using\n" \
+    "                              --check-integrity option, abort download whether\n" \
+    "                              or not download is complete.")
+#define TEXT_CHECKSUM                                                   \
+  _(" --checksum=TYPE=DIGEST       Set checksum. TYPE is hash type. The supported\n" \
+    "                              hash type is listed in \"Hash Algorithms\" in\n" \
+    "                              \"aria2c -v\". DIGEST is hex digest.\n" \
+    "                              For example, setting sha-1 digest looks like\n" \
+    "                              this:\n"                             \
+    "                              sha-1=0192ba11326fe2298c8cb4de616f4d4140213838\n" \
+    "                              This option applies only to HTTP(S)/FTP\n" \
+    "                              downloads.")
+#define TEXT_PIECE_LENGTH                       \
+  _(" --piece-length=LENGTH        Set a piece length for HTTP/FTP downloads. This\n" \
+    "                              is the boundary when aria2 splits a file. All\n" \
+    "                              splits occur at multiple of this length. This\n" \
+    "                              option will be ignored in BitTorrent downloads.\n" \
+    "                              It will be also ignored if Metalink file\n" \
+    "                              contains piece hashes.")
