@@ -48,20 +48,8 @@
 #include <algorithm>
 #include <fstream>
 #include <iomanip>
-#ifndef HAVE_SLEEP
-# ifdef HAVE_WINSOCK_H
-#  define WIN32_LEAN_AND_MEAN
-#  include <windows.h>
-# endif // HAVE_WINSOCK_H
-#endif // HAVE_SLEEP
-
-#ifdef HAVE_LIBGCRYPT
-# include <gcrypt.h>
-#elif HAVE_OPENSSL
-# include <openssl/rand.h>
-# include "SimpleRandomizer.h"
-#endif // HAVE_OPENSSL
 
+#include "SimpleRandomizer.h"
 #include "File.h"
 #include "message.h"
 #include "Randomizer.h"
@@ -81,6 +69,7 @@
 #include "Logger.h"
 #include "Option.h"
 #include "DownloadContext.h"
+#include "BufferedFile.h"
 
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigest.h"
@@ -94,6 +83,105 @@
 
 namespace aria2 {
 
+#ifdef __MINGW32__
+namespace {
+int utf8ToWChar(wchar_t* out, size_t outLength, const char* src)
+{
+  return MultiByteToWideChar(CP_UTF8, 0, src, -1, out, outLength);
+}
+} // namespace
+
+namespace {
+int ansiToWChar(wchar_t* out, size_t outLength, const char* src)
+{
+  return MultiByteToWideChar(CP_ACP, 0, src, -1, out, outLength);
+}
+} // namespace
+
+namespace {
+int wCharToUtf8(char* out, size_t outLength, const wchar_t* src)
+{
+  return WideCharToMultiByte(CP_UTF8, 0, src, -1, out, outLength, 0, 0);
+}
+} // namespace
+
+namespace {
+int wCharToAnsi(char* out, size_t outLength, const wchar_t* src)
+{
+  return WideCharToMultiByte(CP_ACP, 0, src, -1, out, outLength, 0, 0);
+}
+} // namespace
+
+std::wstring utf8ToWChar(const char* src)
+{
+  int len = utf8ToWChar(0, 0, src);
+  if(len == 0) {
+    abort();
+  }
+  array_ptr<wchar_t> buf(new wchar_t[len]);
+  len = utf8ToWChar(buf, len, src);
+  if(len == 0) {
+    abort();
+  } else {
+    std::wstring dest(buf);
+    return dest;
+  }
+}
+
+std::wstring utf8ToWChar(const std::string& src)
+{
+  return utf8ToWChar(src.c_str());
+}
+
+std::string utf8ToNative(const std::string& src)
+{
+  std::wstring wsrc = utf8ToWChar(src);
+  int len = wCharToAnsi(0, 0, wsrc.c_str());
+  if(len == 0) {
+    abort();
+  }
+  array_ptr<char> buf(new char[len]);
+  len = wCharToAnsi(buf, len, wsrc.c_str());
+  if(len == 0) {
+    abort();
+  } else {
+    std::string dest(buf);
+    return dest;
+  }
+}
+
+std::string wCharToUtf8(const std::wstring& wsrc)
+{
+  int len = wCharToUtf8(0, 0, wsrc.c_str());
+  if(len == 0) {
+    abort();
+  }
+  array_ptr<char> buf(new char[len]);
+  len = wCharToUtf8(buf, len, wsrc.c_str());
+  if(len == 0) {
+    abort();
+  } else {
+    std::string dest(buf);
+    return dest;
+  }
+}
+
+std::string nativeToUtf8(const std::string& src)
+{
+  int len = ansiToWChar(0, 0, src.c_str());
+  if(len == 0) {
+    abort();
+  }
+  array_ptr<wchar_t> buf(new wchar_t[len]);
+  len = ansiToWChar(buf, len, src.c_str());
+  if(len == 0) {
+    abort();
+  } else {
+    return wCharToUtf8(std::wstring(buf));
+  }
+}
+#endif // __MINGW32__
+
 namespace util {
 
 const std::string DEFAULT_STRIP_CHARSET("\r\n\t ");
@@ -279,7 +367,12 @@ bool isUtf8(const std::string& str)
     unsigned char firstChar = *s;
     // See ABNF in http://tools.ietf.org/search/rfc3629#section-4
     if(in(firstChar, 0x20u, 0x7eu) ||
-       firstChar == 0x09u || firstChar == 0x0au ||firstChar == 0x0du) {
+       firstChar == 0x08u || // \b
+       firstChar == 0x09u || // \t
+       firstChar == 0x0au || // \n
+       firstChar == 0x0cu || // \f
+       firstChar == 0x0du    // \r
+       ) {
       // UTF8-1 (without ctrl chars)
     } else if(in(firstChar, 0xc2u, 0xdfu)) {
       // UTF8-2
@@ -351,6 +444,25 @@ std::string percentEncode(const std::string& target)
                        target.size());
 }
 
+std::string percentEncodeMini(const std::string& src)
+{
+  std::string result;
+  for(std::string::const_iterator i = src.begin(), eoi = src.end(); i != eoi;
+      ++i) {
+    // Non-Printable ASCII and non-ASCII chars + some ASCII chars.
+    unsigned char c = *i;
+    if(in(c, 0x00u, 0x20u) || c >= 0x7fu ||
+       // Chromium escapes following characters. Firefox4 escapes
+       // more.
+       c == '"' || c == '<' || c == '>') {
+      result += fmt("%%%02X", c);
+    } else {
+      result += c;
+    }
+  }
+  return result;
+}
+
 std::string torrentPercentEncode(const unsigned char* target, size_t len) {
   std::string dest;
   for(size_t i = 0; i < len; ++i) {
@@ -1283,18 +1395,10 @@ std::map<size_t, std::string> createIndexPathMap(std::istream& i)
 
 void generateRandomData(unsigned char* data, size_t length)
 {
-#ifdef HAVE_LIBGCRYPT
-  gcry_randomize(data, length, GCRY_STRONG_RANDOM);
-#elif HAVE_OPENSSL
-  if(RAND_bytes(data, length) != 1) {
-    for(size_t i = 0; i < length; ++i) {
-      data[i] = SimpleRandomizer::getInstance()->getRandomNumber(UINT8_MAX+1);
-    }
+  const SharedHandle<SimpleRandomizer>& rd = SimpleRandomizer::getInstance();
+  for(size_t i = 0; i < length; ++i) {
+    data[i] = static_cast<unsigned long>(rd->getRandomNumber(256));
   }
-#else
-  std::ifstream i("/dev/urandom", std::ios::binary);
-  i.read(reinterpret_cast<char*>(data), length);
-#endif // HAVE_OPENSSL
 }
 
 bool saveAs
@@ -1305,13 +1409,14 @@ bool saveAs
   }
   std::string tempFilename = strconcat(filename, "__temp");
   {
-    std::ofstream out(tempFilename.c_str(), std::ios::binary);
-    if(!out) {
+    BufferedFile fp(tempFilename, BufferedFile::WRITE);
+    if(!fp) {
       return false;
     }
-    out << data;
-    out.flush();
-    if(!out) {
+    if(fp.write(data.data(), data.size()) != data.size()) {
+      return false;
+    }
+    if(fp.close() == EOF) {
       return false;
     }
   }
@@ -1320,13 +1425,22 @@ bool saveAs
 
 std::string applyDir(const std::string& dir, const std::string& relPath)
 {
+  std::string s;
   if(dir.empty()) {
-    return strconcat(A2STR::DOT_C, A2STR::SLASH_C, relPath);
+    s = strconcat(A2STR::DOT_C, A2STR::SLASH_C, relPath);
   } else if(dir == A2STR::SLASH_C) {
-    return strconcat(A2STR::SLASH_C, relPath);
+    s = strconcat(A2STR::SLASH_C, relPath);
   } else {
-    return strconcat(dir, A2STR::SLASH_C, relPath);
+    s = strconcat(dir, A2STR::SLASH_C, relPath);
   }
+#ifdef __MINGW32__
+  for(std::string::iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
+    if(*i == '\\') {
+      *i = '/';
+    }
+  }
+#endif // __MINGW32__
+  return s;
 }
 
 std::string fixTaintedBasename(const std::string& src)
@@ -1412,9 +1526,6 @@ std::string escapePath(const std::string& s)
        || std::find(vbegin(WIN_INVALID_PATH_CHARS),
                     vend(WIN_INVALID_PATH_CHARS),
                     c) != vend(WIN_INVALID_PATH_CHARS)
-       // Since Windows does not understand UTF-8 correctly, we
-       // percent-encode character other than ASCII.
-       || c > 0x7fu
 #endif // __MINGW32__
        ){
       d += fmt("%%%02X", c);
