@@ -22,10 +22,11 @@ class BitfieldManTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testAddNotFilter);
   CPPUNIT_TEST(testAddNotFilter_zeroLength);
   CPPUNIT_TEST(testAddNotFilter_overflow);
-  CPPUNIT_TEST(testGetSparceMissingUnusedIndex);
-  CPPUNIT_TEST(testGetSparceMissingUnusedIndex_setBit);
-  CPPUNIT_TEST(testGetSparceMissingUnusedIndex_withMinSplitSize);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex_setBit);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex_withMinSplitSize);
   CPPUNIT_TEST(testIsBitSetOffsetRange);
+  CPPUNIT_TEST(testGetOffsetCompletedLength);
   CPPUNIT_TEST(testGetMissingUnusedLength);
   CPPUNIT_TEST(testSetBitRange);
   CPPUNIT_TEST(testGetAllMissingIndexes);
@@ -36,6 +37,8 @@ class BitfieldManTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testCountMissingBlock);
   CPPUNIT_TEST(testZeroLengthFilter);
   CPPUNIT_TEST(testGetFirstNMissingUnusedIndex);
+  CPPUNIT_TEST(testGetInorderMissingUnusedIndex);
+  CPPUNIT_TEST(testGetGeomMissingUnusedIndex);
   CPPUNIT_TEST_SUITE_END();
 public:
   void testGetBlockSize();
@@ -52,16 +55,19 @@ public:
   void testAddNotFilter();
   void testAddNotFilter_zeroLength();
   void testAddNotFilter_overflow();
-  void testGetSparceMissingUnusedIndex();
-  void testGetSparceMissingUnusedIndex_setBit();
-  void testGetSparceMissingUnusedIndex_withMinSplitSize();
+  void testGetSparseMissingUnusedIndex();
+  void testGetSparseMissingUnusedIndex_setBit();
+  void testGetSparseMissingUnusedIndex_withMinSplitSize();
   void testIsBitSetOffsetRange();
+  void testGetOffsetCompletedLength();
   void testGetMissingUnusedLength();
   void testSetBitRange();
   void testCountFilteredBlock();
   void testCountMissingBlock();
   void testZeroLengthFilter();
   void testGetFirstNMissingUnusedIndex();
+  void testGetInorderMissingUnusedIndex();
+  void testGetGeomMissingUnusedIndex();
 };
 
 
@@ -277,7 +283,7 @@ void BitfieldManTest::testAddNotFilter_overflow() {
 }
 
 // TODO1.5 add test using ignoreBitfield
-void BitfieldManTest::testGetSparceMissingUnusedIndex() {
+void BitfieldManTest::testGetSparseMissingUnusedIndex() {
   BitfieldMan bitfield(1024*1024, 10*1024*1024);
   const size_t length = 2;
   unsigned char ignoreBitfield[length];
@@ -328,7 +334,7 @@ void BitfieldManTest::testGetSparceMissingUnusedIndex() {
                                                        ignoreBitfield, length));
 }
 
-void BitfieldManTest::testGetSparceMissingUnusedIndex_setBit() {
+void BitfieldManTest::testGetSparseMissingUnusedIndex_setBit() {
   BitfieldMan bitfield(1024*1024, 10*1024*1024);
   const size_t length = 2;
   unsigned char ignoreBitfield[length];
@@ -379,7 +385,7 @@ void BitfieldManTest::testGetSparceMissingUnusedIndex_setBit() {
                                                        ignoreBitfield, length));
 }
 
-void BitfieldManTest::testGetSparceMissingUnusedIndex_withMinSplitSize()
+void BitfieldManTest::testGetSparseMissingUnusedIndex_withMinSplitSize()
 {
   BitfieldMan bitfield(1024*1024, 10*1024*1024);
   const size_t length = 2;
@@ -443,6 +449,28 @@ void BitfieldManTest::testIsBitSetOffsetRange()
   CPPUNIT_ASSERT(!bitfield.isBitSetOffsetRange(pieceLength*100, pieceLength*3));
 }
 
+void BitfieldManTest::testGetOffsetCompletedLength()
+{
+  BitfieldMan bt(1024, 1024*20);
+  // 00000|00000|00000|00000
+  CPPUNIT_ASSERT_EQUAL((uint64_t)0, bt.getOffsetCompletedLength(0, 1024));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)0, bt.getOffsetCompletedLength(0, 0));
+  for(size_t i = 2; i <= 4; ++i) {
+    bt.setBit(i);
+  }
+  // 00111|00000|00000|00000
+  CPPUNIT_ASSERT_EQUAL((uint64_t)3072, bt.getOffsetCompletedLength(2048, 3072));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)3071, bt.getOffsetCompletedLength(2047, 3072));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)3071, bt.getOffsetCompletedLength(2049, 3072));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)0, bt.getOffsetCompletedLength(2048, 0));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)1, bt.getOffsetCompletedLength(2048, 1));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)0, bt.getOffsetCompletedLength(2047, 1));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)3072, bt.getOffsetCompletedLength(0, 1024*20));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)3072,
+                       bt.getOffsetCompletedLength(0, 1024*20+10));
+  CPPUNIT_ASSERT_EQUAL((uint64_t)0, bt.getOffsetCompletedLength(1024*20, 1));
+}
+
 void BitfieldManTest::testGetMissingUnusedLength()
 {
   uint64_t totalLength = 1024*10+10;
@@ -640,4 +668,120 @@ void BitfieldManTest::testGetFirstNMissingUnusedIndex()
   CPPUNIT_ASSERT_EQUAL((size_t)9, out[0]);
 }
 
+void BitfieldManTest::testGetInorderMissingUnusedIndex()
+{
+  BitfieldMan bt(1024, 1024*20);
+  const size_t length = 3;
+  unsigned char ignoreBitfield[length];
+  memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 1024;
+  size_t index;
+  // 00000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
+  bt.setUseBit(0);
+  // 10000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)1, index);
+  minSplitSize = 1024*2;
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, index);
+  bt.unsetUseBit(0);
+  bt.setBit(0);
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)1, index);
+  bt.setAllBit();
+  bt.unsetBit(10);
+  // 11111|11111|01111|11111
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)10, index);
+  bt.setUseBit(10);
+  CPPUNIT_ASSERT
+    (!bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  bt.unsetUseBit(10);
+  bt.setAllBit();
+  // 11111|11111|11111|11111
+  CPPUNIT_ASSERT
+    (!bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  bt.clearAllBit();
+  // 00000|00000|00000|00000
+  for(int i = 0; i <= 1; ++i) {
+    bitfield::flipBit(ignoreBitfield, length, i);
+  }
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, index);
+  bt.addFilter(1024*3, 1024*3);
+  bt.enableFilter();
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)3, index);
+}
+
+void BitfieldManTest::testGetGeomMissingUnusedIndex()
+{
+  BitfieldMan bt(1024, 1024*20);
+  const size_t length = 3;
+  unsigned char ignoreBitfield[length];
+  memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 1024;
+  size_t index;
+  // 00000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
+  bt.setUseBit(0);
+  // 10000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)1, index);
+  bt.setUseBit(1);
+  // 11000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, index);
+  bt.setUseBit(2);
+  // 11100|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)4, index);
+  bt.setUseBit(4);
+  // 11110|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)8, index);
+  bt.setUseBit(8);
+  // 11110|00010|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)16, index);
+  bt.setUseBit(16);
+  // 11110|00010|00000|01000
+  CPPUNIT_ASSERT
+    (bt.getGeomMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length, 2, 0));
+  CPPUNIT_ASSERT_EQUAL((size_t)12, index);
+  bt.setUseBit(12);
+}
+
 } // namespace aria2
