@@ -12,11 +12,12 @@
 #include "FixedNumberRandomizer.h"
 #include "FileEntry.h"
 #include "array_fun.h"
-#include "messageDigest.h"
 #include "a2netcompat.h"
 #include "bencode2.h"
 #include "TestUtil.h"
 #include "base32.h"
+#include "Option.h"
+#include "prefs.h"
 
 namespace aria2 {
 
@@ -50,24 +51,36 @@ class BittorrentHelperTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testLoadFromMemory_overrideName);
   CPPUNIT_TEST(testLoadFromMemory_multiFileDirTraversal);
   CPPUNIT_TEST(testLoadFromMemory_singleFileDirTraversal);
+  CPPUNIT_TEST(testLoadFromMemory_multiFileNonUtf8Path);
+  CPPUNIT_TEST(testLoadFromMemory_singleFileNonUtf8Path);
   CPPUNIT_TEST(testGetNodes);
   CPPUNIT_TEST(testGetBasePath);
   CPPUNIT_TEST(testSetFileFilter_single);
   CPPUNIT_TEST(testSetFileFilter_multi);
   CPPUNIT_TEST(testUTF8Torrent);
   CPPUNIT_TEST(testEtc);
-  CPPUNIT_TEST(testCreatecompact);
   CPPUNIT_TEST(testCheckBitfield);
   CPPUNIT_TEST(testMetadata);
   CPPUNIT_TEST(testParseMagnet);
   CPPUNIT_TEST(testParseMagnet_base32);
   CPPUNIT_TEST(testMetadata2Torrent);
   CPPUNIT_TEST(testTorrent2Magnet);
+  CPPUNIT_TEST(testExtractPeerFromString);
   CPPUNIT_TEST(testExtractPeerFromList);
   CPPUNIT_TEST(testExtract2PeersFromList);
+  CPPUNIT_TEST(testPackcompact);
+  CPPUNIT_TEST(testUnpackcompact);
+  CPPUNIT_TEST(testRemoveAnnounceUri);
+  CPPUNIT_TEST(testAddAnnounceUri);
+  CPPUNIT_TEST(testAdjustAnnounceUri);
   CPPUNIT_TEST_SUITE_END();
 public:
-  void setUp() {
+  SharedHandle<Option> option_;
+
+  void setUp()
+  {
+    option_.reset(new Option());
+    option_->put(PREF_DIR, ".");
   }
 
   void testGetInfoHash();
@@ -95,21 +108,28 @@ public:
   void testLoadFromMemory_overrideName();
   void testLoadFromMemory_multiFileDirTraversal();
   void testLoadFromMemory_singleFileDirTraversal();
+  void testLoadFromMemory_multiFileNonUtf8Path();
+  void testLoadFromMemory_singleFileNonUtf8Path();
   void testGetNodes();
   void testGetBasePath();
   void testSetFileFilter_single();
   void testSetFileFilter_multi();
   void testUTF8Torrent();
   void testEtc();
-  void testCreatecompact();
   void testCheckBitfield();
   void testMetadata();
   void testParseMagnet();
   void testParseMagnet_base32();
   void testMetadata2Torrent();
   void testTorrent2Magnet();
+  void testExtractPeerFromString();
   void testExtractPeerFromList();
   void testExtract2PeersFromList();
+  void testPackcompact();
+  void testUnpackcompact();
+  void testRemoveAnnounceUri();
+  void testAddAnnounceUri();
+  void testAdjustAnnounceUri();
 };
 
 
@@ -117,7 +137,7 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BittorrentHelperTest);
 
 void BittorrentHelperTest::testGetInfoHash() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   std::string correctHash = "248d0a1cd08284299de78d5c1ed359bb46717d8c";
 
@@ -126,7 +146,7 @@ void BittorrentHelperTest::testGetInfoHash() {
 
 void BittorrentHelperTest::testGetPieceHash() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(util::toHex("AAAAAAAAAAAAAAAAAAAA", 20),
                        dctx->getPieceHash(0));
@@ -137,12 +157,12 @@ void BittorrentHelperTest::testGetPieceHash() {
   CPPUNIT_ASSERT_EQUAL(std::string(""),
                        dctx->getPieceHash(3));
 
-  CPPUNIT_ASSERT_EQUAL(MessageDigestContext::SHA1, dctx->getPieceHashAlgo());
+  CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), dctx->getPieceHashType());
 }
 
 void BittorrentHelperTest::testGetFileEntries() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
   // This is multi-file torrent.
   std::vector<SharedHandle<FileEntry> > fileEntries = dctx->getFileEntries();
   // There are 2 file entries.
@@ -162,7 +182,7 @@ void BittorrentHelperTest::testGetFileEntries() {
 
 void BittorrentHelperTest::testGetFileEntriesSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
   // This is multi-file torrent.
   std::vector<SharedHandle<FileEntry> > fileEntries = dctx->getFileEntries();
   // There is 1 file entry.
@@ -178,42 +198,42 @@ void BittorrentHelperTest::testGetFileEntriesSingle() {
 
 void BittorrentHelperTest::testGetTotalLength() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, dctx->getTotalLength());
 }
 
 void BittorrentHelperTest::testGetTotalLengthSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, dctx->getTotalLength());
 }
 
 void BittorrentHelperTest::testGetFileModeMulti() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(MULTI, getTorrentAttrs(dctx)->mode);
 }
 
 void BittorrentHelperTest::testGetFileModeSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(SINGLE, getTorrentAttrs(dctx)->mode);
 }
 
 void BittorrentHelperTest::testGetNameMulti() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-test"), getTorrentAttrs(dctx)->name);
 }
 
 void BittorrentHelperTest::testGetNameSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(std::string("./aria2-0.8.2.tar.bz2"),
                        dctx->getBasePath());
@@ -224,7 +244,7 @@ void BittorrentHelperTest::testGetNameSingle() {
 void BittorrentHelperTest::testOverrideName()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx, "aria2-override.name");
+  load(A2_TEST_DIR"/test.torrent", dctx, option_, "aria2-override.name");
   CPPUNIT_ASSERT_EQUAL(std::string("./aria2-override.name"),
                        dctx->getBasePath());
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-override.name"),
@@ -234,7 +254,7 @@ void BittorrentHelperTest::testOverrideName()
 
 void BittorrentHelperTest::testGetAnnounceTier() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
   SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
   // There is 1 tier.
   CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList.size());
@@ -246,7 +266,7 @@ void BittorrentHelperTest::testGetAnnounceTier() {
 
 void BittorrentHelperTest::testGetAnnounceTierAnnounceList() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
   SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
   // There are 3 tiers.
   CPPUNIT_ASSERT_EQUAL((size_t)3, attrs->announceList.size());
@@ -266,14 +286,14 @@ void BittorrentHelperTest::testGetAnnounceTierAnnounceList() {
 
 void BittorrentHelperTest::testGetPieceLength() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL((size_t)128, dctx->getPieceLength());
 }
 
 void BittorrentHelperTest::testGetInfoHashAsString() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT_EQUAL(std::string("248d0a1cd08284299de78d5c1ed359bb46717d8c"),
                        getInfoHashString(dctx));
@@ -321,7 +341,7 @@ void BittorrentHelperTest::testComputeFastSet()
 
 void BittorrentHelperTest::testGetFileEntries_multiFileUrlList() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("url-list-multiFile.torrent", dctx);
+  load(A2_TEST_DIR"/url-list-multiFile.torrent", dctx, option_);
   // This is multi-file torrent.
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
     dctx->getFileEntries();
@@ -354,7 +374,7 @@ void BittorrentHelperTest::testGetFileEntries_multiFileUrlList() {
 
 void BittorrentHelperTest::testGetFileEntries_singleFileUrlList() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("url-list-singleFile.torrent", dctx);
+  load(A2_TEST_DIR"/url-list-singleFile.torrent", dctx, option_);
   // This is single-file torrent.
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
     dctx->getFileEntries();
@@ -372,7 +392,7 @@ void BittorrentHelperTest::testGetFileEntries_singleFileUrlList() {
 
 void BittorrentHelperTest::testGetFileEntries_singleFileUrlListEndsWithSlash() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("url-list-singleFileEndsWithSlash.torrent", dctx);
+  load(A2_TEST_DIR"/url-list-singleFileEndsWithSlash.torrent", dctx, option_);
   // This is single-file torrent.
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
     dctx->getFileEntries();
@@ -388,12 +408,56 @@ void BittorrentHelperTest::testGetFileEntries_singleFileUrlListEndsWithSlash() {
                        uris1[0]);
 }
 
+void BittorrentHelperTest::testLoadFromMemory_multiFileNonUtf8Path()
+{
+  SharedHandle<List> path = List::g();
+  path->append("path");
+  path->append(util::fromHex("90a28a")+"E");
+  SharedHandle<Dict> file = Dict::g();
+  file->put("length", Integer::g(1024));
+  file->put("path", path);
+  SharedHandle<List> files = List::g();
+  files->append(file);
+  SharedHandle<Dict> info = Dict::g();
+  info->put("files", files);
+  info->put("piece length", Integer::g(1024));
+  info->put("pieces", "01234567890123456789");
+  info->put("name", util::fromHex("1b")+"$B%O%m!<"+util::fromHex("1b")+"(B");
+  Dict dict;
+  dict.put("info", info);
+  SharedHandle<DownloadContext> dctx(new DownloadContext());
+  loadFromMemory(bencode2::encode(&dict), dctx, option_, "default");
+
+  const SharedHandle<FileEntry>& fe = dctx->getFirstFileEntry();
+  CPPUNIT_ASSERT_EQUAL
+    (std::string("./%1B%24B%25O%25m%21%3C%1B%28B/path/%90%A2%8AE"),
+     fe->getPath());
+  CPPUNIT_ASSERT_EQUAL
+    (std::string("./%1B%24B%25O%25m%21%3C%1B%28B"), dctx->getBasePath());
+}
+
+void BittorrentHelperTest::testLoadFromMemory_singleFileNonUtf8Path()
+{
+  SharedHandle<Dict> info = Dict::g();
+  info->put("piece length", Integer::g(1024));
+  info->put("pieces", "01234567890123456789");
+  info->put("name", util::fromHex("90a28a")+"E");
+  info->put("length", Integer::g(1024));
+  Dict dict;
+  dict.put("info", info);
+  SharedHandle<DownloadContext> dctx(new DownloadContext());
+  loadFromMemory(bencode2::encode(&dict), dctx, option_, "default");
+
+  const SharedHandle<FileEntry>& fe = dctx->getFirstFileEntry();
+  CPPUNIT_ASSERT_EQUAL(std::string("./%90%A2%8AE"), fe->getPath());
+}
+
 void BittorrentHelperTest::testLoadFromMemory()
 {
   std::string memory = "d8:announce36:http://aria.rednoah.com/announce.php13:announce-listll16:http://tracker1 el15:http://tracker2el15:http://tracker3ee7:comment17:REDNOAH.COM RULES13:creation datei1123456789e4:infod5:filesld6:lengthi284e4:pathl5:aria23:src6:aria2ceed6:lengthi100e4:pathl19:aria2-0.2.2.tar.bz2eee4:name10:aria2-test12:piece lengthi128e6:pieces60:AAAAAAAAAAAAAAAAAAAABBBBBBBBBBBBBBBBBBBBCCCCCCCCCCCCCCCCCCCCee";
 
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  loadFromMemory(memory, dctx, "default");
+  loadFromMemory(memory, dctx, option_, "default");
 
   std::string correctHash = "248d0a1cd08284299de78d5c1ed359bb46717d8c";
 
@@ -406,7 +470,7 @@ void BittorrentHelperTest::testLoadFromMemory_somethingMissing()
   try {
     std::string memory = "d8:announce36:http://aria.rednoah.com/announce.php4:infod4:name13:aria2.tar.bz26:lengthi262144eee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {
     // OK
@@ -418,7 +482,7 @@ void BittorrentHelperTest::testLoadFromMemory_overrideName()
   std::string memory = "d8:announce36:http://aria.rednoah.com/announce.php13:announce-listll16:http://tracker1 el15:http://tracker2el15:http://tracker3ee7:comment17:REDNOAH.COM RULES13:creation datei1123456789e4:infod5:filesld6:lengthi284e4:pathl5:aria23:src6:aria2ceed6:lengthi100e4:pathl19:aria2-0.2.2.tar.bz2eee4:name10:aria2-test12:piece lengthi128e6:pieces60:AAAAAAAAAAAAAAAAAAAABBBBBBBBBBBBBBBBBBBBCCCCCCCCCCCCCCCCCCCCee";
 
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  loadFromMemory(memory, dctx, "default", "aria2-override.name");
+  loadFromMemory(memory, dctx, option_, "default", "aria2-override.name");
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-override.name"),
                        getTorrentAttrs(dctx)->name);
@@ -430,9 +494,8 @@ void BittorrentHelperTest::testLoadFromMemory_multiFileDirTraversal()
     "d8:announce27:http://example.com/announce4:infod5:filesld6:lengthi262144e4:pathl7:../dir14:dir28:file.imgeee4:name14:../name1/name212:piece lengthi262144e6:pieces20:00000000000000000000ee";
 
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  dctx->setDir("/tmp");
   try {
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
     CPPUNIT_FAIL("Exception must be thrown.");
   } catch(RecoverableException& e) {
     // success
@@ -445,9 +508,8 @@ void BittorrentHelperTest::testLoadFromMemory_singleFileDirTraversal()
     "d8:announce27:http://example.com/announce4:infod4:name14:../name1/name26:lengthi262144e12:piece lengthi262144e6:pieces20:00000000000000000000ee";
 
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  dctx->setDir("/tmp");
   try {
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
   } catch(RecoverableException& e) {
     // success
   }
@@ -465,7 +527,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)2, attrs->nodes.size());
@@ -485,7 +547,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->nodes.size());
@@ -503,7 +565,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->nodes.size());
@@ -521,7 +583,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->nodes.size());
@@ -538,7 +600,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)0, attrs->nodes.size());
@@ -554,7 +616,7 @@ void BittorrentHelperTest::testGetNodes()
       "6:pieces20:AAAAAAAAAAAAAAAAAAAA"
       "ee";
     SharedHandle<DownloadContext> dctx(new DownloadContext());
-    loadFromMemory(memory, dctx, "default");
+    loadFromMemory(memory, dctx, option_, "default");
 
     SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
     CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->nodes.size());
@@ -566,13 +628,13 @@ void BittorrentHelperTest::testGetNodes()
 void BittorrentHelperTest::testGetBasePath()
 {
   SharedHandle<DownloadContext> singleCtx(new DownloadContext());
-  load("single.torrent", singleCtx);
+  load(A2_TEST_DIR"/single.torrent", singleCtx, option_);
   singleCtx->setFilePathWithIndex(1, "new-path");
   CPPUNIT_ASSERT_EQUAL(std::string("new-path"), singleCtx->getBasePath());
 
+  option_->put(PREF_DIR, "downloads");
   SharedHandle<DownloadContext> multiCtx(new DownloadContext());
-  multiCtx->setDir("downloads");
-  load("test.torrent", multiCtx);
+  load(A2_TEST_DIR"/test.torrent", multiCtx, option_);
   CPPUNIT_ASSERT_EQUAL(std::string("downloads/aria2-test"),
                        multiCtx->getBasePath());
 }
@@ -580,7 +642,7 @@ void BittorrentHelperTest::testGetBasePath()
 void BittorrentHelperTest::testSetFileFilter_single()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("single.torrent", dctx);
+  load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
   CPPUNIT_ASSERT(dctx->getFirstFileEntry()->isRequested());
 
@@ -599,7 +661,7 @@ void BittorrentHelperTest::testSetFileFilter_single()
 void BittorrentHelperTest::testSetFileFilter_multi()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
   CPPUNIT_ASSERT(dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(dctx->getFileEntries()[1]->isRequested());
@@ -624,7 +686,7 @@ void BittorrentHelperTest::testSetFileFilter_multi()
 void BittorrentHelperTest::testUTF8Torrent()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("utf8.torrent", dctx);
+  load(A2_TEST_DIR"/utf8.torrent", dctx, option_);
   CPPUNIT_ASSERT_EQUAL(std::string("name in utf-8"),
                        getTorrentAttrs(dctx)->name);
   CPPUNIT_ASSERT_EQUAL(std::string("./name in utf-8/path in utf-8"),
@@ -636,7 +698,7 @@ void BittorrentHelperTest::testUTF8Torrent()
 void BittorrentHelperTest::testEtc()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
   CPPUNIT_ASSERT_EQUAL(std::string("REDNOAH.COM RULES"),
                        getTorrentAttrs(dctx)->comment);
   CPPUNIT_ASSERT_EQUAL(std::string("aria2"),
@@ -645,18 +707,6 @@ void BittorrentHelperTest::testEtc()
                        getTorrentAttrs(dctx)->creationDate);
 }
 
-void BittorrentHelperTest::testCreatecompact()
-{
-  unsigned char compact[6];
-  // Note: bittorrent::createcompact() on linux can handle IPv4-mapped
-  // addresses like `ffff::127.0.0.1', but on cygwin, it doesn't.
-  CPPUNIT_ASSERT(createcompact(compact, "127.0.0.1", 6881));
-
-  std::pair<std::string, uint16_t> p = unpackcompact(compact);
-  CPPUNIT_ASSERT_EQUAL(std::string("127.0.0.1"), p.first);
-  CPPUNIT_ASSERT_EQUAL((uint16_t)6881, p.second);
-}
-
 void BittorrentHelperTest::testCheckBitfield()
 {
   unsigned char bitfield[] = { 0xff, 0xe0 };
@@ -679,10 +729,10 @@ void BittorrentHelperTest::testCheckBitfield()
 
 void BittorrentHelperTest::testMetadata() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
-  std::string torrentData = readFile("test.torrent");
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
+  std::string torrentData = readFile(A2_TEST_DIR"/test.torrent");
   SharedHandle<ValueBase> tr = bencode2::decode(torrentData);
-  SharedHandle<ValueBase> infoDic = asDict(tr)->get("info");
+  SharedHandle<ValueBase> infoDic = downcast<Dict>(tr)->get("info");
   std::string metadata = bencode2::encode(infoDic);
   SharedHandle<TorrentAttribute> attrs = getTorrentAttrs(dctx);
   CPPUNIT_ASSERT(metadata == attrs->metadata);
@@ -749,7 +799,7 @@ void BittorrentHelperTest::testMetadata2Torrent()
 void BittorrentHelperTest::testTorrent2Magnet()
 {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
-  load("test.torrent", dctx);
+  load(A2_TEST_DIR"/test.torrent", dctx, option_);
   
   CPPUNIT_ASSERT_EQUAL
     (std::string("magnet:?xt=urn:btih:248D0A1CD08284299DE78D5C1ED359BB46717D8C"
@@ -760,6 +810,35 @@ void BittorrentHelperTest::testTorrent2Magnet()
      torrent2Magnet(getTorrentAttrs(dctx)));
 }
 
+void BittorrentHelperTest::testExtractPeerFromString()
+{
+  std::string hextext = "100210354527354678541237324732171ae1";
+  hextext += "20010db8bd0501d2288a1fc0000110ee1ae2";
+  std::string peersstr = "36:"+util::fromHex(hextext);
+  SharedHandle<ValueBase> str = bencode2::decode(peersstr);
+  std::deque<SharedHandle<Peer> > peers;
+  extractPeer(str, AF_INET6, std::back_inserter(peers));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, peers.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("1002:1035:4527:3546:7854:1237:3247:3217"),
+                       peers[0]->getIPAddress());
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6881, peers[0]->getPort());
+  CPPUNIT_ASSERT_EQUAL(std::string("2001:db8:bd05:1d2:288a:1fc0:1:10ee"),
+                       peers[1]->getIPAddress());
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6882, peers[1]->getPort());
+
+  hextext = "c0a800011ae1";
+  hextext += "c0a800021ae2";
+  peersstr = "12:"+util::fromHex(hextext);
+  str = bencode2::decode(peersstr);
+  peers.clear();
+  extractPeer(str, AF_INET, std::back_inserter(peers));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, peers.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), peers[0]->getIPAddress());
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6881, peers[0]->getPort());
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.2"), peers[1]->getIPAddress());
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6882, peers[1]->getPort()); 
+}
+
 void BittorrentHelperTest::testExtractPeerFromList()
 {
   std::string peersString =
@@ -769,7 +848,7 @@ void BittorrentHelperTest::testExtractPeerFromList()
   SharedHandle<ValueBase> dict = bencode2::decode(peersString);
   
   std::deque<SharedHandle<Peer> > peers;
-  extractPeer(asDict(dict)->get("peers"), std::back_inserter(peers));
+  extractPeer(downcast<Dict>(dict)->get("peers"), AF_INET, std::back_inserter(peers));
   CPPUNIT_ASSERT_EQUAL((size_t)1, peers.size());
   SharedHandle<Peer> peer = *peers.begin();
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), peer->getIPAddress());
@@ -786,7 +865,7 @@ void BittorrentHelperTest::testExtract2PeersFromList()
   SharedHandle<ValueBase> dict = bencode2::decode(peersString);
 
   std::deque<SharedHandle<Peer> > peers;
-  extractPeer(asDict(dict)->get("peers"), std::back_inserter(peers));
+  extractPeer(downcast<Dict>(dict)->get("peers"), AF_INET, std::back_inserter(peers));
   CPPUNIT_ASSERT_EQUAL((size_t)2, peers.size());
   SharedHandle<Peer> peer = *peers.begin();
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), peer->getIPAddress());
@@ -797,6 +876,116 @@ void BittorrentHelperTest::testExtract2PeersFromList()
   CPPUNIT_ASSERT_EQUAL((uint16_t)2007, peer->getPort());
 }
 
+void BittorrentHelperTest::testPackcompact()
+{
+  unsigned char compact[COMPACT_LEN_IPV6];
+  CPPUNIT_ASSERT_EQUAL(18,
+                       packcompact(compact,
+                                   "1002:1035:4527:3546:7854:1237:3247:3217",
+                                   6881));
+  CPPUNIT_ASSERT_EQUAL(std::string("100210354527354678541237324732171ae1"),
+                       util::toHex(compact, 18));
+
+  CPPUNIT_ASSERT_EQUAL(6, packcompact(compact, "192.168.0.1", 6881));
+  CPPUNIT_ASSERT_EQUAL(std::string("c0a800011ae1"), util::toHex(compact, 6));
+
+  CPPUNIT_ASSERT_EQUAL(0, packcompact(compact, "badaddr", 6881));
+}
+
+void BittorrentHelperTest::testUnpackcompact()
+{
+  unsigned char compact6[] = {
+    0x10, 0x02, 0x10, 0x35, 0x45, 0x27, 0x35, 0x46,
+    0x78, 0x54, 0x12, 0x37, 0x32, 0x47, 0x32, 0x17,
+    0x1A, 0xE1 };
+  std::pair<std::string, uint16_t> p =
+    unpackcompact(compact6, AF_INET6);
+  CPPUNIT_ASSERT_EQUAL(std::string("1002:1035:4527:3546:7854:1237:3247:3217"),
+                       p.first);
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6881, p.second);
+
+  unsigned char compact[] = { 0xC0, 0xa8, 0x00, 0x01, 0x1A, 0xE1 };
+  p = unpackcompact(compact, AF_INET);
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), p.first);
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6881, p.second);
+}
+
+void BittorrentHelperTest::testRemoveAnnounceUri()
+{
+  SharedHandle<TorrentAttribute> attrs(new TorrentAttribute());
+  std::vector<std::string> tier1;
+  tier1.push_back("http://host1/announce");
+  std::vector<std::string> tier2;
+  tier2.push_back("http://host2/announce");
+  tier2.push_back("http://host3/announce");
+  attrs->announceList.push_back(tier1);
+  attrs->announceList.push_back(tier2);
+  
+  std::vector<std::string> removeUris;
+  removeUris.push_back(tier1[0]);
+  removeUris.push_back(tier2[0]);
+  removeAnnounceUri(attrs, removeUris);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList.size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host3/announce"),
+                       attrs->announceList[0][0]);
+
+  removeUris.clear();
+  removeUris.push_back("*");
+
+  removeAnnounceUri(attrs, removeUris);
+  CPPUNIT_ASSERT(attrs->announceList.empty());
+}
+
+void BittorrentHelperTest::testAddAnnounceUri()
+{
+  SharedHandle<TorrentAttribute> attrs(new TorrentAttribute());
+  std::vector<std::string> addUris;
+  addUris.push_back("http://host1/announce");
+  addUris.push_back("http://host2/announce");
+  addAnnounceUri(attrs, addUris);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, attrs->announceList.size());
+
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList[0].size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host1/announce"),
+                       attrs->announceList[0][0]);
+
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList[1].size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host2/announce"),
+                       attrs->announceList[1][0]);
+}
+
+void BittorrentHelperTest::testAdjustAnnounceUri()
+{
+  SharedHandle<TorrentAttribute> attrs(new TorrentAttribute());
+  std::vector<std::string> tier1;
+  tier1.push_back("http://host1/announce");
+  std::vector<std::string> tier2;
+  tier2.push_back("http://host2/announce");
+  tier2.push_back("http://host3/announce");
+  attrs->announceList.push_back(tier1);
+  attrs->announceList.push_back(tier2);
+
+  SharedHandle<Option> option(new Option());
+  option->put(PREF_BT_TRACKER, "http://host1/announce,http://host4/announce");
+  option->put(PREF_BT_EXCLUDE_TRACKER,
+              "http://host1/announce,http://host2/announce");
+  adjustAnnounceUri(attrs, option);
+
+  CPPUNIT_ASSERT_EQUAL((size_t)3, attrs->announceList.size());
+
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList[0].size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host3/announce"),
+                       attrs->announceList[0][0]);
+
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList[1].size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host1/announce"),
+                       attrs->announceList[1][0]);
+
+  CPPUNIT_ASSERT_EQUAL((size_t)1, attrs->announceList[2].size());
+  CPPUNIT_ASSERT_EQUAL(std::string("http://host4/announce"),
+                       attrs->announceList[2][0]);
+}
+
 } // namespace bittorrent
 
 } // namespace aria2
