@@ -13,6 +13,7 @@
 #include "prefs.h"
 #include "Exception.h"
 #include "util.h"
+#include "FileEntry.h"
 #ifdef ENABLE_BITTORRENT
 # include "bittorrent_helper.h"
 #endif // ENABLE_BITTORRENT
@@ -90,7 +91,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri()
     }
     CPPUNIT_ASSERT_EQUAL((unsigned int)7, group->getNumConcurrentCommand());
     SharedHandle<DownloadContext> ctx = group->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), ctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/file.out"), ctx->getBasePath());
   }
   option_->put(PREF_SPLIT, "5");
@@ -135,7 +135,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri()
     CPPUNIT_ASSERT_EQUAL((unsigned int)2,
                          alphaGroup->getNumConcurrentCommand());
     SharedHandle<DownloadContext> alphaCtx = alphaGroup->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), alphaCtx->getDir());
     // See filename is not assigned yet
     CPPUNIT_ASSERT_EQUAL(std::string(""), alphaCtx->getBasePath());
   }
@@ -170,7 +169,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri_parameterized()
 
     CPPUNIT_ASSERT_EQUAL((unsigned int)3, group->getNumConcurrentCommand());
     SharedHandle<DownloadContext> ctx = group->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), ctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/file.out"), ctx->getBasePath());
   }
 }
@@ -180,7 +178,7 @@ void DownloadHelperTest::testCreateRequestGroupForUri_BitTorrent()
 {
   std::string array[] = {
     "http://alpha/file",
-    "test.torrent",
+    A2_TEST_DIR"/test.torrent",
     "http://bravo/file",
     "http://charlie/file"
   };
@@ -206,7 +204,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri_BitTorrent()
 
     CPPUNIT_ASSERT_EQUAL((unsigned int)3, group->getNumConcurrentCommand());
     SharedHandle<DownloadContext> ctx = group->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), ctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/file.out"),
                          ctx->getBasePath());
 
@@ -217,7 +214,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri_BitTorrent()
     CPPUNIT_ASSERT_EQUAL((unsigned int)3,
                          torrentGroup->getNumConcurrentCommand());
     SharedHandle<DownloadContext> btctx = torrentGroup->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), btctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/aria2-test"),
                          btctx->getBasePath());    
   }
@@ -231,7 +227,7 @@ void DownloadHelperTest::testCreateRequestGroupForUri_Metalink()
     "http://alpha/file",
     "http://bravo/file",
     "http://charlie/file",
-    "test.xml"
+    A2_TEST_DIR"/test.xml"
   };
   std::vector<std::string> uris(vbegin(array), vend(array));
   option_->put(PREF_MAX_CONNECTION_PER_SERVER, "1");
@@ -261,7 +257,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri_Metalink()
     }
     CPPUNIT_ASSERT_EQUAL((unsigned int)3, group->getNumConcurrentCommand());
     SharedHandle<DownloadContext> ctx = group->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), ctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/file.out"),
                          ctx->getBasePath());
 
@@ -270,7 +265,6 @@ void DownloadHelperTest::testCreateRequestGroupForUri_Metalink()
                          aria2052Group->getNumConcurrentCommand());
     SharedHandle<DownloadContext> aria2052Ctx =
       aria2052Group->getDownloadContext();
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), aria2052Ctx->getDir());
     CPPUNIT_ASSERT_EQUAL(std::string("/tmp/aria2-0.5.2.tar.bz2"),
                          aria2052Ctx->getBasePath());
     
@@ -285,7 +279,7 @@ void DownloadHelperTest::testCreateRequestGroupForUriList()
 {
   option_->put(PREF_MAX_CONNECTION_PER_SERVER, "3");
   option_->put(PREF_SPLIT, "3");
-  option_->put(PREF_INPUT_FILE, "input_uris.txt");
+  option_->put(PREF_INPUT_FILE, A2_TEST_DIR"/input_uris.txt");
   option_->put(PREF_DIR, "/tmp");
   option_->put(PREF_OUT, "file.out");
 
@@ -303,15 +297,13 @@ void DownloadHelperTest::testCreateRequestGroupForUriList()
   CPPUNIT_ASSERT_EQUAL(std::string("http://charlie/file"), fileURIs[2]);
   CPPUNIT_ASSERT_EQUAL((unsigned int)3, fileGroup->getNumConcurrentCommand());
   SharedHandle<DownloadContext> fileCtx = fileGroup->getDownloadContext();
-  CPPUNIT_ASSERT_EQUAL(std::string("/mydownloads"), fileCtx->getDir());
   CPPUNIT_ASSERT_EQUAL(std::string("/mydownloads/myfile.out"),
                        fileCtx->getBasePath());
 
   SharedHandle<RequestGroup> fileISOGroup = result[1];
   SharedHandle<DownloadContext> fileISOCtx = fileISOGroup->getDownloadContext();
-  CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), fileISOCtx->getDir());
-  CPPUNIT_ASSERT_EQUAL(std::string("/tmp/file.out"),
-                       fileISOCtx->getBasePath());
+  // PREF_OUT in option_ must be ignored.
+  CPPUNIT_ASSERT_EQUAL(std::string(), fileISOCtx->getBasePath());
 }
 
 #ifdef ENABLE_BITTORRENT
@@ -326,7 +318,7 @@ void DownloadHelperTest::testCreateRequestGroupForBitTorrent()
   std::vector<std::string> auxURIs(vbegin(array), vend(array));
   option_->put(PREF_MAX_CONNECTION_PER_SERVER, "2");
   option_->put(PREF_SPLIT, "5");
-  option_->put(PREF_TORRENT_FILE, "test.torrent");
+  option_->put(PREF_TORRENT_FILE, A2_TEST_DIR"/test.torrent");
   option_->put(PREF_DIR, "/tmp");
   option_->put(PREF_OUT, "file.out");
   option_->put(PREF_BT_EXCLUDE_TRACKER, "http://tracker1");
@@ -380,7 +372,7 @@ void DownloadHelperTest::testCreateRequestGroupForBitTorrent()
 void DownloadHelperTest::testCreateRequestGroupForMetalink()
 {
   option_->put(PREF_SPLIT, "3");
-  option_->put(PREF_METALINK_FILE, "test.xml");
+  option_->put(PREF_METALINK_FILE, A2_TEST_DIR"/test.xml");
   option_->put(PREF_METALINK_SERVERS, "5");
   option_->put(PREF_DIR, "/tmp");
   option_->put(PREF_OUT, "file.out");
