@@ -46,15 +46,15 @@ CPPUNIT_TEST_SUITE_REGISTRATION( MultiDiskAdaptorTest );
 
 std::vector<SharedHandle<FileEntry> > createEntries() {
   SharedHandle<FileEntry> array[] = {
-    SharedHandle<FileEntry>(new FileEntry("file0.txt", 0, 0)),
-    SharedHandle<FileEntry>(new FileEntry("file1.txt", 15, 0)),
-    SharedHandle<FileEntry>(new FileEntry("file2.txt", 7, 15)),
-    SharedHandle<FileEntry>(new FileEntry("file3.txt", 0, 22)),
-    SharedHandle<FileEntry>(new FileEntry("file4.txt", 2, 22)),
-    SharedHandle<FileEntry>(new FileEntry("file5.txt", 0, 24)),
-    SharedHandle<FileEntry>(new FileEntry("file6.txt", 3, 24)),
-    SharedHandle<FileEntry>(new FileEntry("file7.txt", 0, 27)),
-    SharedHandle<FileEntry>(new FileEntry("file8.txt", 2, 27)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file0.txt", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file1.txt", 15, 0)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file2.txt", 7, 15)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file3.txt", 0, 22)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file4.txt", 2, 22)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file5.txt", 0, 24)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file6.txt", 3, 24)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file7.txt", 0, 27)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file8.txt", 2, 27)),
   };
   //           1    1    2    2    3
   // 0....5....0....5....0....5....0
@@ -86,12 +86,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -105,12 +105,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     // Because entries[1] spans entries[0]
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -124,14 +124,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[0]->getDiskWriter());
     // Because entries[2] spans entries[1]
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
     CPPUNIT_ASSERT(entries[1]->needsFileAllocation());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -144,14 +144,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
     // Because entries[4] spans entries[3]
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
     CPPUNIT_ASSERT(entries[3]->needsFileAllocation());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -164,13 +164,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
     // entries[3] is 0 length. No overrap with entries[4]
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -184,12 +184,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -204,12 +204,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -222,12 +222,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -240,14 +240,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
     // entries[1] spans entries[2]
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
     CPPUNIT_ASSERT(!entries[2]->needsFileAllocation());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -261,17 +261,17 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
     // entries[6] spans entries[5] in the current implementation.
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[6]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[7]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[6]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[7]->getDiskWriter());
     // entries[6] spans entries[8]
-    CPPUNIT_ASSERT(!entries[8]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[8]->getDiskWriter());
     adaptor->closeFile();
   }
 }
@@ -297,9 +297,9 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg.c_str(), msg.size(), 0);
   adaptor->closeFile();
 
-  CPPUNIT_ASSERT(File("file0.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file0.txt").isFile());
   char buf[128];
-  readFile("file1.txt", buf, 5);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 5);
   buf[5] = '\0';
   CPPUNIT_ASSERT_EQUAL(msg, std::string(buf));
 
@@ -308,10 +308,10 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg2.c_str(), msg2.size(), 5);
   adaptor->closeFile();
 
-  readFile("file1.txt", buf, 15);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 15);
   buf[15] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("1234567890ABCDE"), std::string(buf));
-  readFile("file2.txt", buf, 1);
+  readFile(A2_TEST_OUT_DIR"/file2.txt", buf, 1);
   buf[1] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("F"), std::string(buf));
 
@@ -320,33 +320,33 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg3.c_str(), msg3.size(), 10);
   adaptor->closeFile();
 
-  readFile("file1.txt", buf, 15);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 15);
   buf[15] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("123456789012345"), std::string(buf));
-  readFile("file2.txt", buf, 7);
+  readFile(A2_TEST_OUT_DIR"/file2.txt", buf, 7);
   buf[7] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("1234567"), std::string(buf));
 
-  CPPUNIT_ASSERT(File("file3.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file3.txt").isFile());
 
-  readFile("file4.txt", buf, 2);
+  readFile(A2_TEST_OUT_DIR"/file4.txt", buf, 2);
   buf[2] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("12"), std::string(buf));
 
-  CPPUNIT_ASSERT(File("file5.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file5.txt").isFile());
 }
 
 void MultiDiskAdaptorTest::testReadData() {
-  SharedHandle<FileEntry> entry1(new FileEntry("file1r.txt", 15, 0));
-  SharedHandle<FileEntry> entry2(new FileEntry("file2r.txt", 7, 15));
-  SharedHandle<FileEntry> entry3(new FileEntry("file3r.txt", 3, 22));
+  SharedHandle<FileEntry> entry1(new FileEntry(A2_TEST_DIR"/file1r.txt", 15, 0));
+  SharedHandle<FileEntry> entry2(new FileEntry(A2_TEST_DIR"/file2r.txt", 7, 15));
+  SharedHandle<FileEntry> entry3(new FileEntry(A2_TEST_DIR"/file3r.txt", 3, 22));
   std::vector<SharedHandle<FileEntry> > entries;
   entries.push_back(entry1);
   entries.push_back(entry2);
   entries.push_back(entry3);
 
   adaptor->setFileEntries(entries.begin(), entries.end());
-
+  adaptor->enableReadOnly();
   adaptor->openFile();
   unsigned char buf[128];
   adaptor->readData(buf, 15, 0);
@@ -365,7 +365,7 @@ void MultiDiskAdaptorTest::testReadData() {
 
 void MultiDiskAdaptorTest::testCutTrailingGarbage()
 {
-  std::string dir = "/tmp";
+  std::string dir = A2_TEST_OUT_DIR;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testCutTrailingGarbage_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 256, 0)),
@@ -394,7 +394,7 @@ void MultiDiskAdaptorTest::testCutTrailingGarbage()
 
 void MultiDiskAdaptorTest::testSize()
 {
-  std::string dir = "/tmp";
+  std::string dir = A2_TEST_OUT_DIR;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testSize_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 1, 0)),
@@ -418,7 +418,7 @@ void MultiDiskAdaptorTest::testSize()
 
 void MultiDiskAdaptorTest::testUtime()
 {
-  std::string storeDir = "/tmp/aria2_MultiDiskAdaptorTest_testUtime";
+  std::string storeDir = A2_TEST_OUT_DIR"/aria2_MultiDiskAdaptorTest_testUtime";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/requested", 0, 0)),
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/notFound", 0, 0)),
